#!/opt/VRTSsfmh/bin/perl
#$Id: Setup.pm, 2016/03/31 09:13:33, unknown $
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ($^O !~ /Win32/i);}

package Setup;
require Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw(setup_cs setup_mh unsetup_cs unsetup_mh);

use strict;
use VRTS::Paths;
use File::Spec;
use VRTS::Util;
use VRTS::Logger;
use Getopt::Long;
use VRTS::AddOnInfo;
use File::Path;
use File::Basename;
use File::Copy;
use Data::Dumper;
use JSON qw /decode_json to_json/;

use File::Basename;
require FileHandle;
use JSON;
use Fcntl qw(:DEFAULT :flock);

use File::Basename;
require FileHandle;
use JSON;
use Fcntl qw(:DEFAULT :flock);



use constant SNAS_FAMILY        => 'SNAS';


# TBD -- Do we expose the exact error code to user?
# Do we need more granular error codes?
# Following perl idiom fn () || die " " 
use constant {
    SNAS_ERROR         => 0,
    SNAS_OK            => 1,
};
my $version = "1.0.0.0";

my $app_dir = VRTS::Paths::get_path ("AppDir");
my $addon_dir = $ENV{Store};
my $installdir = VRTS::Paths::get_path ("InstallDir");
my $cs_install_dir = $installdir;
$cs_install_dir   =~ s/VRTSsfmh/VRTSsfmcs/;

my $install_web_dir = $cs_install_dir.'/webgui';  


my $perl = File::Spec->catfile ($installdir, "bin", "perl");

my $log_dir = File::Spec->catdir($app_dir,"logs");
my $log = new VRTS::Logger(Module => 'snas',
                           Logdir => $log_dir,
                           Level => 'debug',
                           Logfile => 'snas.log',
                           UmiString => '');

VRTS::Logger::log("debug", "APPDIR: $app_dir");
VRTS::Logger::log("debug", "ADDON_DIR: $addon_dir");
VRTS::Logger::log("debug", "INSTALLDIR: $installdir");
VRTS::Logger::log("debug", "LOGDIR: $log_dir");

my $ret;
my $msg;
my %opts;
my $si_installed;
my $si_ver;
chdir $addon_dir;

sub cat_file
{
    my $path = shift;
    local $/ = undef;
    my $FH = undef;
    unless(open($FH, "<$path")) { 
        VRTS::Logger::log("error", "Failed to read [$path] $!");
        return SNAS_ERROR;
    }

    my $data = <$FH>;
    close($FH);
    return $data;
}

sub clean_up
{
    # unregister_snas_family();
    # remove_files_from_cs();
    # remove_files_from_mh();
    return SNAS_OK;
}

sub unsetup_cs
{
    unregister_snas_family();
    remove_files_from_cs();
    #remove ui related files from cs. 
    remove_ui_from_cs();
    return SNAS_OK;
}

sub unsetup_mh
{
    #remove_files_from_mh();
    return SNAS_OK;
}

#
# Add SNAS add-on as a family into VOM.
#
sub register_snas_family
{
    my $l_mods = File::Spec->catdir($addon_dir, "be", "cs", "defs");
    my $sql = File::Spec->catfile($l_mods, "snas.sql");
    if (! -f $sql)
    {
        VRTS::Logger::log('error', "Failed to register family." .
                    "Failed to find sql file");
        return SNAS_ERROR;
    }

    VRTS::Logger::log('debug', $sql);
    my $rc = VRTS::Util::exec_sql_file($sql);
    if ($rc != 0) {
        VRTS::Logger::log('error', "Failed to register family. " .
                    "SQL execution error $rc.");
        clean_up();                                    
        return SNAS_ERROR;
    }

    VRTS::Logger::log('info', "Successfully added " . SNAS_FAMILY .
                                " as an enclosure type.");
    return SNAS_OK;
}

#
# Add SNAS add-on as a family into VOM.
#
sub unregister_snas_family
{
    VRTS::Logger::log('info', "	unregister_snas_family");
    my $l_mods = File::Spec->catdir($addon_dir, "be", "cs", "defs");
    my $sql = File::Spec->catfile($l_mods, "snas_unset.sql");
    if (! -f $sql) 
    {
        VRTS::Logger::log('error', "Failed to register family." .
                    "Failed to find sql file");
        return SNAS_ERROR;
    }

    VRTS::Logger::log('debug', $sql);
    my $rc = VRTS::Util::exec_sql_file($sql);
    if ($rc != 0) 
    {
        VRTS::Logger::log('error', "Failed to unregister family. " .
                    "SQL execution error $rc.");
        clean_up();                                    
        return SNAS_ERROR;
    }

    VRTS::Logger::log('info', "Successfully removed " . SNAS_FAMILY .
                                "as an enclosure type.");
    return SNAS_OK;
}

sub deploy_ui_to_cs
{
    # Deploy ui web

    my $uiweb_src = File::Spec->catdir($addon_dir, "vom_snas.tar");
    my $uiweb_target = File::Spec->catdir($install_web_dir, "vom");
    my $cmd = "tar -vxf $uiweb_src -C $uiweb_target";
    system($cmd);

    if($? != 0) {
            VRTS::Logger::log('error', "Failed to deploy SNAS addon vom_snas.tar " .
                                        "on CS. $!");
            return SNAS_ERROR;
    };

    ## Deploy ui lib
    my $uilib_src = File::Spec->catdir($addon_dir, "vom_snas.jar");
    my $uilib_target = File::Spec->catdir($install_web_dir, "vom", "WEB-INF", "lib");
    unless(copy($uilib_src, $uilib_target)) {
            VRTS::Logger::log('error', "Failed to deploy SNAS addon lib " .
                                        "on CS. $!");
            return SNAS_ERROR;
    }

    return SNAS_OK;
}

sub deploy_files_to_cs
{
    
    VRTS::Logger::log('error', "Calling deploy_files_to_cs");

    my $mods_src = File::Spec->catdir($addon_dir, "be", "cs", "lib");
    for my $file (glob("$mods_src/*.pm")) {
        $file = basename($file, ".pm");
        my $mod_target = File::Spec->catdir($installdir,"lib", "modules",
                                        "VRTS", "Enclosure", $file);

        printf("COPYING $mods_src/$file.pm to $mod_target\n/");
        unless(copy("$mods_src/$file.pm", "$mod_target/" . SNAS_FAMILY . ".pm")) {
            VRTS::Logger::log('error', "Failed to deploy SNAS family " .
                                        "on CS. $!");
            clean_up();                                    
            return SNAS_ERROR;
        }
    }


    # TODO: Temporary Handling - To keep this around till VOM adopts this fix into TOT
    my @srcFiles = ("array_discovery_sched.pm", "refresh_array_config.pm", "refresh_device_manager.pm");
    foreach (@srcFiles) { 
	    my $opr_src_file = File::Spec->catfile($addon_dir, "be", "cs", "operations", "jobs", $_);
	    my $opr_dest_dir = File::Spec->catdir($installdir, "operations", "jobs");
	    unless(copy($opr_src_file, $opr_dest_dir)) 
	    {
	        VRTS::Logger::log('error', "Failed to deploy VOM CMS jobs on CS. $!");
	        clean_up();
	        return SNAS_ERROR;
	    }
	}

    my $isa_sec_mgmt_src_file = File::Spec->catfile($addon_dir, "be", "mh", "bin", "isa_sec_mgmt.pl");
    my $isa_sec_mgmt_dest_dir = File::Spec->catdir($installdir, "bin");
    unless(copy($isa_sec_mgmt_src_file, $isa_sec_mgmt_dest_dir))
    {
        VRTS::Logger::log('error', "Failed to deploy VOM CMS isa_sec_mgmt.pl on CS. $!");
        clean_up();
        return SNAS_ERROR;
    }
    # Allow perl files to be a executable 
    my $cmd = '/bin/chmod +x ' .  $isa_sec_mgmt_dest_dir .'/isa_sec_mgmt.pl';
    VRTS::Logger::log('error', "cmd $cmd");
    `$cmd`;
    
    my $isa_snapshot_mgmt_src_file = File::Spec->catfile($addon_dir, "be", "mh", "bin", "isa_snapshot_mgmt.pl");
    my $isa_snapshot_mgmt_dest_dir = File::Spec->catdir($installdir, "bin");
    unless(copy($isa_snapshot_mgmt_src_file, $isa_snapshot_mgmt_dest_dir))
    {
        VRTS::Logger::log('error', "Failed to deploy VOM CMS isa_snapshot_mgmt.pl on CS. $!");
        clean_up();
        return SNAS_ERROR;
    }
    
     # Allow perl files to be a executable 
    my $cmd = '/bin/chmod +x ' .  $isa_snapshot_mgmt_dest_dir .'/isa_snapshot_mgmt.pl';
    VRTS::Logger::log('error', "cmd $cmd");
    `$cmd`;
    
 
    my $vf_user_src_file = File::Spec->catfile($addon_dir, "be", "mh", "bin", "verify_user.pl");
    my $vf_user_mgmt_dest_dir = File::Spec->catdir($installdir, "bin");
    unless(copy($vf_user_src_file, $vf_user_mgmt_dest_dir))
    {
        VRTS::Logger::log('error', "Failed to deploy VOM CMS verify_user.pl on CS. $!");
        clean_up();
        return SNAS_ERROR;
    }

    # Allow perl files to be a executable 
    $cmd = '/bin/chmod +x ' . $vf_user_mgmt_dest_dir .'/verify_user.pl';
    `$cmd`;
    VRTS::Logger::log('error', "cmd $cmd");

    my $halssh_src_file = File::Spec->catfile($addon_dir, "be", "cs", "bin", "halssh");
    my $halssh_dest_dir = File::Spec->catdir($installdir, "bin");
    unless(copy($halssh_src_file, $halssh_dest_dir))
    {
        VRTS::Logger::log('error', "Failed to deploy VOM CMS halssh binary on CS. $!");
        clean_up();
        return SNAS_ERROR;
    }

    my $crypto_src_lib = File::Spec->catfile($addon_dir, "be", "cs", "lib", "libvxcrypto.so.6");
    my $crypto_dest_lib = File::Spec->catdir($installdir, "lib");
    unless(copy($crypto_src_lib, $crypto_dest_lib))
    {
        VRTS::Logger::log('error', "Failed to deploy VOM CMS halssh vxcrypto lib on CS. $!");
        clean_up();
        return SNAS_ERROR;
    }

    # Deploy Constants
    my $const_src = File::Spec->catdir($addon_dir, "be", "cs", "Constants");
    my $const_target = File::Spec->catdir($installdir,"agentlet", "VRTS", "Constants");

    # TBD -- What is the rollback plan here.. Could corrupt the Constants
    # dir and render the CS unusable. Specially with the constans for
    # updated (add columns) tables.
    unless(copy_recursively($const_src, $const_target)) {
            VRTS::Logger::log('error', "Failed to deploy SNAS schema constants " .
                                        "on CS. $!");
            return SNAS_ERROR;
    }

    # Deploy TDF files
    my $tdf_src = File::Spec->catdir($addon_dir, "be", "cs", "defs", "tdf");
    my $tdf_target = File::Spec->catdir($installdir, "etc", "tdf", SNAS_FAMILY);

    # TBD -- What is the rollback plan here.. Could corrupt the Constants
    # dir and render the CS unusable. Specially with the constans for
    # updated (add columns) tables.
    unless(copy_recursively($tdf_src, $tdf_target)) {
            VRTS::Logger::log('error', "Failed to deploy SNAS tdf constants " .
                                        "on CS. $!");
            return SNAS_ERROR;
    }

    merge_model_file();

    # Deploy utils
    my $util_src = File::Spec->catdir($addon_dir, "be", "cs", "utils");
    my $util_target = File::Spec->catdir($installdir, "addons", SNAS_FAMILY, "utils");
    unless(copy_recursively($util_src, $util_target)) {
            VRTS::Logger::log('error', "Failed to deploy SNAS addon utils " .
                                        "on CS. $!");
            return SNAS_ERROR;
    }

    $util_src = File::Spec->catdir($addon_dir, "be", "mh", "operations");
    $util_target = File::Spec->catdir($installdir, "addons", SNAS_FAMILY, "utils");
    unless(copy_recursively($util_src, $util_target)) {
            VRTS::Logger::log('error', "Failed to deploy SNAS specific operation " .
                                        "on CS. $!");
            return SNAS_ERROR;
    }

	$util_src = File::Spec->catdir($addon_dir, "be", "mh", "etc");
    $util_target = File::Spec->catdir($installdir, "etc");
    unless(copy_recursively($util_src, $util_target)) {
            VRTS::Logger::log('error', "Failed to deploy SNAS specific operation " .
                                        "on CS. $!");
            return SNAS_ERROR;
    }

    return SNAS_OK;
}

sub remove_files_from_cs
{
    my $l_mods = File::Spec->catdir($addon_dir, "be", "cs", "lib");
    for my $file (glob("$l_mods/*.pm")) {
        $file = basename($file, ".pm");
        my $mod_target = File::Spec->catdir($installdir,"lib", "modules",
                                        "VRTS", "Enclosure", $file);

        printf("Removing file.." .  "$mod_target/" . SNAS_FAMILY . ".pm");
        unless(unlink("$mod_target/" . SNAS_FAMILY . ".pm")) {
            VRTS::Logger::log('error', "Failed to remove SNAS family " .
                                        "on CS. $!");
            return SNAS_ERROR;
        }
    }

    # TBD -- Finish this.. move to rpm for a cleaner approach.

    return SNAS_OK;
}

sub remove_ui_from_cs
{
    #Remove jar and web part.

     my $uilib_target = File::Spec->catdir($install_web_dir, "vom", "WEB-INF", "lib");    
     my $remove_target = $uilib_target . "/vom_snas.jar";
     my $cmd = "rm -f $remove_target";     
     system($cmd);
 
     print "Finished remove $remove_target.\n";
     my $uijsp_target = File::Spec->catdir($install_web_dir, "vom", "jsp");
     my $remove_target = $uijsp_target . "/snas";
     my $cmd = "rm -rf $remove_target";   
     system($cmd);
     print "Finished remove all jsp file under $remove_target.\n";
     
     #Remove spring and js files
     my $uijs_target = File::Spec->catdir($install_web_dir, "vom", "js", "snas");    
     my $remove_target = $uijs_target;
     my $cmd = "rm -rf $remove_target";     
     system($cmd);
     print "Finished remove all js files under $remove_target.\n";
 
     my $uispg_target = File::Spec->catdir($install_web_dir, "vom", "WEB-INF", "spring","addon");    
     my $remove_target = $uispg_target . "/snas-*";
     my $cmd = "rm -rf $remove_target";     
     system($cmd);
     print "Finished remove all spring files: $remove_target.\n";
 
     return SNAS_OK;
}

sub setup_cs
{
    my $rc = 0;
    # UI:
    # Note: please move jar files into src/ui/stage/lib like src/ui/stage/lib/vom_snas.jar
    # and tar files into src/ui/stage/web before doing make file like src/ui/stage/web/vom_snas.tar.
    # my $file = File::Spec->catdir($addon_dir, "ui", "stage", "web", "vom_snas.tar");
    my $tar_file = File::Spec->catdir($addon_dir, "vom_snas.tar");
    my $jar_file = File::Spec->catdir($addon_dir, "vom_snas.jar");
    
    if(-e $tar_file && -e $jar_file){
    	$rc = deploy_ui_to_cs() || return $rc;
    }

    $rc = deploy_files_to_cs() || return $rc;
    $rc = register_snas_family() || return $rc;

    return SNAS_OK;
}


sub deploy_files_to_mh
{
    my ($console_flag) = @_;
    my $mod_src = File::Spec->catdir($addon_dir, "be", "mh", "agentlet", "VRTS");
    my $mod_target = File::Spec->catdir($installdir, "agentlet", "VRTS", SNAS_FAMILY);

    # Create SNAS family in agentlet directory
    mkpath($mod_target, { verbose => 1, });
    unless(-d $mod_target) {
            VRTS::Logger::log('error', "Failed to deploy SNAS family " .
                                        "on MH. $!");
            return SNAS_ERROR;
    }

    # Delpoy SNAS agentlet modules
    unless(copy_recursively($mod_src, $mod_target)) {
            VRTS::Logger::log('error', "Failed to deploy SNAS family " .
                                        "on MH. $!");
            return SNAS_ERROR;
    } else {
            VRTS::Logger::log('info', "Copied the SNAS addon agentlet scripts to MH");
    }

    # Deploy Constants
    my $const_src = File::Spec->catdir($addon_dir, "be", "cs", "Constants");
    my $const_target = File::Spec->catdir($installdir, "agentlet", "VRTS", "Constants");
    unless(copy_recursively($const_src, $const_target)) {
            VRTS::Logger::log('error', "Failed to deploy SNAS schema constants " .
                                        "on MH. $!");
            return SNAS_ERROR;
    } else {
            VRTS::Logger::log('info', "Copied the schema constants to MH");
    }

    # Deploy Constants
    my $tdf_src = File::Spec->catdir($addon_dir, "be", "cs", "defs", "tdf");
    my $tdf_target = File::Spec->catdir($installdir, "etc", "tdf", SNAS_FAMILY);

    # TBD -- What is the rollback plan here.. Could corrupt the Constants
    # dir and render the CS unusable. Specially with the constans for
    # updated (add columns) tables.
    unless(copy_recursively($tdf_src, $tdf_target)) {
            VRTS::Logger::log('error', "Failed to deploy SNAS tdf constants " .
                                        "on CS. $!");
            return SNAS_ERROR;
    }
	
    my $etc_src = File::Spec->catdir($addon_dir, "be", "mh", "etc", "SNASops.tmpl");
    my $etc_target = '/opt/VRTSsfmh/etc/SNASops.tmpl';
    VRTS::Logger::log('error'," src $etc_src tgt $etc_target");
    copy($etc_src, $etc_target);
    
    _cp_tdfs();
    my $snasAlFl = File::Spec->catfile($addon_dir, "be", "mh", "defs", "snas.alert.definition");
    merge_alert_definition( '/opt/VRTSsfmh/etc/alert.definition', $snasAlFl,0);

    #deploy isa_sec_mgmt.pl
    my $isa_sec_mgmt_src_file = File::Spec->catfile($addon_dir, "be", "mh", "bin", "isa_sec_mgmt.pl");
    my $isa_sec_mgmt_dest_dir = File::Spec->catdir($installdir, "bin");
    unless(copy($isa_sec_mgmt_src_file, $isa_sec_mgmt_dest_dir))
    {
        VRTS::Logger::log('error', "Failed to deploy  isa_sec_mgmt.pl on MH. $!");
        clean_up();
        return SNAS_ERROR;
    }
    # Allow perl files to be a executable 
    my $cmd = '/bin/chmod +x ' .  $isa_sec_mgmt_dest_dir .'/isa_sec_mgmt.pl';
    `$cmd`;
    VRTS::Logger::log('error', "cmd $cmd");
    
    my $isa_snapshot_mgmt_src_file = File::Spec->catfile($addon_dir, "be", "mh", "bin", "isa_snapshot_mgmt.pl");
    my $isa_snapshot_mgmt_dest_dir = File::Spec->catdir($installdir, "bin");
    unless(copy($isa_snapshot_mgmt_src_file, $isa_snapshot_mgmt_dest_dir))
    {
        VRTS::Logger::log('error', "Failed to deploy VOM CMS isa_snapshot_mgmt.pl on CS. $!");
        clean_up();
        return SNAS_ERROR;
    }
    
     # Allow perl files to be a executable 
    my $cmd = '/bin/chmod +x ' .  $isa_snapshot_mgmt_dest_dir .'/isa_snapshot_mgmt.pl';
    VRTS::Logger::log('error', "cmd $cmd");
    `$cmd`;

    my $vf_user_src_file = File::Spec->catfile($addon_dir, "be", "mh", "bin", "verify_user.pl");
    my $vf_user_mgmt_dest_dir = File::Spec->catdir($installdir, "bin");
    unless(copy($vf_user_src_file, $vf_user_mgmt_dest_dir))
    {
        VRTS::Logger::log('error', "Failed to deploy verify_user_mgmt.pl on MH. $!");
        clean_up();
        return SNAS_ERROR;
    }
    # Allow perl files to be a executable 
    $cmd = '/bin/chmod +x ' . $vf_user_mgmt_dest_dir .'/verify_user.pl';
    `$cmd`;
    VRTS::Logger::log('error', "cmd $cmd");

    if ($console_flag == 1) {
        print("This is console node. Setting up cronjob\n");
        setup_cron();
    } else {
        print("This is NOT console node. Setting up cronjob and putting it into paused state\n");
        setup_cron();
        pause_cron();
    }

    return 0;
}

sub setup_cron
{
    my $xprtlc = $installdir . "/bin/xprtlc";

    # Get the CS name
    my $cmd = "$xprtlc -l https://localhost:5634/world/getvitals";
    my $out  = `$cmd`;
    my $decoded = decode_json($out);
    unless ($decoded) {
        print("Failed to decode getvitals json output\n");
        return undef;
    }
    my $hsh = $decoded->{"DOMAINS"};
    my @cs = keys(%$hsh);

    # Add the cron job
    $cmd = "$installdir/bin/mh_ctl.pl --family HOST --schedule 1440 ".
        " --register --domain $cs[0] --id discovery_S10-HOST --class " .
        "discover.family.mh.HOST";
    print($cmd);
    print `$cmd`;

    # Add the cron job
    $cmd = "$installdir/bin/mh_ctl.pl --family " . SNAS_FAMILY . " --schedule 30 ".
        " --register --domain $cs[0] --id discovery_S80-". SNAS_FAMILY . " --class " .
        "discover.family.mh." . SNAS_FAMILY;
    print($cmd);
    print `$cmd`;
}

sub pause_cron
{
    my $xprtlc = $installdir . "/bin/xprtlc";

    # Get the CS name
    my $cmd = "$xprtlc -l https://localhost:5634/world/getvitals";
    my $out  = `$cmd`;
    my $decoded = decode_json($out);
    unless ($decoded) {
        print("Failed to decode getvitals json output\n");
        return undef;
    }
    my $hsh = $decoded->{"DOMAINS"};
    my @cs = keys(%$hsh);

    # Add the cron job
    $cmd = "$installdir/bin/mh_ctl.pl --family " . SNAS_FAMILY . " --pause ";
    print($cmd);
    print `$cmd`;
}

sub remove_files_from_mh
{

    my $mod_target = File::Spec->catdir($installdir, "agentlet", "VRTS", SNAS_FAMILY);
    rmtree($mod_target);
    my $tdf_target = File::Spec->catdir($installdir, "etc", "tdf", SNAS_FAMILY);
    rmtree($tdf_target);
}

sub copy_recursively
{
    my($src_dir, $dest_dir) = @_;
    mkpath($dest_dir) unless (-d $dest_dir);
    return undef unless (-d $src_dir and -d $dest_dir);
    return system("cp -r $src_dir/* $dest_dir") == 0;
}

sub decode_json_file
{
    my $path = shift;
    unless (-e $path) {
        VRTS::Logger::log('error', "No file $path exists.");
        return undef;
    }

    local $/;
    my $mfh = undef;

    if (open($mfh, "<", $path) == 0) {
            VRTS::Logger::log('error', "Failed to open $path. $!");
            return undef;
    }

    my $model_hsh = decode_json(<$mfh>);
    close($mfh);

    unless($model_hsh) {
            VRTS::Logger::log('error', "Failed to decode json file $path");
            return undef;
    }

    return $model_hsh;
}

sub _cp_tdfs
{
    copy('/opt/VRTSsfmh/etc/tdf/HOST/Fault_Obj.tdf', '/opt/VRTSsfmh/etc/tdf/SNAS/Fault_Obj.tdf');
    copy('/opt/VRTSsfmh/etc/tdf/HOST/Host_Fault_Obj.tdf', '/opt/VRTSsfmh/etc/tdf/SNAS/Host_Fault_Obj.tdf');
}

sub read_file_json
{
    my $file_name = shift;

    open FILE, "<$file_name";
    my $buf = do { local $/; <FILE> };
    my $jsonobject = new JSON ();

    my $json_content = undef;
    eval
    {
        $json_content = $jsonobject->jsonToObj($buf);
        1;
    } or do
    {
        $json_content = undef;
    };

    close (FILE);

    return $json_content;
}

sub dump_hash_to_file
{
    my $file_name = shift;
    my $perl_hash = shift;


    my $fd = new FileHandle $file_name, O_WRONLY|O_CREAT;
    my $final_buf = JSON::objToJson($perl_hash, {pretty => 1, indent => 1});

    print $fd "$final_buf\n";

    close($fd);

    return 0;
}

sub merge_alert_definition
{
    my $source_f = shift;
    my $mini_f = shift;
    my $dump = shift;

    my $source_json = read_file_json($source_f);

    if (not defined $source_json)
    {
        return 1;
    }

    my $mini_json = read_file_json($mini_f);

    if (not defined $mini_json)
    {
        return 1;
    }

    # Merge mini to source file
    while (my($key,$value) = each(%$mini_json))
    {
       $source_json->{"declarations"}->{$key} = $value;
    }

    if ($dump == 1)
    {
        print Dumper($source_json);
    }

    dump_hash_to_file($source_f, $source_json);

    return 0;
}
sub merge_model_file{

    # Deploy Model file
    my  $model_file = File::Spec->catdir($installdir,"etc","tdf","sfm.model");
    open FILE, "<$model_file";
    my $buf = do { local $/; <FILE> };
    my $jsonobject = new JSON ();    
    my $org_hash = $jsonobject->jsonToObj($buf);
    close (FILE);

    my $arr_model_file = File::Spec->catfile($addon_dir, "be", "cs", "defs", "snas.model");
    open FILE, "<$arr_model_file";
    my $buf1 = do { local $/; <FILE> };
    my $arr_hash = $jsonobject->jsonToObj($buf1);
    close (FILE);

    while (my($key,$value) = each(%{$arr_hash}))
    {
        if (!exists($org_hash->{$key})) {
            $org_hash->{$key} = $value;
        } else {
            # Merge the two values which are hashes themselves
            my $orig_val = $org_hash->{$key};
            my %union_val = (%$orig_val, %$value);
            $org_hash->{$key} = \%union_val;
        }
    }


    my $fd = new FileHandle $model_file, O_WRONLY|O_CREAT;
    my $final_buf = JSON::objToJson($org_hash, {pretty => 1, indent => 1});
    print $fd "$final_buf\n";
    close($fd)

}
sub setup_mh
{
    my ($console_flag) = @_;
    my $rc = 0;
    $rc = deploy_files_to_mh($console_flag) || return $rc;
}
1;
