#!/usr/bin/perl

use JSON qw /decode_json to_json/;
use File::Path qw /mkpath/;
use Data::Dumper;
use strict;

use constant {
    GEN_TDF_OK => 0,
    GEN_TDF_ERR => 1
};

sub parse_snas_tdf
{
    my $file = "snas.tdf";
    my $model = "snas.model";
    my $tdfdir = "tdf";
    my %model_hsh = ();
    mkpath($tdfdir);
    unless(-d $tdfdir) {
        print("Failed to create tdf dir.\n");
        return undef;
    }

    open(my $fh, "<", $file) || die "Failed to open file $file: $!";
    local $/ = '}';
    while(my $tstr = <$fh>) {
        $tstr =~ s/^[^T]*//g;
        next if($tstr eq "");
        # print("----->  $tstr\n");


        # Write to model file
        my @lines = split(/\n/, $tstr);
        my $tablename = undef;
        my %attrs = ();
        foreach my $line (@lines) {
            if ($line =~ /^\s*TABLE\s*(\w+)/) {
                $tablename = $1;
            } elsif ($line =~ /^\s*INDEX/) {
            } elsif ($line =~ /^\s*PRIMARY/) {
            } elsif ($line =~ /^\s*[{}]/) {
            } elsif ($line =~ /^\s*$/) {
            } elsif ($line =~ /\s*(\w+)\s+(\w+);/) {
                # print "#######" . $1 . "\n";
                $attrs{$1} = "";
            }
        }

        if ($tablename and scalar(keys(%attrs))) {
            # print($tablename . "\n");
            $model_hsh{$tablename} = \%attrs;
        } else {
            print("Invalid format. Couldn't parse the tdf file.\n");
            exit(1);
        }


        # Write the TDF file
        my $tdf_file = "$tdfdir/$tablename.tdf";
        open(my $tfh, ">", $tdf_file) || die "Failed to open $tdf_file: $!";
        print $tfh "$tstr;";
        close($tfh);
    }
    close($fh);

    # Model file
    open(my $mfh, ">", $model) || die "Failed to open file $model: $!";
    print $mfh to_json(\%model_hsh, {pretty => 1});
    close($mfh);
}

parse_snas_tdf();
