#$Id: Discover.pm, 2016/04/07 23:40:19, venkat $
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $

package VRTS::Enclosure::Discover::SNAS;
use strict;
use Carp;
require Exporter;
require XML::Simple;
require VRTS::Enclosure::VDID;
require VRTS::Enclosure::Utils;
require VRTS::Paths;
require VRTS::Util;
require VRTS::Logger;
require VRTS::SFMHCAT;

use File::Spec;
use Data::Dumper;
use JSON;
use File::Temp qw/ :POSIX /;

my ($xml, $vdid_obj);

my %capabilities = ("VOL_CAPABLE" => 0, "SHOW_RG_LDEV" => 1, "SHOW_RG_PDEV" => 1, "LOGICAL_NAS_USED" => 1, "NAS_CAPABLE" => 1,
   "SHOW_PDEV" => 1, "SHOW_LDEV" => 1, "CONSIDER_SEGMENT_LUN" => 1,
   "SHOW_REPLICATION" => 1, "SHOW_TP_SOURCELDEV" => 0, "SHOW_TP_THINLDEV" => 1, "SHOW_LDEV_PDEV" => 1, "LDEV_PDEV_MAP" => 1,
   "PRIMARY_REMOTE_REP_CAPABLE" => 1, "SHOW_VSET" => 0, "SHOW_TP" => 1, "SHOW_RG" => 1, "SECONDARY_REMOTE_REP_CAPABLE" => 1,
   "SHOW_PDEV_FOR_THIN_LDEV" => 1, "SHOW_PDEV_FOR_TP_SOURCE_THINLDEV" => 0, "SHOW_DEVICE_GROUP_REPLICATION" => 0);

use constant SNAS_FAMILY => 'SNAS';

my $mh_install_dir = VRTS::Paths::get_path("InstallDir");
my $log_dir = VRTS::Paths::get_path("LogDir");
my $appdirmh = VRTS::Paths::get_path ("AppDir");
my $xprtlc = File::Spec->catfile($mh_install_dir, "bin", "xprtlc");
my $perlcmd = File::Spec->catfile($mh_install_dir, "bin", "perl");
my $cms_info = VRTS::Util::get_cms_config_info();
my $cur_cms_name = $cms_info->{'cs_name'};
my $is_cs = $cms_info->{'cs'};
my $gendeploy_path = $mh_install_dir . "/bin/gendeploy.pl";

sub new 
{
    my $class = shift;
    my $self = {};

    bless ($self, $class);
    return $self;
}

sub initialize
{
    my $top = shift;
    my $module = new VRTS::Enclosure::Discover::SNAS;
    return $module;

}

sub configure_enclosure
{
    my $command = shift;
    my $params = shift;
    my $config_id = shift;
    my $encl_data = shift;
    

    VRTS::Logger::log('info', "In configure enclosure for [$command]");

    if ($command eq 'set')
    {

        my %encl_params = (
           'control-host', $params->{'CONTROLER_HOST'},
           'server', $params->{'DEVICE_MANAGER'},
           'port', $params->{'PORT'},
           'user', $params->{'USERNAME'}, 
           'pass',  $params->{'PASSWD'},
           'config_id', $config_id, 
           'ID', $params->{'ID'}	
        );

        run_gendeploy(\%encl_params);
    }
    elsif ($command eq 'unset')
    {
    	# Place holder for future code changes in unset. No required for now.
    	 
        # /opt/VRTSsfmh/bin/vomadm host-mgmt --remove 192.168.10.76 {Use the control host parameter from array config}
        if ($params->{DEVICE_MANAGER} ne "")
        {
            my $vomadm = File::Spec->catfile(VRTS::Paths::get_path("InstallDir"), "bin", "vomadm");
            VRTS::Logger::log('error', "Params:".Dumper($params));
            VRTS::Logger::log('error', "encData:".Dumper($encl_data));

            my @nodeIds= get_array_node_ids($encl_data->{VENDOR_ID});

            foreach(@nodeIds){
               my $hname = get_snas_hostname_byid($_);
               my $vomadm_rm_cmd = VRTS::Util::make_command($vomadm, "host-mgmt", "--remove", "--host", $hname);
               $vomadm_rm_cmd = '/opt/VRTSsfmh/bin/perl '. $vomadm_rm_cmd;
               VRTS::Logger::log('error', "In configure_enclosure for unset ", $vomadm_rm_cmd);
               $ENV{"MODE"}=1;
               VRTS::Util::run_exec($vomadm_rm_cmd);
            }
        }
    }
}
sub get_snas_hostname_byid{
    my $hostid=shift;
    my $vomadm = File::Spec->catfile(VRTS::Paths::get_path("InstallDir"), "bin", "vomadm");
    my $vomadm_rm_cmd = VRTS::Util::make_command($vomadm, "host-mgmt", "--list");
    my @lines = grep (/$hostid/, qx/$vomadm_rm_cmd  2>&1/);
    if (scalar @lines != 1){
        VRTS::Logger::log('error', "Unable to file hostname for id $hostid"); 
        return undef ;
    }
    my @data  = split(/\s/,$lines[0] );
    return $data[0] ;
}
sub check_time_skew {

    my ($server, $user, $pass) =@_;
    my ($rc, $rTime) = ssh_execcmd_out($server, $user, $pass, 'date +%s 2>&1'); 
    my $lTime = `date +%s`;

    VRTS::Logger::log('debug', "CS time $lTime MH time $rTime " );

    if ($rTime =~ m/Access denied/){
        VRTS::Logger::log('error',"Authentication failure.");
        return &VRTS::Enclosure::Discover::AUTHENTICATION_FAILED;
    }	    
    if ($rc>0) {
        return &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }
    my $diff = abs($lTime - $rTime);
    if($diff > 5400)
    {
       VRTS::Logger::log('error',"Excessive time difference between VOM server and SNAS Clsuter");	    
       return &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }
    return 0;
}
sub get_enclosure_listing
{
    my $self = shift;
    my $params = shift;
    VRTS::Logger::log('info', "In get_enclosure_listing");

    my $ch = $params->{"control-host"};
    my $server = $params->{"server"};
    my $port = $params->{"port"};
    my $user = $params->{"user"};
    my $pass = $params->{"pass"};
    my $id = $params->{ID};
    my $cmd = "";
    my $rc;	

    # Remove the mh key from the sshhostkeys file before trying to do halssh for the first time.
    if (-f "/root/.putty/sshhostkeys")
    {
        $cmd = "/bin/ex +g/" . $server. "/d -cwq /root/.putty/sshhostkeys";
        system($cmd);
    }

    ($rc, $pass) = VRTS::Util::decrypt_encoded_passwd($pass, $id);
    if ($rc) 
    {
        VRTS::Logger::log('error',"decrypt_encoded_passwd failed with error code = [$rc]");
        return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }
    my $rc = check_time_skew($server,$user,$pass);
    if ($rc != 0){
        VRTS::Logger::log('error',"Excessive time difference between VOM server and SNAS Clsuter");
        return undef, $rc;
    }	    
    #$pass = quotemeta($params->{'pass'});

    my $probe_timeout = $self->{"probe_timeout"}*2;

    # Due to change to halssh to work in a non-interactive mode, SNAS discovery will only work
    #  with managed host version 5.0 and above, we need to check this as part of probe and provide the correct error code.
    my $xprtlc = File::Spec->catfile(VRTS::Paths::get_path("InstallDir"), "bin", "xprtlc");

    my $my_grep = "grep";
    $my_grep = "findstr" if ($^O =~ /Win32/i);
    my $op = undef;
    my $xprtlc_cmd = VRTS::Util::make_command($xprtlc, "-l", "https://$ch/world/getvitals");
    my $xprtlout = `$xprtlc_cmd`;
    chomp $xprtlout;

    my $jsonobject = new JSON ();
    eval {
      $op = $jsonobject->jsonToObj($xprtlout);
    };
    if ($@)
    {
        return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    if (not defined $op->{XPRTLD_VERSION})
    {
        return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    if ($op->{XPRTLD_VERSION} =~ /^[5-9]\./)
    {
        VRTS::Logger::log('debug', "Managed host version is 5.0 or above, supported version**********");
    }
    else
    {
        VRTS::Logger::log('error', "Managed host version is less than 5.x, not supported");
        return undef, &VRTS::Enclosure::Discover::PLATFORM_UNSUPPORTED;
    }

    # First check if we support the firmware version which should be greater than 2.2.3
    # This is required as the cli options and printing for this version and must be for versions below is very different.

    my $args = " -ssh -l $user -pw $pass -4 $server date";
    my $vardir= VRTS::Paths::get_path("AppDir");
    my $script = "$vardir/SNAS.pl";

    _create_probe_script($args, $script);

    VRTS::Logger::log('debug', "hello - $vardir\SNAS.pl");
    VRTS::Logger::log('debug', "hello - [$script]");
    VRTS::Logger::log('debug', "hello - [$ch]");
    VRTS::Logger::log('debug', "hello - $vardir\/SNAS.pl");
    my $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "$vardir\/SNAS.pl", $probe_timeout);
    VRTS::Logger::log('debug', "hello - [$out->[1]]");
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            print "Request timed out due to no activity for timeout period\n";
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         print "Error in push_exec\n";
         return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    unlink $script;

    my $blob = $out->[1];
    my $not_supported_fmware = 0;

    my @arr_list = split /\n/, $blob;
    foreach my $line (@arr_list)
    {
        if ($line =~ /Release version/)
        {
            my $version = $line;
            VRTS::Logger::log('debug', "Firmware version detected as [$line]");
            $line =~ s/Release version\s+([0-9])\.([0-9])\.([0-9])//;
            if ($1 > 2)
            {
                $not_supported_fmware = 0;
            }
            elsif ($1 == 2 and $2 > 2)
            {
                $not_supported_fmware = 0;
            } 
            elsif ($1 == 2 and $2 <= 2 and $3 > 3)
            {
                $not_supported_fmware = 0;
            }
            else
            {
                $not_supported_fmware = 1;
            }

            last;
        }
    }

    if ($blob =~ /Connection timed out/)
    {
        return undef, &VRTS::Enclosure::Discover::ARRAY_NOT_REACHABLE;
    }
    elsif ($blob =~ /not found/ or $blob =~ /No such file/)
    {
        # SYMCLI path not found.
        return undef, &VRTS::Enclosure::Discover::VENDOR_COMP_VERSION_INCOMPAT;
    }
    elsif ($blob =~ /invalid credentials/ or $blob =~ /Authentication failur/)
    {
        return undef, &VRTS::Enclosure::Discover::AUTHENTICATION_FAILED;
    }
    elsif ($blob =~ /Too many CLI connections/ or $blob =~ /No route to host/ or $blob =~ /Host does not exist/)
    {
        return undef, &VRTS::Enclosure::Discover::ARRAY_NOT_REACHABLE;
    }
    elsif ($not_supported_fmware == 1)
    {
        return undef, &VRTS::Enclosure::Discover::ARRAY_DISCOVERY_FM_LOWER_VERSION_ERROR;
    }

    VRTS::Logger::log('debug', 
        "Firmware version on the array side is higher than the minimum requirement, proceeding with probe");

    $vdid_obj = new VRTS::Enclosure::VDID ();
    $vdid_obj->initialize("SNAS", 0);

    $args = " -ssh -l $user -pw $pass -4 $server /opt/SYMCsnas/pysnas/bin/controller.py cluster discover";
    $vardir= VRTS::Paths::get_path("AppDir");
    $script = "$vardir/SNAS.pl";
    VRTS::Logger::log('debug',$script);
    _create_probe_script($args, $script);

    my $out = VRTS::Enclosure::Utils::push_exec($ch, $script, "$vardir\/SNAS.pl", $probe_timeout);
    if ($out->[0] != 0)
    {
        my $err = $out->[0] >> 8;
        if ($err == 192)
        {
            print "Request timed out due to no activity for timeout period\n";
            return undef, &VRTS::Enclosure::Discover::SESSION_TIMEDOUT;
        }
         print "Error in push_exec\n";
         return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }

    unlink $script;

    my $blob = $out->[1];
    VRTS::Logger::log('debug', $blob);
    if ($blob =~ /not found/ or $blob =~ /No such file/)
    {
        # SYMCLI path not found.
        return undef, &VRTS::Enclosure::Discover::VENDOR_COMP_VERSION_INCOMPAT;
    }

    my $enc_hash = get_enclosure_details($blob);

    unlink $script;
    return $enc_hash;
}

sub refresh_device_manager
{
    my $self = shift;
    my $params = shift;

    my $out;
    my $ch = $params->{"control-host"};
    my $ch1 = $params->{"secondary-ch"};
    my $install_path = $params->{"install-path"};
    my $array_id = $params->{"array-id"};

    return 0;
}

sub get_enclosure_details
{
    my $data = shift;
    my $hsh = decode_json($data);

    my %val_hash;
    $val_hash{VDID} = $hsh->{vdid};
    $val_hash{serial} = $hsh->{id};
    $val_hash{model} = $hsh->{product};
    $val_hash{displayname} = $hsh->{name};
    $val_hash{product} = $hsh->{product};
    $val_hash{vendor} = $hsh->{vendor};

    my $op_hash;
    $op_hash->{$hsh->{vdid}} = \%val_hash;
    return $op_hash;
}

sub _create_probe_script
{
    my $args = shift;
    my $script = shift;

    open(S, ">$script");
    binmode S;

    print S "\#\!\/opt\/VRTSsfmh\/bin\/perl\n\n";

    print S "BEGIN { \@INC = (\"/opt/VRTSsfmh/lib/modules\") if ( \$\^O !~ /Win32/i ) }\n\n";

    print S "require VRTS\:\:Paths\;\n";
    print S "require File\:\:Spec\;\n\n";

    print S "my \$cmd = VRTS\:\:Paths\:\:get_path(\"MHBin\")\;\n";
    print S "\$cmd = File\:\:Spec\-\>catfile(\$cmd, \"halssh\")\;\n";
    print S "if ( \$\^O !~ /Win32/i )\n {\n";
    print S "\$ENV{'LD_LIBRARY_PATH'} = \"/opt/VRTSsfmh/lib\";\n";
    print S "print \"not found\" if (\! -x \"\$cmd\")\;\n";
    print S "system(\"\$cmd $args\")\;\n}\n";
    print S "else\n {\n";
    print S "\$cmd = \$cmd\.\"\.exe\"\;\n";
    print S "print \"not found\" if (\! -x \"\$cmd\")\;\n";
    print S "system(\" \\\"\$cmd\\\" $args\")\;\n}\n";
    close(S);
    VRTS::Logger::log('debug', "Done writing probe script [$script]");
}

sub provide_capabilities
{
    my $self = shift;
    VRTS::Logger::log('debug', "SNAS is being queried for capabilities");
    return \%capabilities;
}

sub run_gendeploy
{
    my $params = shift;
    my $ch = $params->{"control-host"};
    my $server = $params->{"server"};
    my $port = $params->{"port"};
    my $user = $params->{"user"};
    my $pass = $params->{"pass"};
    my $id = $params->{ID};
    my $config_id = $params->{"config_id"};
    my $rc;
    ($rc, $pass) = VRTS::Util::decrypt_encoded_passwd($pass, $id);
    if ($rc)
    {
        VRTS::Logger::log('error',"decrypt_encoded_passwd failed with error code = [$rc]");
        return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }
    VRTS::Logger::log('debug', "Running gendeploy as child process\n");
    my $cmd = "/opt/VRTSsfmh/addons/SNAS/utils/setuptrust.pl $server $user $pass ".$config_id." & ";
    my $op = `$cmd`;
    VRTS::Logger::log('debug', "Done Running gendeploy as child process \n");
    return 0;
}
sub ssh_execcmd
{
    my ($mh, $user, $pass, $cmd) = @_;

    my $args = " -ssh -l $user -pw $pass -4 $mh";
    $ENV{'LD_LIBRARY_PATH'} = "/opt/VRTSsfmh/lib";

    my $halssh = VRTS::Paths::get_path("MHBin") . "halssh $args \"$cmd\"";
    #VRTS::Logger::log('error', "Command:$halssh");
    my $ret =  system($halssh);
    VRTS::Logger::log('debug', "halssh Return code: $ret\n");
    return $ret;
}
sub ssh_execcmd_out
{
    my ($mh, $user, $pass, $cmd) = @_;

    my $args = " -ssh -l $user -pw $pass -4 $mh";
    $ENV{'LD_LIBRARY_PATH'} = "/opt/VRTSsfmh/lib";

    my $halssh = VRTS::Paths::get_path("MHBin") . "halssh $args \"$cmd\"";
    #VRTS::Logger::log('error', "Command:$halssh");
    #
    my $rc=0;
    my @out =  qx/$halssh 2>&1 /;
    $rc = $?;
    $rc = $rc >> 8 unless ($rc == -1);
    VRTS::Logger::log('debug', "halssh Return code: $rc\nout @out \n");
    return ($rc, join('',@out));
}
sub refresh_enclosure
{
    my ($config_id, $taskid, $prntTid) =@_; 
    VRTS::Logger::log('error',"refresh_enclosure::$config_id");
	
    my $nodeInfo = get_array_info($config_id);
    return if (!defined $nodeInfo);
     
    my $node = $nodeInfo->{DEVICE_MANAGER};
    my $pass = $nodeInfo->{PASSWD};
    my $id   = $nodeInfo->{ID};
    my $user = $nodeInfo->{USERNAME};
    my $rc;
    
    ($rc, $pass) = VRTS::Util::decrypt_encoded_passwd($pass, $id);
    if ($rc)
    {
        VRTS::Logger::log('error',"decrypt_encoded_passwd failed with error code = [$rc]");
        return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }
    VRTS::Logger::log('error',"refresh_enclosure::$config_id  id $id ");
    ssh_execcmd($node, $user, $pass, 
    '/usr/bin/nohup /opt/VRTSsfmh/bin/perl /opt/VRTSsfmh/bin/mh_driver.pl --family SNAS --hidden --wait --id discovery_S80-SNAS --context optimize=off:snas_enc_refresh=yes:update_cache=1:configid='.$id.' > /dev/null 2>&1  &' );
    
 }
my $drivervardir = VRTS::Paths::get_path("AppDir");
my $dis_method;
my $arrayinfo = undef;
sub get_array_info
{

    my $config_id=shift;
    my $rc = 0;
    my $cmd;
    my $sql;
    my $js = new JSON;
    my $timestamp = time;

    my $dblog_file = "/var/opt/VRTSsfmcs/xdbadm_error_$$.$timestamp.log";
    if ($^O =~ /Win32/i)
    {
        $drivervardir =~ s/VRTSsfmh/VRTSsfmcs/g;
        $dblog_file = $drivervardir."xdbadm_error_$$.$timestamp.log";
    }

    # Get the method to perform the configuration...
    $sql = "SELECT * FROM HABDBSYNC.ARRAY_VIEW_CONFIG_R WHERE ID ='$config_id'";
    VRTS::Logger::log('debug', "Generating sql stmt [$sql] to get method details");
    $cmd = generate_cmd($sql, $dblog_file);
    VRTS::Logger::log('debug', "Generated cmd = [$cmd]");
    my $out=`$cmd`;
    VRTS::Logger::log('debug', "Cmd return code = [$?]");
    my $output = Dumper($out);
    VRTS::Logger::log('debug', "Output = [$output]");
    my $arr_details = $js->jsonToObj($out);
    $rc = $?;

    if($? != 0)
    {
        VRTS::Logger::log('error', "Failed to execute sql: $out");
        open(my $FH, $dblog_file);
        if($FH) {
            local $/;
            VRTS::Logger::log('error', <$FH>);
            close($FH);
            unlink($dblog_file);
        } else {
            VRTS::Logger::log('error', "Couldnot open [$dblog_file]");
        }
        return undef;
    }
    $arrayinfo = $arr_details->{Records}[0];;
    my $json_arrayinfo = Dumper($arrayinfo);
    VRTS::Logger::log('debug', "json_arrayinfo =  [$json_arrayinfo]");
    return $arrayinfo;
}

sub generate_cmd
{
    my $sql = shift;
    my $dblog_file =  shift;

    my $cmd = "/opt/VRTSsfmh/bin/xdbadm -x \"$sql\" -o /etc/vx/VRTSsfmcs/.odbc.ini -c /var/opt/VRTSsfmcs/conf -d SFMdb3 -v  2>$dblog_file";
    if ($^O =~ /Win32/i)
    {
        my $installDirMH = VRTS::Paths::get_path("InstallDir");

        $cmd = "\"".$installDirMH."bin\\xdbadm.exe"."\"".
               " -x ".
               "\"$sql\"".
               " -o ".
               "\"".$drivervardir.".odbc.ini"."\"".
               " -c ".
               "\"".$drivervardir."conf"."\"".
               " -d SFMdb3 -v  ".
               " 2>". "\""."$dblog_file"."\"";
    }
    VRTS::Logger::log('debug', "Executing $cmd ");

    return $cmd;
}

# 
# return all nodeIds associated with enclosure
#
sub get_array_node_ids
{

    my $enc_id=shift;
    my $rc = 0;
    my $cmd;
    my $sql;
    my $js = new JSON;
    my $timestamp = time;
    my @nodeIds=();

    my $dblog_file = "/var/opt/VRTSsfmcs/xdbadm_error_$$.$timestamp.log";
    if ($^O =~ /Win32/i)
    {
        $drivervardir =~ s/VRTSsfmh/VRTSsfmcs/g;
        $dblog_file = $drivervardir."xdbadm_error_$$.$timestamp.log";
    }

    # Get the method to perform the configuration...
    $sql = "select * from HABDBSYNC.array_view_subsystem_nodes_r WHERE SUBSYS_ID ='$enc_id'";
    VRTS::Logger::log('debug', "Generating sql stmt [$sql] to get method details");
    $cmd = generate_cmd($sql, $dblog_file);
    VRTS::Logger::log('debug', "Generated cmd = [$cmd]");
    my $out=`$cmd`;
    VRTS::Logger::log('debug', "Cmd return code = [$?]");
    my $output = Dumper($out);
    VRTS::Logger::log('debug', "Output = [$output]");
    my $arr_details = $js->jsonToObj($out);
    $rc = $?;

    if($? != 0)
    {
        VRTS::Logger::log('error', "Failed to execute sql: $out");
        open(my $FH, $dblog_file);
        if($FH) {
            local $/;
            VRTS::Logger::log('error', <$FH>);
            close($FH);
            unlink($dblog_file);
        } else {
            VRTS::Logger::log('error', "Couldnot open [$dblog_file]");
        }
        return undef;
    }

    my $json_arrayinfo = Dumper($arr_details);
    VRTS::Logger::log('debug', "json_arrayinfo =  [$json_arrayinfo]");
    foreach(@{ $arr_details->{Records} }){
            push @nodeIds, $_->{ID};
    }
    return @nodeIds;
}

1;
