#$Id: array_discovery_sched.pm, 2016/03/18 13:56:51, deepti.shenai $
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $

package jobs::array_discovery_sched;

require VRTS::Paths;
my $bdir = VRTS::Paths::get_path("InstallDir");
$bdir=$bdir."/agentlet/";
push @INC, $bdir;


use strict;
use threads;
use threads::shared;
require Exporter;
require VRTS::Enclosure::Utils;
require VRTS::Enclosure::Discover;
require VRTS::Task;
use File::Path qw (rmtree );
use vars qw($VERSION @ISA @EXPORT );

@ISA = qw(Exporter);

$VERSION = '1.00';

@EXPORT = qw(initialize execute checkenv get_operation_name);

# Fill this information in data structure of this Job
my $job=undef;
my $previousjob = undef;
my $configid = undef;
my $old_config_id = undef;
my $old_list = undef;
my $new_list = undef;
my $command = undef;
my $arrayinfo = undef;
my $old_vdid = undef;
my @remove_list = undef;
my $dis_method;
my $xprtlc_cmd = VRTS::Paths::get_path("xprtlc");
my $drivervardir = VRTS::Paths::get_path("AppDir");
my $bindir = VRTS::Paths::get_path("InstallDir");
my $perl = File::Spec->catfile ($bindir, "bin", "perl");
my $array_driver = File::Spec->catfile(VRTS::Paths::get_path("InstallDir"), "bin", "array_driver.pl");
my $mh_ctl_pl = File::Spec->catfile(VRTS::Paths::get_path("InstallDir"), "bin", "mh_ctl.pl");

if ($^O =~ /Win32/i) 
{
    $xprtlc_cmd = "\"".VRTS::Paths::get_path("xprtlc")."\"";
    $perl .= ".exe";
}
my $tmp_str = "";

sub initialize
{
    my $agent = shift;
    my $index = shift;

    # Get params passed to this operation
    my $params = $agent->get_param();

    # Get a handle to this Job created during driver initialization
    $job = $agent->get_job_by_name("array_discovery_sched");

    # Store interested values from PARAM or CLIPBOARD of previous job
    if(not defined $job)
    {
        return 1;
    }

    $configid = $params->{"config-id"};
    $command = $params->{"command"};
    if($command ne 'set' and $command ne 'unset' and $command ne 'edit' and $command ne 'upgrade')
    {
        VRTS::Logger::log('debug', "Invalid command");
        $agent->set_fatal(1);
        return 100;
    }
    $old_list = $params->{"remove-list"};
    $new_list = $params->{"add-list"};

    $old_vdid = $params->{"OLDVDID"} if (defined $params->{"OLDVDID"} and $params->{"OLDVDID"} ne "");
    $old_config_id = $params->{"old-config-id"} if (defined $params->{"old-config-id"} and $params->{"old-config-id"} ne "");

    if ($^O !~ /Win32/i)
    {
        $ENV{"PATH"} .= ":/opt/VRTSsfmcs/asa11/bin:/opt/VRTSsfmcs/asa11/res";
    }

    return 0;
}

sub get_operation_name
{
    my $agent = shift;
    my $params = shift;

    my $op_name = "Set/Unset Array Configuration";
    my $command = $params->{"command"};

    if ($command eq "set")
    {
        $op_name = "Configure Enclosure";
    }
    elsif ($command eq "unset")
    {
        $op_name = "Unconfigure Enclosure";
    } 
    elsif ($command eq "edit")
    {
        $op_name = "Edit Enclosure Configuration";
    }

    return $op_name;
}

sub 
get_array_info
{
    my $agent = shift;
    my $rc = 0;
    my $cmd;
    my $sql;
    my $js = new JSON;

    # Get the method to perform the configuration...
    $sql = "SELECT * FROM HABDBSYNC.ARRAY_VIEW_CONFIG_R WHERE ID ='$configid'";
    VRTS::Logger::log('debug', "Generating sql stmt [$sql] to get method details");
    $cmd = generate_cmd($sql);
    my $out=`$cmd`;
    $rc = $?;

    if($? != 0)
    {
        VRTS::Logger::log('debug', "Failed to execute sql: $out");
        $agent->set_fatal(1);
        return 100;
    }
    my $arr_details = $js->jsonToObj($out);
    $dis_method = $arr_details->{Records}[0]->{METHOD};
    $dis_method =~ tr/[a-z]/[A-Z]/;
    VRTS::Logger::log('debug', "Array discovery method is [$dis_method]");
    $job->set_clipboard("array_config", $arr_details->{Records}[0]);

    $sql = "SELECT * FROM HABDBSYNC.ARRAY_VIEW_CONFIG_ARRAY_MAPPING_R WHERE ID='$configid' AND STATE != 0";
    VRTS::Logger::log('debug', "Generating sql stmt");

    $cmd = generate_cmd($sql);
    my $out=`$cmd`;
    $rc = $?;

    if($? != 0)
    {
        VRTS::Logger::log('debug', "Failed to execute sql: $out");
        $agent->set_fatal(1);
        return 100;
    }

    $arrayinfo = $js->jsonToObj($out);
    $job->set_clipboard("arrayinfo", $arrayinfo);
    return $rc;
}

sub generate_cmd
{
    my $sql = shift;

    my $cmd = "/opt/VRTSsfmh/bin/xdbadm -x \"$sql\" -o /etc/vx/VRTSsfmcs/.odbc.ini -c /var/opt/VRTSsfmcs/conf -d SFMdb3 -v  2>/var/opt/VRTSsfmcs/xdbadm_error.log";
    if ($^O =~ /Win32/i)
    {
        my $installDirMH = VRTS::Paths::get_path("InstallDir");
        my $appDirCS     = VRTS::Paths::get_path("AppDir");
        $appDirCS        =~ s/VRTSsfmh/VRTSsfmcs/g;

        $cmd = "\"".$installDirMH."bin\\xdbadm.exe"."\"".
               " -x ".
               "\"$sql\"".
               " -o ".
               "\"".$appDirCS.".odbc.ini"."\"".
               " -c ".
               "\"".$appDirCS."conf"."\"".
               " -d SFMdb3 -v  ".
               " 2>". "\"".$appDirCS."xdbadm_error.log"."\"";
    }
    VRTS::Logger::log('debug', "Executing $cmd ");

    return $cmd;
}

sub set_sched
{
    my $agent = shift;
    my $rc = 0;
    my $cmd;
    my $arrays = $arrayinfo->{Records}; 
    my $numarrays = @$arrays;

    for(my $i=0;$i<$numarrays;$i++)
    {
        my $encid = $arrayinfo->{Records}[$i]->{VENDOR_ID};

        my $arr_config = $job->get_clipboard_data("array_config");
        my $method = undef;

        if ($arr_config ne "")
        { 
            $method = $arr_config->{METHOD};
            VRTS::Logger::log('debug', " Method = [$method]");
        }

        my $schedid = $encid;

        my $enc_info = new VRTS::Enclosure::Discover();
        $enc_info->initialize("$method");

        my $threxit:shared = 0;

        # Start a thread that will print '\n' every 4 mins to keep the
        # conenction alive for more than 5 mins if array probe doesn't
        # return in that time
        my $to_thr = threads->new( sub {
            lock($threxit); # This is a blocking call
            my $wait_time_sec = 240; # Max time to wait: 240 sec
            while ($threxit == 0) {
                # cond_timedwait takes a locked variable and abs time for
                # timeout. It comes out because of timeout or because it was
                # signalled. In either case, it will relock the variable before
                # returning
                if (!cond_timedwait($threxit, time() + $wait_time_sec)) {
                    # Timeout happened
                    print "\n";
                } else {
                    # We were signalled. The loop will exit if $threxit != 0
                }
            }
            });

        #get_enclosure_listing will go ahead with executing probe script
        #with given parameters
        if (defined $method && $method ne "")
        {
            my $module_name = "VRTS::Enclosure::Discover::$method";
            VRTS::Logger::log('debug', " Method = [$module_name]");
            if( defined &{\&{"${module_name}::configure_enclosure"}})
            {
                eval {
                    &{\&{"${module_name}::configure_enclosure"}}($command, $arr_config , $configid, $arrayinfo->{Records}[$i]);
                };

                my $ret_code = $?;
                if ($ret_code == 0)
                {
                    next;
                }
                else
                {
                    $rc = $ret_code;
                }
            
            }
        }


        my $mh_ctl;
        my $sch_mh_ctl;
        if ($^O =~ m/Win32/i)
        {
            my $perl = $bindir . "\\bin\\perl.exe";
            my $installDirMH = VRTS::Paths::get_path("InstallDir");
            $mh_ctl = "\"$perl\" \"$installDirMH\\bin\\array_driver.pl\" --family $dis_method --hidden --discover --report-task --context \"vardir=$encid\"";
            $sch_mh_ctl = "\\\"$perl\\\" \\\"$installDirMH\\bin\\array_driver.pl\\\" --family $dis_method --hidden --context vardir=$encid";
        }
        else
        {
            $mh_ctl = "/opt/VRTSsfmh/bin/array_driver.pl --family $dis_method --hidden --discover --report-task --context \"vardir=$encid\"";
            $sch_mh_ctl = "/opt/VRTSsfmh/bin/array_driver.pl --family $dis_method --hidden --context vardir=$encid";
        }

        if($command eq 'set')
        {
            $cmd = "\"".$perl."\""." "."\"".$mh_ctl_pl."\""." --register --domain sfm:// --family $dis_method --id $schedid --schedule 360 ".
                   " --command=\"$sch_mh_ctl\""." --class discover.family.array.$dis_method";

            VRTS::Logger::log('debug', "Executing $cmd ");
            my $out=`$cmd`;
            $rc = $?;

            if($? != 0)
            {
                VRTS::Logger::log('debug', "Failed to set schedule: $out");
                $agent->set_fatal(1);
                return 100;
            }

            # run discovery in background
            if ($^O =~ /Win32/i) 
            {
                require Win32::Process;
                #Run cmd in background
                my $child_proc;
                my $perl_install_dir = $bindir."bin\\perl.exe";
                Win32::Process::Create($child_proc,
                      $perl_install_dir,
                      $mh_ctl,
                      0,
                      &Win32::Process::DETACHED_PROCESS |
                      &Win32::Process::CREATE_NO_WINDOW |
                      &Win32::Process::NORMAL_PRIORITY_CLASS,
                      ".") || VRTS::Logger::log('error' , "Failed to   trigger discovery");
            }
            else
            {
                `$mh_ctl  1>/dev/null 2>&1 &`;
            }
        }
        else
        {
            push(@remove_list, $schedid);

            $cmd = "\"".$perl."\""." "."\"".$mh_ctl_pl."\""." --unregister --id $schedid ";

            VRTS::Logger::log('debug', "Executing $cmd ");
            my $out=`$cmd`;
            $rc = $?;

            if($? != 0)
            {
                VRTS::Logger::log('debug', "Failed to unset schedule: $out");
                $agent->set_fatal(1);
                return 100;
            }

            my $familyvardir = $dis_method . "." . $encid;
            VRTS::Logger::log('debug', "Removing old vardir $familyvardir");
            File::Path::rmtree(File::Spec->catdir($drivervardir, $familyvardir)); 

            VRTS::Logger::log('debug', "Removing vardir $encid");
            File::Path::rmtree(File::Spec->catdir($drivervardir, $dis_method, $encid));
        }
    }

    if ($command eq "unset")
    {
        my $conf_details = VRTS::Enclosure::Utils::get_config_by_id($configid);
        my $task_obj = new VRTS::Task;
        my $taskinfo =
        {
            Name => "Unconfigure Enclosure",
            TargetType => "vrts_ddl_enclosure_config",
            Target => $conf_details->[0]->{NAME},
            TargetId => $configid,
            RptOnStChnge => 1
        };

        my $taskid = $task_obj->create(undef, $taskinfo);
        $task_obj->update_field($taskid, 'state', "FINISHED");
    }
    
    # Now the new enclosure discovery is setup, we need to purge the old array.
    if (defined $old_vdid and $old_vdid ne "")
    {
        purge_old_enclosure($configid, $old_vdid, $command, $old_config_id);
    }
    return $rc;
}

sub edit_sched
{
    my $agent = shift;
    my $rc = 0;
    $old_list =~ s/[\[\]]+//g;
    my @old = split /,/, $old_list;
    foreach my $o (@old)
    {
        $o =~ s/^\s*//;
        $o =~ s/\s*$//;
        my $schedid = $o;
        push(@remove_list, $schedid);
        my $cmd = $xprtlc_cmd." -l https://localhost:5634/admin/cgi-bin/schedule.pl/del/$schedid";
        VRTS::Logger::log('debug', "Executing $cmd ");
        my $out=`$cmd`;
        $rc = $?;

        if($? != 0)
        {
            VRTS::Logger::log('debug', "Failed to remove schedule: $out");
            $agent->set_fatal(1);
            return 100;
        }
    }
    $new_list =~ s/[\[\]]+//g;
    my @newlist = split /,/, $new_list;
    foreach my $n (@newlist)
    {
        # Trim spaces before and after to ensure that a correct schedule id is constructed...
        $n =~ s/^\s+//;
        $n =~ s/\s+$//;
 
        my $schedid = $n;
        my $mh_ctl;
        my $sch_mh_ctl;
        if ($^O =~ m/Win32/i)
        {
            my $installDirMH = VRTS::Paths::get_path("InstallDir");
            $mh_ctl = "\"$perl\" \"$installDirMH\\bin\\array_driver.pl\" --family $dis_method --hidden --discover --context \"vardir=$n\\\"";
            $sch_mh_ctl = "\\\"$perl\\\" \\\"$installDirMH\\bin\\array_driver.pl\\\" --family $dis_method --hidden --context vardir=$n";
        }
        else
        {
            $mh_ctl = "/opt/VRTSsfmh/bin/array_driver.pl --family $dis_method --hidden --discover --context \"vardir=$n\"";
            $sch_mh_ctl = "/opt/VRTSsfmh/bin/array_driver.pl --family $dis_method --hidden --context vardir=$n";
        }

       # run discovery in background
       if ($^O =~ /Win32/i) 
       {
           require Win32::Process;
           #Run cmd in background
           my $child_proc;
 	       my $perl_install_dir = $bindir."bin\\perl.exe";
           Win32::Process::Create($child_proc,
                  $perl_install_dir,
                  $mh_ctl,
                  0,
                  &Win32::Process::DETACHED_PROCESS |
                  &Win32::Process::CREATE_NO_WINDOW |
                  &Win32::Process::NORMAL_PRIORITY_CLASS,
                  ".") || VRTS::Logger::log('error' , "Failed to trigger discovery");
       }
       else
       {
        `$mh_ctl  1>/dev/null 2>&1 &`;
       }
       my $cmd = $xprtlc_cmd." -l https://localhost:5634/admin/cgi-bin/schedule.pl/set/$schedid -d sched=\"360\" -d command=\"$sch_mh_ctl\"";
        VRTS::Logger::log('debug', "Executing $cmd ");
        my $out=`$cmd`;
        $rc = $?;

        if($? != 0)
        {
            VRTS::Logger::log('debug', "Failed to create schedule: $out");
            $agent->set_fatal(1);
            return 100;
        }
    }

    # Now we need to take care of removing the old enclosure if it was already configured
    purge_old_enclosure($configid, $old_vdid, $old_config_id, $command);

    return 0 if (scalar(@newlist) != 0 or scalar(@old) != 0);

    # If there is a case where there is no new/old list specified, it implies that is just a config change,
    #  even in such cases we need to force trigger off the discovery for all enclosures with the configuration ID.
    my $array_info = VRTS::Enclosure::Utils::_get_enclosures_by_configuration($configid);
    return 0 if (not defined $array_info or scalar(@{$array_info}) == 0);

    my $perl_cmd = undef;
    if ($^O =~ /Win32/i)
    {
        $perl_cmd = File::Spec->catfile(VRTS::Paths::get_path("InstallDir"),
                                    "bin", "perl.exe");
    }
    else
    {
        $perl_cmd = File::Spec->catfile(VRTS::Paths::get_path("InstallDir"),
                                    "bin", "perl");
    }
    $array_driver = File::Spec->catfile(VRTS::Paths::get_path("InstallDir"), "bin", "array_driver.pl");

    foreach my $array_c (@{$array_info})
    {
        my $arrayid = $array_c->{VENDOR_ID};
        my $method = $array_c->{METHOD};

        # Convert the method into upper case for family name use.
        $method =~ tr/a-z/A-Z/;

        my $command = VRTS::Util::make_command($perl_cmd, $array_driver,
                                                "--family", "$method", "--hidden", "--context",
                                                "optimize=off:update_cache=1:vardir=$arrayid");

        # Now report against the config id that the current state is running/discovering.
        VRTS::Enclosure::Discover::set_discovery_state($configid, $arrayid,
                                 &VRTS::Enclosure::Discover::DISCOVERY_IN_PROGRESS);

        VRTS::Logger::log('debug', "Issuing discovery in background = [$command]");
        VRTS::Util::execute_background($command, $perl_cmd);
    }

    # Get config details to extract the config name.
    my $conf_details = VRTS::Enclosure::Utils::get_config_by_id($configid);

    my $task_obj = new VRTS::Task;
    my $taskinfo =
    {
        Name => "Edit Enclosure Configuration",
        TargetType => "vrts_ddl_enclosure_config",
        Target => $conf_details->[0]->{NAME},
        TargetId => $configid,
        RptOnStChnge => 1
    };

    my $taskid = $task_obj->create(undef, $taskinfo);
    $task_obj->update_field($taskid, 'state', "FINISHED");
}

sub purge_old_enclosure
{
    my $config_id = shift;
    my $vdid = shift;
    my $old_config_id = shift;
    my $type = shift;

    if (not defined $vdid or $vdid eq "")
    {
        return 0;
    }

    VRTS::Logger::log('debug', "Array is previously configured with different identifier, need to purge"); 

    my $schedid = $config_id . "_" . $vdid;
    my $cmd = $xprtlc_cmd." -l https://localhost:5634/admin/cgi-bin/schedule.pl/del/$schedid";

    VRTS::Logger::log('debug', "Executing $cmd ");

    my $out=`$cmd`;
    my $rc = $?;

    if($? != 0)
    {
        VRTS::Logger::log('debug', "Failed to purge old enclosure $vdid : $out");
    }

    # Now delete the records from array config map table as well
    my $config_sql_file = File::Spec->catdir(VRTS::Paths::get_path("AppDir"), "unconfigure.sql");
    my $config_sql = "DELETE FROM HABDBSYNC.P_ARRAY_CONFIG_ARRAY_MAPPING where ID = '$config_id' and VENDOR_ID = '$vdid';";

    open HAN, "> $config_sql_file";
    print HAN $config_sql;
    close(HAN);

    VRTS::Enclosure::Utils->exec_sql_file($config_sql_file);

    unlink($config_sql_file);

    if ($type eq "set" and $old_config_id ne "")
    {
        $config_sql = "DELETE FROM HABDBSYNC.P_ARRAY_CONFIG where ID = '$old_config_id';";

        open HAN, "> $config_sql_file";
        print HAN $config_sql;
        close(HAN);
 
        VRTS::Enclosure::Util->exec_sql_file($config_sql);

        unlink($config_sql_file);
    }
}

sub execute
{
    my $agent = shift;
    my $rc = 0;

    $rc = get_array_info($agent);
    goto out if($rc != 0);

    if($command eq 'edit')
    {
        $rc = edit_sched($agent);
        goto out;
    }
    elsif($command eq 'upgrade')
    {
        $rc = 0;
        goto out;
    }
    $rc = set_sched($agent);
    if($rc != 0)
    {
        goto out;
    }

out:
    $job->{RETURNCODE} = $rc;
    if(defined @remove_list)
    {
        $job->set_clipboard("removelist", \@remove_list);
    }

    return $rc;
}

sub checkenv
{
    my $agent = shift;
    if(not defined $configid)
    {
        $agent->set_fatal(1);
        $job->{RETURNCODE} = 100;
        return 100;
    }
    return 0;
}

sub rollback
{
    my $agent = shift;
    return 0;
}

    
1;
