package jobs::refresh_array_config;

require VRTS::Paths;
my $bdir = VRTS::Paths::get_path("InstallDir");
$bdir=$bdir."/agentlet/";
push @INC, $bdir;

require VRTS::OS;
require VRTS::CMD;
require VRTS::Task;
require VRTS::Enclosure::Discover;

use File::Spec;
use Sys::Hostname;

use Data::Dumper;
use JSON;
use strict;
require Exporter;
use vars qw($VERSION @ISA @EXPORT );
use VRTS::CGILOCALE;

use Data::Dumper;

@ISA = qw(Exporter);

$VERSION = '1.00';

@EXPORT = qw(initialize execute checkenv);

# Fill this information in data structure of this Job
my $job=undef;
my $previousjob = undef;
my $rc;

my $in_params;

sub initialize
{
    my $agent = shift;
    my $index = shift;

    # Get params passed to this operation
    $in_params = $agent->get_param();

    # Get a handle to this Job created during driver initialization
    $job = $agent->get_job_by_name("refresh_array_config");
    return 1 if(not defined $job);

    return 0;
}

sub execute
{
    my $agent = shift;
    my $finalrc=0;

    my $configid = $in_params->{"config-id"};

    return 1 if (not defined $configid or $configid eq "");

    VRTS::Logger::log('debug', "Calling refresh configuration for config ID [$configid]");

    my $perl_cmd = undef;
    if ($^O =~ /Win32/i)
    {
        $perl_cmd = File::Spec->catfile(VRTS::Paths::get_path("InstallDir"), 
                                    "bin", "perl.exe");
    }
    else
    {
        $perl_cmd = File::Spec->catfile(VRTS::Paths::get_path("InstallDir"), 
                                    "bin", "perl");
    }

    my $array_driver = File::Spec->catfile(VRTS::Paths::get_path("InstallDir"), "bin", "array_driver.pl");

    # Now get a list of enclosures that are setup for a given configuration ID.
    # for every enclosure under the configuration call refresh array (force fire a delta discovery).

    my $array_info = VRTS::Enclosure::Utils::_get_enclosures_by_configuration($configid);

    if (not defined $array_info or scalar(@{$array_info}) == 0)
    {
        VRTS::Logger::log('error', "Did not get any enclosure for the specified configuration [$configid]");
        return 1;
    }

    my $parent_tinfo =
    {
        Name => "Refresh Enclosure Configuration",
        TargetId => $configid,
        Target => $configid,
        TargetType => "vrts_ddl_enclosure_config",
        IsParent => 1
    };

    my $chld_tsk_list = undef;
    foreach  my $array_c (@{$array_info})
    {
    	
    	my $output = Dumper($array_c);
    	VRTS::Logger::log('debug', "Array Info = [$output]");
    	
    	my $encl_name = VRTS::Enclosure::Utils::get_encl_name_using_vendorid($array_c->{VENDOR_ID});
        if (not defined $encl_name or $encl_name eq "")
        {
            $encl_name = $array_c->{VENDOR_ID};
        }

        my $chld_info =
        {
            Name => "Refresh Enclosure",
            TargetId => $array_c->{VENDOR_ID},
            Target => $encl_name,
            TargetType => "vrts_ddl_enclosure"
        };
 
        push @$chld_tsk_list, $chld_info;
    }

    my $taskid =undef; 
    $parent_tinfo->{ChldTsks} = $chld_tsk_list;
    my $task_obj = new VRTS::Task;
    my $task_counter = 0;
    $taskid = $task_obj->create(undef, $parent_tinfo);
    if (!defined $taskid) {
	VRTS::Logger::log ('error', "Failed to create parent child task mapping");
    }

    foreach my $array_c (@{$array_info})
    {
        my $arrayid = $array_c->{VENDOR_ID};
        my $method = $array_c->{METHOD};

        # Convert the method into upper case for family name use.
        $method =~ tr/a-z/A-Z/;

		my $chld_tid = undef;
        my $chld_tasks = $task_obj->get_chld_tsks_by_field($taskid, "TargetId", $array_c->{VENDOR_ID});
        if (defined $chld_tasks)
        {
            my @arr = keys %$chld_tasks;
            $chld_tid = $arr[0];
        }

        my $command = undef;
        if ($chld_tid ne "") 
        {
            my $encl_name = VRTS::Enclosure::Utils::get_encl_name_using_vendorid($array_c->{VENDOR_ID});
            if (not defined $encl_name or $encl_name eq ""){
                $encl_name = $array_c->{VENDOR_ID};
            }

            my $chld_info =
            {
                taskid => $chld_tid,
                ParentTskId => $taskid,
                Name => "Refresh Enclosure",
                TargetId => $array_c->{VENDOR_ID},
                Target => $encl_name,
                TargetType => "vrts_ddl_enclosure",
                RptOnStChnge => 1
            };

            $task_obj->create(undef, $chld_info);
            if ($array_c->{METHOD} eq 'SNAS'){
		VRTS::Logger::log ('debug', "Refreshing SNAS array");
		$command  = $perl_cmd.' /opt/VRTSsfmh/addons/SNAS/utils/isa_refresh.pl '."  $chld_tid  $configid  $arrayid $taskid";
		$command .= " >/dev/null 2>&1 &";
        	system($command );
    		VRTS::Logger::log ('debug', "Launched SNAS refresh operation cmd $command $?.");
    		$agent->set_operation_result("refresh successfully submitted");
    		return; 
    	    }else {    
		$command = VRTS::Util::make_command($perl_cmd, $array_driver, "--wait",
                                                "--family", "$method", "--hidden", "--context", 
                                                "optimize=off:taskid=$chld_tid:update_cache=1:vardir=$arrayid:configid=$configid");
	    }                                                
        } 
        else
        {
            $command = VRTS::Util::make_command($perl_cmd, $array_driver, "--wait",
                                                "--family", "$method", "--hidden", "--context", 
                                                "optimize=off:update_cache=1:vardir=$arrayid:config_id=$configid");
        } 

        # Now report against the config id that the current state is running/discovering.
        VRTS::Enclosure::Discover::set_discovery_state($configid, $arrayid, 
                                 &VRTS::Enclosure::Discover::DISCOVERY_IN_PROGRESS);

        VRTS::Logger::log('debug', "Calling the following command = [$command]");
        VRTS::Util::execute_background($command, $perl_cmd);
        $task_counter++;
    }

    $agent->set_operation_result("refresh successfully submitted");

    return 0;
}

sub checkenv
{
    my $agent = shift;
    return 0;
}

sub rollback
{
    my $agent = shift;
    return 0;
}

1;

