#$Id: refresh_device_manager.pm, 2016/03/02 02:18:59, venkat $
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
package jobs::refresh_device_manager;

require VRTS::Paths;
my $bdir = VRTS::Paths::get_path("InstallDir");
$bdir=$bdir."/agentlet/";
push @INC, $bdir;

require VRTS::OS;
require VRTS::CMD;
require VRTS::Task;
require VRTS::Enclosure::Discover;

use File::Spec;
use Sys::Hostname;

use JSON;
use strict;
require Exporter;
use vars qw($VERSION @ISA @EXPORT );
use VRTS::CGILOCALE;

use Data::Dumper;

@ISA = qw(Exporter);

$VERSION = '1.00';

@EXPORT = qw(initialize execute checkenv);

# Fill this information in data structure of this Job
my $job=undef;
my $previousjob = undef;
my $rc;

my $in_params;

sub initialize
{
    my $agent = shift;
    my $index = shift;

    # Get params passed to this operation
    $in_params = $agent->get_param();

    # Get a handle to this Job created during driver initialization
    $job = $agent->get_job_by_name("refresh_device_manager");
    return 1 if(not defined $job);

    return 0;
}

sub execute
{
    my $agent = shift;
    my $finalrc=0;

    my $method = $in_params->{"method"};
    my $arrayid = $in_params->{"array-id"};
    my $arrayname = $in_params->{"array-name"};
    my $arrayvdid = $in_params->{"array-vdid"};
    my $configid = $in_params->{"config-id"};

    $method =~ s/\s*//g;
    return 1 if (not defined $method or $method eq "");

    VRTS::Logger::log('debug', "Calling refresh using method [$method] for array = [$arrayid] with name as [$arrayname]");

    # Here is what will happen with refresh array operations.
    ### 1) setup a task object which will indicate which array is being refreshed.
    ### 2) report the task object as running before we issue the refresh for the array.
    ### 3) as array refresh take more than 5 mins, we will leave the array discovery in background and hence the task.
    ### 4) If there is already an instance running or one instance of the array discovery waiting to run, we will fail the run and not spawn another run.
    ### 5) fire the discovery in the background once, it is complete, force the task to be marked as completed.
    ###     a. the discovery will be fired with added context called optimize=off and also taskid=<identifier> to update as done.
    ###     b. If there is no delta, we will report an update for p_array_subsystem  and array_config_mapping to ensure that the time stamp is updated.

    # Create a task object and get the task id.
    my $task_obj = new VRTS::Task; 

    my $taskinfo =
    {
        Name => "Refresh Enclosure",
        TargetType => "vrts_ddl_enclosure",
        Target => $arrayname,
        TargetId => $arrayvdid
    };

    my $taskid = $task_obj->create(undef, $taskinfo);

    my $perl_cmd = undef;
    if ($^O =~ /Win32/i)
    {
        $perl_cmd = File::Spec->catfile(VRTS::Paths::get_path("InstallDir"), 
                                    "bin", "perl.exe");
    }
    else
    {
        $perl_cmd = File::Spec->catfile(VRTS::Paths::get_path("InstallDir"), 
                                    "bin", "perl");
    }
    my $array_driver = File::Spec->catfile(VRTS::Paths::get_path("InstallDir"), "bin", "array_driver.pl");
    my $command;
    if ($method eq 'SNAS'){
        VRTS::Logger::log ('debug', "Refreshing SNAS array");
	$command  = $perl_cmd.' /opt/VRTSsfmh/addons/SNAS/utils/isa_refresh.pl '."  $taskid  $configid  $arrayid $taskid";
	$command .= " >/dev/null 2>&1 &";
       	system($command );
	VRTS::Logger::log ('debug', "Launched SNAS refresh operation cmd $command $?.");
    	next; 
    }
    else{
        $command = VRTS::Util::make_command($perl_cmd, $array_driver, "--wait",
                                                "--family", "$method", "--hidden", "--context", 
                                                "optimize=off:taskid=$taskid:update_cache=1:vardir=$arrayid");
    }                                                

    # Now report against the config id that the current state is running/discovering.
    VRTS::Enclosure::Discover::set_discovery_state($configid, $arrayid, 
                                 &VRTS::Enclosure::Discover::DISCOVERY_IN_PROGRESS);

    VRTS::Logger::log('debug', "Calling the following command = [$command]");
    VRTS::Util::execute_background($command, $perl_cmd);

    $job->set_output("refresh successfully submitted");
    $agent->set_operation_result("refresh successfully submitted");

    return 0;
}

sub checkenv
{
    my $agent = shift;
    return 0;
}

sub rollback
{
    my $agent = shift;
    return 0;
}
1;
