#
#
#$Id: isa_refresh.pl, 2016/03/02 21:58:51, venkat $
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#
#

#!/opt/VRTSsfmh/bin/perl

BEGIN { unshift(@INC, "/opt/VRTSsfmh/lib/modules"); }

require Exporter;
require VRTS::Paths;
require VRTS::Util;
require VRTS::Agent;
require VRTS::Util;
require VRTS::EXITCODE;
require VRTS::Paths;
require VRTS::Logger;
require VRTS::OS;
require File::Path;
require VRTS::Config;
require VRTS::Task;

use Data::Dumper;
use JSON;



my $drivervardir = VRTS::Paths::get_path("AppDir");

sub generate_cmd
{
    my $sql = shift;
    my $dblog_file =  shift;

    my $cmd = "/opt/VRTSsfmh/bin/xdbadm -x \"$sql\" -o /etc/vx/VRTSsfmcs/.odbc.ini -c /var/opt/VRTSsfmcs/conf -d SFMdb3 -v  2>$dblog_file";
    if ($^O =~ /Win32/i)
    {
        my $installDirMH = VRTS::Paths::get_path("InstallDir");

        $cmd = "\"".$installDirMH."bin\\xdbadm.exe"."\"".
               " -x ".
               "\"$sql\"".
               " -o ".
               "\"".$drivervardir.".odbc.ini"."\"".
               " -c ".
               "\"".$drivervardir."conf"."\"".
               " -d SFMdb3 -v  ".
               " 2>". "\""."$dblog_file"."\"";
    }
    VRTS::Logger::log('debug', "Executing $cmd ");

    return $cmd;
}


sub get_array_info
{

    my $config_id=shift;
    my $rc = 0;
    my $cmd;
    my $sql;
    my $js = new JSON;
    my $timestamp = time;
    my $arrayinfo = undef;
    my $dblog_file = "/var/opt/VRTSsfmcs/xdbadm_error_$$.$timestamp.log";
    if ($^O =~ /Win32/i)
    {
        $drivervardir =~ s/VRTSsfmh/VRTSsfmcs/g;
        $dblog_file = $drivervardir."xdbadm_error_$$.$timestamp.log";
    }

    # Get the method to perform the configuration...
    $sql = "SELECT * FROM HABDBSYNC.ARRAY_VIEW_CONFIG_R WHERE ID ='$config_id'";
    VRTS::Logger::log('debug', "Generating sql stmt [$sql] to get method details");
    $cmd = generate_cmd($sql, $dblog_file);
    VRTS::Logger::log('debug', "Generated cmd = [$cmd]");
    my $out=`$cmd`;
    VRTS::Logger::log('debug', "Cmd return code = [$?]");
    my $output = Dumper($out);
    VRTS::Logger::log('debug', "Output = [$output]");
    my $arr_details = $js->jsonToObj($out);
    $rc = $?;

    if($? != 0)
    {
        VRTS::Logger::log('error', "Failed to execute sql: $out");
        open(my $FH, $dblog_file);
        if($FH) {
            local $/;
            VRTS::Logger::log('error', <$FH>);
            close($FH);
            unlink($dblog_file);
        } else {
            VRTS::Logger::log('error', "Couldnot open [$dblog_file]");
        }
        return undef;
    }
    $arrayinfo = $arr_details->{Records}[0];;
    my $json_arrayinfo = Dumper($arrayinfo);
    VRTS::Logger::log('debug', "json_arrayinfo =  [$json_arrayinfo]");
    return $arrayinfo;
}

sub ssh_execcmd
{
    my ($mh, $user, $pass, $cmd) = @_;

    my $args = " -ssh -l $user -pw $pass -4 $mh";
    $ENV{'LD_LIBRARY_PATH'} = "/opt/VRTSsfmh/lib";
    my $halssh = VRTS::Paths::get_path("MHBin") . "halssh $args \"$cmd\"";
    my $ret =  system($halssh);
    VRTS::Logger::log('debug', "halssh Return code: $ret\n");
    return $ret;
}


sub refresh_enclosure
{
    my ($config_id) =@_; 
    VRTS::Logger::log('error',"refresh_enclosure::$config_id");
	
    my $nodeInfo = get_array_info($config_id);
    return if (!defined $nodeInfo);
     
    my $node = $nodeInfo->{DEVICE_MANAGER};
    my $pass = $nodeInfo->{PASSWD};
    my $id   = $nodeInfo->{ID};
    my $user = $nodeInfo->{USERNAME};
    my $rc;
    
    ($rc, $pass) = VRTS::Util::decrypt_encoded_passwd($pass, $id);
    if ($rc)
    {
        VRTS::Logger::log('error',"decrypt_encoded_passwd failed with error code = [$rc]");
        return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }
    VRTS::Logger::log('error',"refresh_enclosure::$config_id  id $id ");
    my $ret = ssh_execcmd($node, $user, $pass, 
                          'nohup /opt/VRTSsfmh/bin/perl /opt/VRTSsfmh/bin/mh_driver.pl --family SNAS --hidden --wait --id discovery_S80-SNAS --context optimize=off:snas_enc_refresh=yes:update_cache=1:configid='.$id.' > /dev/null 2>&1  ' );
    return $ret;


}

sub main
{
    my ($childTid, $config_id, $arrayid) = @ARGV;
    VRTS::Logger::log('debug',"isa_refresh Child TID $childTid, ConfigID $config_id, ArrayId $arrayid");

    my $taskobj = new VRTS::Task();
    if (defined $taskobj){
        my $ret = $taskobj->load (undef, $childTid);
        my $r = $taskobj->update ($childTid, {Name => "Refresh Enclosure ",
                                     state => &VRTS::Task::TASK_STATE_RUNNING, 
                                     StartTime =>""});
	if ($r) {
	    VRTS::Logger::log('debug',"isa_refresh Failed to update TID $childTid");
	}	
  
        refresh_enclosure($config_id);

        $r = $taskobj->update ($childTid, {Name => "Refresh Enclosure ",
                                     state => &VRTS::Task::TASK_STATE_FINISHED, 
                                     StartTime =>""});
        if ($r) {
	    VRTS::Logger::log('debug',"isa_refresh Failed to update TID $childTid");
	}	
    }
    #print( "is_refresh $childTid, $config_id, $arrayid, $prntTid\n");
}

main()

