#!/opt/VRTSsfmh/bin/perl
##$Id: setuptrust.pl, 2016/04/08 20:24:55, venkat $
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $

BEGIN { unshift(@INC, "/opt/VRTSsfmh/lib/modules"); }

require Exporter;
require XML::Simple;
require VRTS::Paths;
require VRTS::Util;

my $SCP = "/usr/bin/scp";
my $NOHUP = "/usr/bin/nohup";
my $SNAS_UTILS_PATH = "/opt/VRTSsfmh/addons/SNAS/utils/";
my $SNAS_CONFIGURE_OPERATION = "configure_snas_node";
my $VOM_OPERATION_DIR = File::Spec->catdir(VRTS::Paths::get_path("InstallDir"), "operations");

use File::Temp;
use JSON;

sub adjust_putty{
    my $host=shift;
    VRTS::Logger::log('debug', "Checking putty settings for $host home $ENV{HOME}");
    my $file=$ENV{"HOME"}.'/.putty/sshhostkeys';
    return if (not -e $file);

    my $fh;
    open ($fh, '<', $file);
    my @lines = <$fh>;
    my $newFile=$ENV{"HOME"}.'/.putty/sshhostkeys.new';
    open  NEWFILE, ">$newFile";
    foreach(@lines){
       next if ($_ =~ m/$host/) ;
       print NEWFILE $_;
    }
    close NEWFILE;
    close FILE;
    rename $file, $file.".save";
    rename $newFile, $file;
    VRTS::Logger::log('debug', "Adjusted putty settings for $host file $file");
}

sub find_cluster_nodes
{
    my ($mh, $user, $pass) = @_;
    my $cmd = "/opt/SYMCsnas/pysnas/bin/controller.py cluster networkinfo";
    VRTS::Logger::log('debug', "find_cluster_nodes $cmd");    
    my $args = " -ssh -l $user -pw $pass -4 $mh";

    $ENV{'LD_LIBRARY_PATH'} = "/opt/VRTSsfmh/lib";

    my $halssh = VRTS::Paths::get_path("MHBin") . "halssh $args \"$cmd\"";
    my $out = `$halssh`;
    VRTS::Logger::log('debug', "Cluster network information : \ncmd $cmd \nout: [$out]");
    my $json = new JSON();
    my $jsonobject = $json->decode($out);

    return $jsonobject;
}

sub set_up_paswordless_ssh
{
    my ($mh, $user, $pass) = @_;

    my $pub_key = `cat /root/.ssh/id_rsa.pub`;
    chomp($pub_key);
    if ($pub_key eq "")
    {
    	my $ret =  system("ssh-keygen -t rsa -f /root/.ssh/id_rsa -N \"\"");
    	VRTS::Logger::log('debug', "Generating key: Return code $ret\n");
    	$pub_key = `cat /root/.ssh/id_rsa.pub`;
        chomp($pub_key);
    	print $pub_key;
    }

    # TBD -- make sure to clear the keys, after the job is done.
    my $cmd = "mkdir -p ~/.ssh; echo $pub_key >> ~/.ssh/authorized_keys";
    if (ssh_execcmd($mh, $user, $pass, $cmd)) {
        VRTS::Logger::log('debug', "Failed to run cmd over ssh");
    }
}

sub set_up_paswordless_ssh_cluster
{
    my ($mh, $user, $pass, $node_to_pip) = @_;

    foreach (@$node_to_pip) {
        set_up_paswordless_ssh($_->{pip}, $user, $pass);
    }
}

sub ssh_execcmd
{
    my ($mh, $user, $pass, $cmd ) = @_;

    my $args = " -ssh -l $user -pw $pass -4 $mh";
    $ENV{'LD_LIBRARY_PATH'} = "/opt/VRTSsfmh/lib";
    VRTS::Logger::log('debug', "ssh_execcmd:\nhost $mh user $user\n $cmd");
    my $halssh = VRTS::Paths::get_path("MHBin") . "halssh $args \"$cmd\"";
    #VRTS::Logger::log('debug', "Running command [$halssh]\n");
    my $ret =  system($halssh);
    VRTS::Logger::log('debug', "ssh_execcmd Return code:$ret\n");
    return $ret;
}

sub create_gendeploy
{
    my ($deployfile) = @_;

    my $gendeploy_cmd = File::Spec->catfile($SNAS_UTILS_PATH, "snas-gendeploy.pl");
    my $cmd = VRTS::Util::make_command($gendeploy_cmd, "--out", $deployfile);
    VRTS::Logger::log('debug', "\nGenDeploy CMD: [$cmd]\nSCRIPT:[$deployfile]");
    # gendeploy uses MODE as hidden param to figure execution env.
    # invoked from a CGI vs regular mode.
    $ENV{MODE} = 0;
    my $out = `export MODE=0;unset DOCUMENT_ROOT;$cmd`;
   
    unless (-f $deployfile) {
        VRTS::Logger::log('debug', "Failed to run gendeploy.");
        return undef, &VRTS::Enclosure::Discover::MH_NOT_REACHABLE;
    }
    VRTS::Logger::log('debug', "Generated deploy script for SNAS");
}

sub run_gendeploy_cluster
{
    my ($mh, $user, $pass, $deployfile, $node_to_pip) = @_;
    my $node_ip, $cmd, $ret, $configure_snas_mh_cmd;

    foreach (@$node_to_pip) {
        print $_->{nodename} . "\n";
        $node_ip = $_->{pip};
        # Push the file to the MH
        $cmd = "$SCP -o StrictHostKeyChecking=no $deployfile $user\@$node_ip:$deployfile";
        VRTS::Logger::log('debug', "Running command [$cmd]\n");
        $ret = system($cmd);
        VRTS::Logger::log('debug', "Return code $ret\n\n");
        $configure_snas_mh_cmd = "$SCP -o StrictHostKeyChecking=no $SNAS_UTILS_PATH/$SNAS_CONFIGURE_OPERATION $user\@$node_ip:$VOM_OPERATION_DIR";
        VRTS::Logger::log('debug', "Running command [$configure_snas_mh_cmd]\n");
        system($configure_snas_mh_cmd);
       	adjust_putty($mh);
        # Execute the copied script.
        ssh_execcmd($node_ip, $user, $pass, "chmod +x $deployfile");
        ssh_execcmd($node_ip, $user, $pass, $deployfile);
    }
}

sub snas_deploy
{
    VRTS::Logger::log('debug', "Enter snas_deploy");
    my ($mh, $user, $pass, $node_to_pip) = @_;

    # Setup password-less ssh.
    set_up_paswordless_ssh_cluster($mh, $user, $pass, $node_to_pip);
    VRTS::Logger::log('debug', "Established passwordless ssh to all nodes");

    # Generate the deploy file on CS.
    my $deployfile =  VRTS::Paths::get_path("AppDir") . tmpnam() . '.pl';
    create_gendeploy($deployfile);

    # copy the deploy file on all cluster nodes and run.
    run_gendeploy_cluster($mh, $user, $pass, $deployfile, $node_to_pip);

    VRTS::Logger::log('debug', "Finished snas_deploy"); 
}

sub install_mh
{
    my ($mh, $user, $pass, $console_flag) = @_;
    my $addon_name = "VRTSsfmsnas-1.0.0.0";
    my $addon_path = "/var/opt/VRTSsfmcs/store/$addon_name".
                        "/Linux/$addon_name.tar.gz";
    my $dest_dir = "/var/opt/VRTSsfmh/spool/addons/store/$addon_name/";
   
    ssh_execcmd($mh, $user, $pass, "mkdir -p $dest_dir"); 

    my $cmd = "$SCP -o StrictHostKeyChecking=no $addon_path $user\@$mh:$dest_dir/";
    VRTS::Logger::log('debug', "Copying the SNAS addon to MH $mh CMD: $cmd\n");
    system($cmd);

    # Untar and install the pkg.
    ssh_execcmd($mh, $user, $pass, "cd $dest_dir; tar -xzvf $addon_name.tar.gz");

    # Run the setup
    ssh_execcmd($mh, $user, $pass, "chmod +x $dest_dir/setup.pl");
    ssh_execcmd($mh, $user, $pass, "cd $dest_dir; export Store=$dest_dir; ./setup.pl $console_flag");

    VRTS::Logger::log('debug', "Finished install_mh");
}

sub install_mh_cluster
{
    VRTS::Logger::log('debug', "Enter install_mh");
    my ($mh, $user, $pass, $node_to_pip, $console_node, $config_id) = @_;
    foreach (@$node_to_pip) {
        VRTS::Logger::log('debug', "Node $_->{nodename} IP $_->{pip}  console Node $console_node");
        $node_ip = $_->{pip};
        if ($_->{nodename} eq $console_node) {
            install_mh($node_ip, $user, $pass, 1);
            # start the discovery
	    #
            ssh_execcmd($node_ip, $user, $pass, $NOHUP . ' /opt/VRTSsfmh/bin/perl /opt/VRTSsfmh/bin/mh_driver.pl --family HOST --hidden --discover --id discovery_S10-HOST &');
	    ssh_execcmd($node_ip, $user, $pass, $NOHUP . ' /opt/VRTSsfmh/bin/perl /opt/VRTSsfmh/bin/mh_driver.pl --family SNAS --hidden --discover --id discovery_S80-SNAS --context configid=' . $config_id . '  > /dev/null 2>&1  &' );
        } else {
            install_mh($node_ip, $user, $pass, 0);
	    ssh_execcmd($node_ip, $user, $pass, $NOHUP . ' /opt/VRTSsfmh/bin/perl /opt/VRTSsfmh/bin/mh_driver.pl --family HOST --hidden --discover --id discovery_S10-HOST &');	
           
        }


    }
}

sub main
{

    my $log = new VRTS::Logger(Module => 'MHRUN',
                               Logdir => '/var/opt/VRTSsfmh/logs',
                               Level => 'debug',
                               Logfile => 'setuptrust.log',
                               Logsize => 1024*1024*10,
                               UmiString => 'V-0000-0000');	

    my ($mh, $user, $pass, $config_id) = @ARGV;
    adjust_putty($mh);
    my $jsonobject = find_cluster_nodes($mh, $user, $pass);
    $node_to_pip = $jsonobject->{node_to_pip};
    $console_node = $jsonobject->{console_node};

    foreach (@$node_to_pip) {
        $node_ip = $_->{pip};
        if ($_->{nodename} eq $console_node) {
           ssh_execcmd($node_ip, $user, $pass, '/opt/SYMCsnas/pysnas/bin/snas_addon.py install; /opt/VRTSvcs/bin/hagrp -online VOM_SNAS_ADDON -any' );
        }
    }

    snas_deploy($mh, $user, $pass, $node_to_pip);
    install_mh_cluster($mh, $user, $pass, $node_to_pip, $console_node, $config_id);
}

main();

