#!/opt/VRTSsfmh/bin/perl
# $Id: snas-gendeploy.pl, 2016/03/16 00:47:36, venkat $
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $


BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules"); }
use strict;
use Getopt::Long;
use File::Spec;
use File::Path;
use CGI;

require VRTS::Ini;
require VRTS::Paths;
require VRTS::Util;
require VRTS::OS;

if ($^O =~ /Win32/i) 
{
    $ENV{'SystemRoot'} =~ /([A-Za-z]:\\[A-Za-z0-9_ ]+)/;
    my $system32_dir = $1."\\system32".";".$1."\\system32\\wbem";
    $ENV{'PATH'} = $system32_dir;
}
else
{
    $ENV{PATH}="/usr/bin:/bin:/sbin:/usr/sbin";
}
delete @ENV{'IFS', 'CDPATH', 'ENV', 'BASH_ENV'};


my %opts;
my @domains;
my $csip = undef;
my $cs = undef;
my $outfile = undef;
my $disable_families = 0;
my $cmd;
my $mode = 0;
my $query;
my $cs_guid = VRTS::OS->get_hostguid();


## Global Variable declaration and Path setting

my $_installdir         = VRTS::Paths::get_path ("InstallDir");
my $_appdir             = VRTS::Paths::get_path ("AppDir");
my $appdirmh            = File::Spec->canonpath ($_appdir);
my $appdircs            = $appdirmh;
   $appdircs            =~ s/VRTSsfmh/VRTSsfmcs/;
my $installdirmh        = File::Spec->canonpath ($_installdir);
my $installdircs        = $installdirmh;
   $installdircs        =~ s/VRTSsfmh/VRTSsfmcs/;
my $sfm_resolve_conf    = VRTS::Paths::get_path ("SFMResolv");
my $atutil_command      = File::Spec->catfile   ($installdirmh, "bin", "atutil");

## Global Variable declaration Ends here

sub usage
{
    print ("$0 --out <output file>\n");
    print ("$0 [--help]\n");
}

sub gen_log
{
    my $str = shift;

    if($mode == 0)
    {
        print $str;
    }
}


if (defined $ENV{"DOCUMENT_ROOT"} and $ENV{"DOCUMENT_ROOT"} ne "")
{
    $mode = 1;
    $query = new CGI;
    binmode STDOUT;

    print $query->header ("text/plain");

    # Get all the query parameters
    $outfile = $query->param("out");
    $disable_families = $query->param("disable_families");
}
else
{
    GetOptions(\%opts,qw (out=s disable-families help));
    if ($opts{"out"}) { $outfile = $opts{"out"} };
    if ($opts{"disable-families"}) { $disable_families = 1 };

    if ($opts{"help"})
    {
        usage ();
        exit (0);
    }
    if (not defined $outfile)
    {
        usage ();
        exit (1);
    }
}

my $sfm_cs = 0;
VRTS::Ini->query_section($sfm_resolve_conf, "domains", \@domains);
for my $dom (@domains)
{
    VRTS::Ini->query_value($sfm_resolve_conf, $dom, "ip", \$csip);
    if($dom =~ m/^sfm\:\/\/([a-zA-Z0-9\-_.]+)\:5634\/$/)
    {
        VRTS::Ini->query_value($sfm_resolve_conf, $dom, "sfm_cs", \$sfm_cs);
        if ($sfm_cs == 1)
        {
            $cs = $1;
            last;
        }
    }
}

if(!$cs || !$csip)
{
    gen_log "Could not figure out the central server hostname and IP\n";
    exit (1);
}

gen_log "Exporting agent credentials\n";

my $gendeploy_temp_dir = File::Spec->catdir ($appdirmh,"gendeploy");
my $tmp_cred_path      = File::Spec->catfile($gendeploy_temp_dir, "cred.xml");
if (-d $gendeploy_temp_dir) 
{
	unlink($tmp_cred_path);
}
else
{
    eval{ File::Path::mkpath($gendeploy_temp_dir,0,755)};
    if($@)
    {
        gen_log "Unable to create deploy store $gendeploy_temp_dir: $@\n";
    }
}
my $sec_data_dir       = File::Spec->catfile($appdirmh,"sfm_".$cs,"sec");

$cmd = VRTS::Util::make_command($atutil_command,
                                "export",
                                "-c", "-z",
                                $sec_data_dir,"-f",$tmp_cred_path,
                                "-p", $cs);
`$cmd`;
if(not -e $tmp_cred_path)
{
    gen_log "Credential export failed\n";
    exit (1);
}

gen_log "Creating deployment script\n";

my $perl_hash_bang_line = "#!/opt/VRTSsfmh/bin/perl";
my $perl_lib__line      = "BEGIN { \@INC = (\"/opt/VRTSsfmh/lib/modules\"\, \"\$ENV{PROGRAMW6432}\\\\Veritas\\\\VRTSsfmh\\\\lib\\\\modules\"\, \"\$ENV{ProgramFiles}\\\\Veritas\\\\VRTSsfmh\\\\lib\\\\modules\"\) }";

if($mode == 0)
{
    open(D, ">$outfile");
    binmode D;
}
else
{
    open(D, ">&STDOUT");
}

while(<DATA>)
{
    $_ =~ s/##CS_HOSTNAME##/$cs/; 
    $_ =~ s/##CS_IP##/$csip/; 
    $_ =~ s/##CS_GUID##/$cs_guid/;
    $_ =~ s/##AGP##/$cs/;
    $_ =~ s/##PERL_HASH_BANG_LINE##/$perl_hash_bang_line/;
    $_ =~ s/##PERL_LIB_LINE##/$perl_lib__line/;
    print D "$_";
}
print D "__DATA__\n";

open ( CREDFILE , $tmp_cred_path );
while(<CREDFILE>)
{
	print D $_;
}
close(CREDFILE);
close(D);

File::Path::rmtree("$gendeploy_temp_dir");
gen_log "\nManaged host deployment script is $outfile\n";
gen_log "You can use this script on a host with VRTSsfmh package installed\n";
gen_log "to configure it as a managed host for $cs\n"; 

exit(0);

__DATA__
##PERL_HASH_BANG_LINE##
##PERL_LIB_LINE##

use strict;
use Getopt::Long;
use File::Spec;
use JSON;

require VRTS::Paths;
require VRTS::CMD;
require VRTS::Util;
require VRTS::OS;
require VRTS::Logger;
require VRTS::Config;
require VRTS::SFMHCAT;
use VRTS::CGILOCALE;

## Global Variable declaration and Path setting
my $_installdir         = VRTS::Paths::get_path ("InstallDir");
my $_appdir             = VRTS::Paths::get_path ("AppDir");
my $appdirmh            = File::Spec->canonpath ($_appdir);
my $appdircs            = $appdirmh;
   $appdircs            =~ s/VRTSsfmh/VRTSsfmcs/;
my $installdirmh        = File::Spec->canonpath ($_installdir);
my $installdircs        = $installdirmh;
   $installdircs        =~ s/VRTSsfmh/VRTSsfmcs/;
my $sfm_resolve_conf    = VRTS::Paths::get_path ("SFMResolv");
my $atutil_command      = File::Spec->catfile   ($installdirmh, "bin", "atutil");
my $perl_command        = File::Spec->catfile   ($installdirmh, "bin", "perl");
$perl_command        = File::Spec->catfile   ($installdirmh, "bin", "perl.exe") if ($^O =~ /Win32/i);
my $sfme_command        = File::Spec->catfile   ($installdirmh, "bin", "sfme");
my $xprtlc              = File::Spec->catfile   ($installdirmh, "bin", "xprtlc");
my $devnull             = File::Spec->devnull();
my $ping                = File::Spec->catfile   ($installdirmh, "bin", "ping");
my $mh_logger;

my $mh_version = &VRTS::Config::MANIFEST_VERSION;
if ($mh_version =~ /^2.0/)
{
    my_print("Older MH version $mh_version is not supported on this CMS version\n");
    exit(0)
}

my $log_path = VRTS::Paths::get_path("LogDir");
$mh_logger = new VRTS::Logger(Module => 'gendeploy',
                              Logdir => $log_path,
                              Level => 'debug',
                              Logfile => 'mhrun.log',
                              Logsize => 1024*1024,
                              UmiString => 'V-0000-0000');

## Global Variable declaration and Path setting Ends here

my %opts;
my $quiet=0;
my $mh_cred;
my $sfme;
my $MH_HOSTNAME = undef; 
my $disable_families = 1;
my $cmd = undef;
my $out;
my $js;
my $obj;
my $hostid;
#task related variables
my $task_handler;
my %task_hash;
my $task_id;
my $TASK_SUPPORTED = 1;
my $cs = "##CS_HOSTNAME##";
my $cs_config_name = "##CS_HOSTNAME##";
my $cs_guid = "##CS_GUID##";

# putting eval check for Task as it is  
# not supported in pre 3.0 MH 
eval { require VRTS::Task; };
if($@) {
    $TASK_SUPPORTED = 0;
}

GetOptions(\%opts,qw (quiet=s hostname=s domain=s disable-families help));

if ($opts{"help"})
{
   usage();
   exit(0);
}

if ($opts{"hostname"}) { $MH_HOSTNAME = $opts{"hostname"} };

if ($opts{"disable-families"}) { $disable_families = 1 };

if ($opts{"quiet"}) { $quiet = 1 };
if (defined $opts{"domain"} && $opts{"domain"} ne "" ) 
{ 
    if ($mh_version =~ /^[2-3]\.|^4\.0\.1097\.0$/)
    {
        my_print("domain option is supported only with VRTSsfmh package version 4.0RU1 or greater\n");
        exit(1);
    }
    $cs = $opts{"domain"};
};

if($^O =~ /Win32/i)
{
    $atutil_command = "\"". $atutil_command . "\"";
    $perl_command = "\"". $perl_command . "\"";
    $mh_cred = "$appdirmh"."\\mh_cred.xml";
    $sfme = "$perl_command -I\"$installdirmh\\lib\\modules\" "."\"$installdirmh\\web\\admin\\cgi-bin\\sfme.pl\"";
}
else
{
    $mh_cred = "$appdirmh"."/mh_cred.xml";
    $sfme = $sfme_command;
}

sub my_print
{
    my $str = shift;
    VRTS::Logger::log('debug',$str);
    if($quiet == 0)
    {
        print $str; 
    }
}

my_print "\nVeritas Operations Manager Managed Host Deployment\n";

my_print "Initializing\n";

$cmd = VRTS::Util::make_command ($xprtlc, "-l", "https://localhost/world/getvitals", "-u", "nobody");
$out = `$cmd`;
if($? != 0)
{
    my_print "Cannot contact xprtld on this host\n";
    my_print "Managed host cannot be configured\n";
    exit(1);
}
$obj = jsonToObj ($out);

$cmd = VRTS::Util::make_command ($xprtlc, "-l", "https://$cs/world/getvitals", "-u", "nobody");
my $cs_vitals = `$cmd`;
if($? != 0)
{
    my_print "Cannot contact central server on $cs\n";
    my_print "Managed host cannot be configured\n";
    exit(1);
}

VRTS::Logger::log('debug',"Managed host version $obj->{XPRTLD_VERSION}");

# Set the hostname to default if not set through cmd line option
if (not defined $MH_HOSTNAME)
{
    my $use_default = 0;

    $obj = jsonToObj ($cs_vitals);
    $MH_HOSTNAME = $obj->{PEER_NAME};
    if(not defined $MH_HOSTNAME or $MH_HOSTNAME eq '' or $MH_HOSTNAME eq 'UNKNOWN')
    {
        $MH_HOSTNAME = $obj->{PEER_ADDR};
    }
}

# Check 2-way connectivity.
my_print "Attempting to configure host using hostname $MH_HOSTNAME.\nVerifying if it is reachable from the CMS $cs...\n";
$cmd = VRTS::Util::make_command ($xprtlc, "-u", "nobody", "-l", "https://$cs/https://$MH_HOSTNAME:5634/world/getvitals");
$out = `$cmd`;
if ($? != 0)
{
    my_print "Cannot contact managed host $MH_HOSTNAME\n";
    my_print "from central server $cs\n";
    my_print "Managed host cannot be configured\n";
    exit(1);
}


# Get the hostid
$hostid = VRTS::OS->get_hostguid();
my $is_memeber = VRTS::Util::make_command($xprtlc,"-l","https://$cs/agent/cgi-bin/is_member.pl",
                                "-d","hostid=$hostid");


my $host_in_enum = 0;
my $host_in_db = 0;

# Figure out if host is already added to domain
$cmd = VRTS::Util::make_command ($xprtlc, "-l", "https://$cs/agent/domain/enum");
$out = `$cmd 2>&1`;
if($? == 0)
{
    if($out =~ /$hostid/)
    {
        $host_in_enum = 1;
    }
}

$out = `$is_memeber 2>&1`;
if ($? == 0)
{
    if ($out =~ /Yes/)
    {
       $host_in_db  = 1;  
    }
 
}
if ($host_in_enum && $host_in_db)
{
    # Assume host is already configured to this CS only if it is both in the DB as well as in enum
    my_print "$MH_HOSTNAME is already part of domain on $cs\n";
    exit(1);
}

my_print "Extracting credential\n";
open(D, ">$mh_cred");
while(<DATA>)
{
    print D "$_";
}
close(D);

my_print "Configuring managed host\n";
my $mode = $ENV{MODE};
$ENV{MODE} = 1; 
my $cmd = "$sfme configure_snas_node cs-hostname=$cs cs-ip=##CS_IP## cs-config-name=$cs_config_name mh-hostname=$MH_HOSTNAME agent-password=##AGP## auto-config=1 disable-families=$disable_families";
if($quiet == 1)
{
   system("$cmd 1>$devnull 2>&1 &"); 
   if($? == 0) {
       #add audit entry
       _add_audit_entry();
       #create and update task to FINISHED state on successful completion
       create_task() if ($TASK_SUPPORTED);
       update_task(0, "Configure Enclosure to VOM MS $cs is successful") if ($TASK_SUPPORTED);
       upload_data ();  
   }   
   exit(0);
}

my $mh_log = File::Spec->catfile($appdirmh,'logs','mhrun.log');
my $out = `$cmd`;
if($? != 0)
{
    if (defined $out)
    {
        if ($out =~ /Error : ([^\n]+)/)
        {
            my $error_code = $1;
            if (defined $error_code)
            {
                $error_code = VRTS::SFMHCAT->$_ for $error_code;
                my $error_msg = VRTS::CGILOCALE::get_msgcat->i18nget($error_code);
                my_print "\n$error_msg" if (defined $error_msg);
            }
        }
    }
    my_print "\nFailed to configure managed host\n";
    my_print "Please refer to the log file $mh_log\n";
    $ENV{MODE} = $mode; 
    exit(1);
}
else
{
    if ($disable_families == 0)
    {
        # Check if host is added to the DB.
        my $out_put = `$is_memeber 2>&1`;
        if (($? == 0 && $out_put =~ /No/) || $? != 0)
        {
            # if host is not added to the database then fail the add host and rollback configuration
            if ($? != 0)
            {
                VRTS::Logger::log('debug',"Unable to contact CMS to verify if Host information available in DB: $out_put");
                VRTS::Logger::log('debug',"Looks like configuration was not sucessfull. Rolling it back");
            }
            else
            {
	            if ($mh_version !~ /^2./)
                {
		            ### This is post configure_mh check for duplicate VM_IDs, it is required only for older MHs(<6.1).
             		### For latest MH, we check during configure_mh operation through check_duplicate_vmid operation.
		            my %virtual_info=();
             		VRTS::OS->get_virtual_info(\%virtual_info);
            		if(defined $virtual_info{'VMGUID'})
                    {
              		    ### If host is VM then do a DB query for duplicate VM_ID
              		    ###Doing duplicate VM_ID checks for VMware env only
		                my $mh_vm_id=$virtual_info{'VMGUID'};
		                my $jsondata = JSON::to_json({'VM_ID' => $mh_vm_id,'VMM' => 'VMware'});
		                my ($ret,$out) = VRTS::Util::xprtlc_payload_send("https://$cs:5634/agent/cgi-bin/getdbinfo.pl/hosts", 
								     $jsondata,'application/json');
		  
		                my $obj;
                        if((!defined $out) || ($out eq '')) 
                        {
                              VRTS::Logger::log('debug', 'xprtlc payload output is either undefined or empty...proceed without checking for duplicate VMID');
                        } 
                        else 
                        {                
                              eval {
                                $obj = JSON::from_json($out);        
                                1;
                              } or do {
                                $obj = undef;
                                VRTS::Logger::log('debug', "could not convert xprtlc payload output to hash..output is [$out]...proceed without checking for duplicate VMID");
                              };
                        }

		                if(defined $obj)
                        {
                		    my $records = $obj->{Records};
                       		my $nh = scalar (@$records);
                    		if($nh)
                            {
                                ### If already exist a host with duplicate VM_ID then populate its hostname and type.
                                my $dup_host_type =  $obj->{Records}[0]->{TYPE};
                                my $dup_host_name =  $obj->{Records}[0]->{NAME};
                                if($dup_host_type)
                                {
                                      VRTS::Logger::log('debug',"$MH_HOSTNAME and CMS having same Virtual Machine ID. Please change $MH_HOSTNAME's Virtual Machine ID to proceed further.");
                                      my_print "$MH_HOSTNAME and CMS having same Virtual Machine ID. Please change Virtual Machine ID of $MH_HOSTNAME to proceed further.\n";
			                    }
                    			else
                                {
                                      VRTS::Logger::log('debug',"$dup_host_name is already reporting to CMS having same Virtual Machine ID as $MH_HOSTNAME. Either remove $dup_host_name or change Virtual Machine ID of $MH_HOSTNAME to proceed further.");
                                      my_print "$dup_host_name is already reporting to CMS having same Virtual Machine ID as $MH_HOSTNAME . Either remove $dup_host_name or change Virtual Machine ID of $MH_HOSTNAME to proceed further.\n";
                                }
                            }
                            else
                            {
                                VRTS::Logger::log('debug',"Host not found in database post configuration. cleaning up");
                            }
                        }
		            }
	            }
            }

            $cmd = VRTS::Util::make_command($xprtlc,"-l", "https://$cs/agent/domain/unregister","-d","id=$hostid");
            `$cmd`;

            # ping will cleanup configuration
            $cmd = VRTS::Util::make_command($perl_command,$ping);
            `$cmd`;

            my_print "\nFailed to configure managed host\n";
            my_print "Please refer to the log file $mh_log\n";
            $ENV{MODE} = $mode;
            exit(1);
        }    
    }
        
    #add audit entry
    _add_audit_entry();
    #create and update task to FINISHED state on successful completion
    create_task() if ($TASK_SUPPORTED);
    update_task(0, "$MH_HOSTNAME successfully configured as a managed host to VOM MS $cs") if ($TASK_SUPPORTED);
    upload_data ();
    my_print "\n$MH_HOSTNAME successfully configured as a managed host\n";
    my_print "and is now reporting to central server on $cs\n";    
}

$ENV{MODE} = $mode; 
exit 0;

sub upload_data
{
    my $mh_ip = $obj->{PEER_ADDR};
    my $cmd = VRTS::Util::make_command ($xprtlc, "-l", "https://$cs/agent/cgi-bin/catalog.pl/refresh/$mh_ip", "1>$devnull", "2>&1");
    system ($cmd);

    #Upload the catalogs and alert declarations.
    my $decl_file = File::Spec->catfile ($appdirmh, "declarations.json.$$");
    eval {
        $cmd = VRTS::Util::make_command ($xprtlc, "-l", "https://localhost/admin/cgi-bin/eventadm.pl/listdecl");
        # Dont report anything if the declarations cant be download.
        open (DATA, "$cmd |") || exit 0;
        my $text = join ("", <DATA>);
        close (DATA);
        my $obj = jsonToObj ($text);
        if ($obj->{"RESULT"}->{"RETURNCODE"} == 0)
        {
            my @decls;
            foreach my $topic (keys(%{$obj->{"RESULT"}->{"RESULT"}->{'declarations'}})) {
                my $decl;
                $decl->{'topic'} = $topic;
                $decl->{'data'} = $obj->{"RESULT"}->{"RESULT"}->{'declarations'}->{$topic};
                push(@decls, $decl);
            }
            my $final = {};
            $final->{'declarations'} =\@decls;
            $text = objToJson($final);
        }
        open (DATA, "> $decl_file");
        print DATA $text;
        close (DATA);
        $cmd = VRTS::Util::make_command ($xprtlc, "-l", "https://$cs/agent/cgi-bin/event.pl/declare",
            "-f", "declarations=\@$decl_file", "1>$devnull", "2>&1");
        system($cmd);
    };
    unlink ($decl_file);
}

sub usage
{
    print "$0 [--hostname <hostname>] [--domain <domain>] [--quiet] [--disable-families] [--help]\n".
          " This script adds the host to the domain of the VOM Management Server $cs\n".
          " The host must have the VRTSsfmh package installed on it.\n".
          " hostname: This is an optional hostname for the Managed Host as seen by the VOM Management Server $cs\n".
          "           The host will be discovered with this name in the VOM UI.\n".
          " quiet:    If specified the script will not print any messages on console.\n".
          " disable-families: If specified the process, will not configure any existing MH families.\n".
          " domain:   Alternate hostname/IP address for the CMS.\n".
          "           This option is supported only if VRTSsfmh package version is 4.0RU1 or greater\n";
}

sub create_task
{
    my @tmp_out_arr; 
    push @tmp_out_arr, 'Enclosure is getting added to VOM MS through gendeploy';
    $task_handler = new VRTS::Task;
    $task_hash{'Name'} = 'Add Enclosure';
    $task_hash{'source'} = $cs_config_name;
    $task_hash{'SourceId'} = $cs_guid;
    $task_hash{'state'} = 'RUNNING';    
    $task_hash{'outputs'} = \@tmp_out_arr;
    $task_id = $task_handler->create(undef, \%task_hash);
}

sub update_task
{
    my $rc = shift;
    my $op = shift;
    my @tmp_out_arr; 
    push @tmp_out_arr, $op;
    $task_hash{'state'} = 'FINISHED';
    $task_hash{'exitcodes'} = $rc;    
    $task_hash{'outputs'} = \@tmp_out_arr;
    $task_handler->update($task_id, \%task_hash);     
}

sub _add_audit_entry
{
    my @values = split(/\./, $mh_version);  
    #audit is not available pre 6.0
    if($values[0] < 6) {
        return;
    }
    #though have put version check already
    #putting eval check for more safety
    eval { require VRTS::Audit; };
    if($@) {
        return;
    }    
    my $user_who;
    my $ip_where = $MH_HOSTNAME;
    my $op_what = 'Add Enclosure';
    my $op_what_display = 'Add Enclosure';
    my $reason_why = undef;
    my $auth_status = undef;
    my $target_name = $MH_HOSTNAME;
    my $target_id = $hostid;

    #Default user
    if($^O =~ /Win32/i) {
        $user_who = 'administrator';
    }
    else {
        $user_who = 'root';
    }
    $user_who = $user_who . '@' . $MH_HOSTNAME;
 
    my $ret_code = VRTS::Audit::logit($user_who, $ip_where,
                       $op_what, $auth_status, $reason_why, $target_id, $target_name);
}
