#
# $Id
#
# $Copyright:
#

package VRTS::SNAS::S01Enclosure;

require Exporter;
require VRTS::Util;
require VRTS::Summary;
require VRTS::CMD;
require VRTS::Constants::P_ARRAY_SUBSYSTEM;
require VRTS::Constants::P_ARRAY_CONFIG_ARRAY_MAPPING;
require VRTS::Constants::P_ARRAY_UI_STRING;

use VRTS::Agent;
use strict;
use vars qw($VERSION @ISA @EXPORT );
use Data::Dumper;
use JSON;

# TODO: This needs to move to some appropriate place.
use VRTS::SNAS::SNASUtils;

$VERSION = '1.00';
@ISA     = qw(Exporter);
@EXPORT  = qw(initialize discover);

my %string_map = ("LDEV" => "LUN", "PDEV" => "Physical Disk", "REPLICATION" => "Replication",
                  "RAIDGROUP" => "Storage Pool", "THINPOOL" => "Thin Pool", "LAST_SYNC_TIME" => "Creation time");


sub initialize {

    VRTS::Logger::log('debug', "In SNAS Enclosure discovery");
    my $agent = shift;
}

sub _discover_ui_strings
{
    my $agent = shift;
    my $enc_vdid = shift;

    my $ui_tb = $agent->gettable("P_ARRAY_UI_STRING");
    while (my($key,$value) = each(%string_map))
    {
        my @ui_row;

        $ui_row[&VRTS::Constants::P_ARRAY_UI_STRING::ID] = $enc_vdid;
        $ui_row[&VRTS::Constants::P_ARRAY_UI_STRING::PROP_KEY] = $key;
        $ui_row[&VRTS::Constants::P_ARRAY_UI_STRING::PROP_VALUE] = $value;

        $ui_tb->addrow(\@ui_row);
    }
}


sub discover {

    my $agent = shift;

    my $context = $agent->get_context();
    my $table = $agent->gettable("P_ARRAY_SUBSYSTEM");

    unless($table) {
       VRTS::Logger::log( 'debug', "No table P_ARRAY_SUBSYSTEM");
       return 0;
    }
    wait_till_cvm_online();
    my $pycmd = '/opt/SYMCsnas/pysnas/bin/controller.py cluster discover';
    my $out = `$pycmd`;
    VRTS::Logger::log( 'debug', "CMD: $pycmd");
    VRTS::Logger::log( 'debug', $out);
    return unless ( defined $out );
    VRTS::Logger::log( 'debug', "Updating table @ Enclosure");

    my $hsh = from_json_out($out);
    return 0 if (!defined $hsh );
	
    my @row;

    my ($error, $clus_id) = generate_uuid();
    if (defined $error and $error == 0)
    {
        $row[&VRTS::Constants::P_ARRAY_SUBSYSTEM::ID] = $clus_id;
    }
    $row[&VRTS::Constants::P_ARRAY_SUBSYSTEM::NAME] = $hsh->{name};
    $row[&VRTS::Constants::P_ARRAY_SUBSYSTEM::IP] = $hsh->{consoleip};
    $row[&VRTS::Constants::P_ARRAY_SUBSYSTEM::VENDOR] = $hsh->{vendor};
    $row[&VRTS::Constants::P_ARRAY_SUBSYSTEM::VENDOR_ID] = $clus_id;
    $row[&VRTS::Constants::P_ARRAY_SUBSYSTEM::PRODUCT] = $hsh->{product};
    $row[&VRTS::Constants::P_ARRAY_SUBSYSTEM::FAMILY] = $hsh->{family};
    $row[&VRTS::Constants::P_ARRAY_SUBSYSTEM::VERSION] = $hsh->{version};
    $row[&VRTS::Constants::P_ARRAY_SUBSYSTEM::SERIAL_NUMBER] = "VERITAS_".$clus_id;
    $row[&VRTS::Constants::P_ARRAY_SUBSYSTEM::ARRAY_TYPE] = "NAS";

    $table->addrow(\@row);
    #$table->print_table();

    #Need to setup the enclosure VDID as this can be re-used in all other agentlets
    $agent->set_context("enclosure_vdid", $clus_id);
    $agent->set_context("enclosure_name", $hsh->{name});

    VRTS::Logger::log( 'debug', "Discovering SNAS Enclosure. context $context configid $context->{configid} refresh $context->{snas_enc_refresh}");
    if (defined $context->{configid}){
        my @row;
        my $ary_mapTable = $agent->gettable("P_ARRAY_CONFIG_ARRAY_MAPPING");

        my $config_id=$context->{configid};

        $row[&VRTS::Constants::P_ARRAY_CONFIG_ARRAY_MAPPING::ID] = $config_id;
        $row[&VRTS::Constants::P_ARRAY_CONFIG_ARRAY_MAPPING::VENDOR_ID] = $clus_id;
        $row[&VRTS::Constants::P_ARRAY_CONFIG_ARRAY_MAPPING::DISCOVERY_STATE] = 0;
        $row[&VRTS::Constants::P_ARRAY_CONFIG_ARRAY_MAPPING::TIME] = time();
        $ary_mapTable->addrow(\@row);
    }
    else{
        my @ign_tbs = ('P_ARRAY_CONFIG_ARRAY_MAPPING');
        $agent->ignoretables (\@ign_tbs);
    }
    
			
    VRTS::Logger::log('debug', "Setup vendor specific terminologies");
    _discover_ui_strings($agent, $clus_id);
    
    my $tbl = $agent->gettable("FAULT_OBJ");
    my $file = $ENV{"DRIVERVARDIR"}."/SNAS/.FAULT_OBJ" ."_r.dat";
    $tbl->bulkload($file);
    
    VRTS::Logger::log( 'debug', "Discovering SNAS Enclosure. context $context configid $context->{eventfile}");
    if (defined $context->{eventfile})
    {
    	my $events = parse_events($context->{eventfile});
    	if (defined $events) {
    	 	$agent->set_context("snasevents", $events);
    	}
	unlink($context->{eventfile});  
    }
    return 1;
}
sub read_file_json
{
    my $file = shift;
	open FILE, "<$file";
    my $buf = do { local $/; <FILE> };
    my $jsonobject = new JSON ();

    my $json_content = undef;
    eval
    {
        $json_content = $jsonobject->jsonToObj($buf);
        VRTS::Logger::log('debug', $buf);
        1;
    } or do
    {
    	VRTS::Logger::log('error', "Failed to parse JSON file $file. File Content.\n****");
    	VRTS::Logger::log('error', $buf);
    	VRTS::Logger::log('error', "****");
        $json_content = undef;
    };

    close (FILE);
    return $json_content;
}
sub parse_events {
	my $file=shift;
	my $res=undef;	
	if (! -e $file){
		VRTS::Logger::log('debug', "parse_events: file does not exist $file");
		return $res;
	}
	my $json = read_file_json($file);
	if (!defined $json){
		return $res;
	}

	my @pool=();
	my @file=();
	my @node=();
	my @share=();
	foreach (@{ $json->{events} }) {
		#print Dumper($_);
		VRTS::Logger::log('debug', "Event Desc $_->{eventDesc}");
	  	if ($_->{eventDesc} =~ /All pools are (disabled|detached)/){
	  		push @pool, $_;
	  		next;
	  	}
	  	if ($_->{eventDesc} =~ /Filesystem/ || $_->{eventDesc} =~ /File System/){
	  		push @file, $_;
	  		next;
	  	}
	  	if ($_->{eventDesc} =~ /Node /){
	  		push @node, $_;
	  		next;
	  	}
		if ($_->{eventDesc} =~ /^(NFS|CIFS|FTP) server/){
	  		push @share, $_;
	  		next;
	  	}
	}
	$res->{pool}=\@pool if (scalar @pool);
	$res->{file}=\@file if (scalar @file);
	$res->{node}=\@node if (scalar @node);
	$res->{share}=\@share if (scalar @share);
	VRTS::Logger::log('debug', "pool Events ", scalar @pool, "file Events ", scalar @file, "node Events ", scalar @node, "share Events ", scalar @share);
	return $res;
}
sub wait_till_cvm_online{
    my $cmd = '/opt/VRTSvcs/bin/hagrp -state';
    my $online=0;
    my $tries=0;
    whileNext: while($online == 0 && $tries< 18){# try for three minutes and then give up
        my @out  = qx/$cmd 2>&1/;
        my $r = $? >> 8;
        if ( $? == -1 || $r ne 0 ){
            sleep(10);
            $tries++;
            next whileNext;
        }
        my @mgmt = grep(/ONLINE/, grep(/^ManagementConsole/, @out));
        my @mgmtNode = split /\s+/,$mgmt[0];

        if (scalar @mgmt != 1){
            VRTS::Logger::log('debug', "Number of online management console node != 1. It is ".(scalar @mgmt));
            sleep(10);
            $tries++;
            next whileNext;
        }
	VRTS::Logger::log('debug', "mgmtNode:$mgmtNode[2]");
	my @cvm = grep(/ONLINE/, (grep(/$mgmtNode[2]/, (grep(/^cvm /, @out)))));
        if (scalar @cvm == 1){
            $online=1;
            last;
        }
	VRTS::Logger::log('debug', "CVM not online on mgmtNode:$mgmtNode[2] yet. Sleeping and retrying");
        $tries++;
        sleep(10);
    }
    if ($online == 0){
        VRTS::Logger::log('error', "*****  CVM is not online on this ManagementConsole node. Retried for three minutes. Discovery aborted.*****");
        exit(1);
    }
    return 1;
}
1;
