#
# $Id
#
# $Copyright:
#

package VRTS::SNAS::S02Nodes;

require Exporter;
require VRTS::Util;
require VRTS::Summary;
require VRTS::CMD;
require VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES;
require VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES_PROP;

use VRTS::Agent;
use strict;
use vars qw($VERSION @ISA @EXPORT );
use Data::Dumper;
use JSON;
use File::Path qw(mkpath);

use VRTS::RWJson;
use VRTS::SNAS::SNASUtils;

$VERSION = '1.00';
@ISA     = qw(Exporter);
@EXPORT  = qw(initialize discover);
our $gHostIds={};
our $gHostIdFile='/shared/SNAS/hostids';

sub initialize {

    VRTS::Logger::log('debug', "In SNAS Node discovery");
    my $agent = shift;
}

# return OSUUID
sub getNodeId{
    my $host = shift;
    my $cmd = &VRTS::CMD::XPRTLC." -l https://$host:5634/world/getvitals";
    VRTS::Logger::log('debug', "getNodeId: cmd $cmd");
    my $out = `$cmd 2>&1`;
    if ($?)
    {
        VRTS::Logger::log('error', "get_cms_config_info: getvitals failed :$?,$out");
	return undef;
    }
    my $js = new JSON;
    my $obj = $js->jsonToObj($out);
    return $obj->{OSUUID};
}

# create SNAS directory if it does not exist
sub createSNASDir{
    mkpath('/shared/SNAS');	
}


# reads hostid from json file -- if it exists
sub readHostIds{
    my $rc;	
    ($rc, $gHostIds) = read_json($gHostIdFile);
    $gHostIds={} if (!(defined $gHostIds));
    VRTS::Logger::log('debug', "readHostIds: rc:$rc\ndata:\n".Dumper($gHostIds));
}

sub writeHostId{
    my ($host, $hostId)=@_;
    if (defined $gHostIds->{$host}){
	return;    
    }
    $gHostIds->{$host} = $hostId;
    write_json($gHostIds, $gHostIdFile);
}

sub getHostIdForNode{
    my $host = shift;
    my $file = '/opt/SYMCsnas/conf/net_pip_list.conf';
    my $hostId;
    if (!(-e $file)){
        VRTS::Logger::log('Error', "Missing file $file");
	return 'bad_beef';
    }
    open(FILE, $file);
    my @ipList = <FILE>;
    close(FILE);
    foreach(@ipList){
        VRTS::Logger::log('debug', "getHostIdForNode: $_");
        if ($_ =~ /\b$host\b/){
            my @ips = split(/\s/,$_);
            $hostId = getNodeId($ips[0]);
	    if (defined $hostId){
	       writeHostId($host,$hostId);
	       return $hostId;
	    }
	}
    }
    if (!defined $hostId) {
        VRTS::Logger::log('Error', "getHostIdForNode:\n**** Fatal error **** ");
        VRTS::Logger::log('Error', "Discovery data will be incorrect");
	$hostId='bad_beef';
    }
    return $hostId;
}

sub getHostId{
    my ($host)=@_;
    #VRTS::Logger::log('debug', "getHostId host: $host crntId: $gHostIds->{$host}");
    return $gHostIds->{$host} if (exists $gHostIds->{$host});   
    return getHostIdForNode($host);
}

sub discover {

    my $agent = shift;

    my $context = $agent->get_context();

    VRTS::Logger::log( 'debug', "Discovering SNAS Nodes");

    my $table = $agent->gettable("P_ARRAY_SUBSYSTEM_NODES");
    #VRTS::Logger::log( 'debug', Dumper($table));
    unless($table) {
        VRTS::Logger::log( 'debug', "No table P_ARRAY_SUBSYSTEM_NODES");
        return 0;
    }
    my $table_r = $agent->gettable("P_ARRAY_SUBSYSTEM_NODES_PROP");
    unless($table_r) {
        VRTS::Logger::log( 'debug', "No table P_ARRAY_SUBSYSTEM_NODES_PROP");
        return 0;
    }
    createSNASDir();
    readHostIds();
    my $out = run_clish_command 'cluster show';
    return unless ( defined $out );
    
    VRTS::Logger::log( 'debug', "Updating table @ Nodes");
    
    $agent->set_context("cluster_show", $out);
    my $hsh = from_json_out($out);
    return 0 if (!defined $hsh );
	
    my $encl_id = get_clusteruuid();

    for my $item(@{$hsh->{"output"}->{"clusterload"}->{"nodes"}}){
        my @row;
        my @temp = split(/_/, $item->{"nodename"});
        VRTS::Logger::log( 'debug', "Added node NodeId @temp[-1] name $item->{nodename}"); 
        my $nodeId = getHostId($item->{"nodename"});
        $row[&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES::SUBSYS_ID] = $encl_id; 
        $row[&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES::ID] = $nodeId;
        $row[&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES::NAME] = $item->{"nodename"};
        $row[&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES::STATE] = $item->{"state"};
        $table->addrow(\@row);

        VRTS::Logger::log( 'debug', "OS_Version Fetch");
        my $cmd = 'cat /etc/redhat-release 2>/dev/null';
        my $cmd_out = `$cmd`;
        chomp($cmd_out);
        my @row_r;
        $row_r[&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES_PROP::NODE_ID] = $nodeId;
        $row_r[&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES_PROP::KEY] = "os_version";
        $row_r[&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES_PROP::VALUE] = $cmd_out;
        $row_r[&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES_PROP::TYPE] = "node_property";
        $table_r->addrow(\@row_r);
    }
    
    

    my $context = $agent->get_context();
    if (not defined $context->{snasevents}){
    	VRTS::Logger::log( 'debug', "SNAS events not defined");
    	return 1;
    }
    if ((not defined $context->{snasevents}->{node}) && (not defined $context->{snasevents}->{share})){
    	VRTS::Logger::log( 'debug', "SNAS node/share events not defined");
    	return 1;
    }
    my $fault_tbl = $agent->gettable("FAULT_OBJ");
    my $enc_name = $context->{"enclosure_name"};

    
    my @col_arr = (&VRTS::Constants::FAULT_OBJ::FID, &VRTS::Constants::FAULT_OBJ::OBJ_ID);
    my @val_arr=();

=pod    
    # Node online event is not seen. Hence code is disabled.
    VRTS::Logger::log( 'debug', "Events\n",@{ $context->{snasevents}->{node}});

    foreach (@{ $context->{snasevents}->{node}}){
    if (defined $context->{snasevents}->{node}) {
        VRTS::Logger::log( 'debug', "Events\n",@{ $context->{snasevents}->{node}});
    
        foreach (@{ $context->{snasevents}->{node}}){
            VRTS::Logger::log( 'debug', "$_->{eventDesc}" );
  	    if ($_->{eventDesc} =~ /Node\s+(.*)\s+went offline\./){
      	        my $node=$1;
    	        VRTS::Logger::log( 'debug', "FaultyNode $_->{eventDesc}" );
                VRTS::Summary->generate($agent, "vrts_ddl_enclosure", $encl_id.$node,
	                    "vrts_array_enclosure_node", $encl_id."_".$node, $node, 
	                    $encl_id."_".$node, "NAS_NODE_FAILING_RISK",
	                    undef, undef, undef, undef, undef, undef, $enc_name,$encl_id,"vrts_ddl_enclosure");
	        # Propagate the risk condition to the enclosure level as well...
	        VRTS::Summary->generate($agent, "vrts_ddl_enclosure", $encl_id, "vrts_ddl_enclosure",
	                    $encl_id."_".$node, $node, $enc_name, "NAS_NODE_FAILING_RISK",
	                    $encl_id."_".$node, "vrts_ddl_enclosure", "NAS_NODE_FAILING_RISK");
	    }
        }
    }
=cut
    VRTS::Logger::log( 'debug', "Events\n",@{ $context->{snasevents}->{share}});
    foreach (@{ $context->{snasevents}->{share}}){
    	VRTS::Logger::log( 'debug', "$_->{eventDesc}" );
    	if ($_->{eventDesc} =~ /^(CIFS|NFS|FTP) server {0,}(.*) is offline on (.*)/){
	    my $type=$1;	
  	    my $node=$3;
    	    VRTS::Logger::log( 'debug', "Share type $type node $node" );
	    my $risk;
	    if ($type eq "CIFS"){
                $risk =  "CIFS_SERVER_FAILING_RISK";
	    }elsif ($type eq "NFS"){
		$risk =  "NFS_SERVER_FAILING_RISK";
	    }elsif ($type eq "FTP"){
		$risk =  "FTP_SERVER_FAILING_RISK";
	    }else{
                VRTS::Logger::log( 'error', "Share type $type not supported" );
		next;
	    }    

            VRTS::Summary->generate($agent, "vrts_ddl_enclosure", $encl_id.$node,
	                    "vrts_array_enclosure_node", $encl_id."_".$node, $node, 
	                    $encl_id."_".$node, $risk,
	                    undef, undef, undef, undef, undef, undef, $enc_name,$encl_id,"vrts_ddl_enclosure");
	    # Propagate the risk condition to the enclosure level as well...
	    VRTS::Summary->generate($agent, "vrts_ddl_enclosure", $encl_id, "vrts_ddl_enclosure",
	                    $encl_id."_".$node, $node, $enc_name, $risk,
	                    $encl_id."_".$node, "vrts_ddl_enclosure", $risk);
	}
    	if ($_->{eventDesc} =~ /^(CIFS|NFS) server {0,}(.*) is online on (.*)/){
	    my $type=$1;	
  	    my $node=$3;
    	    VRTS::Logger::log( 'debug', "Share type $type node $node" );
	    my $risk;
	    if ($type eq "CIFS"){
                $risk =  "CIFS_SERVER_FAILING_RISK";
	    }elsif ($type eq "NFS"){
		$risk =  "NFS_SERVER_FAILING_RISK";
	    }elsif ($type eq "FTP"){
		$risk =  "FTP_SERVER_FAILING_RISK";
	    }else{
                VRTS::Logger::log( 'error', "Share type $type not supported" );
		next;
	    }    
            #printf("Deleting $item->{fsname}\n");
            @val_arr = ($risk,$encl_id."_".$node);
            $fault_tbl->deleteallrowswithvaluez(\@col_arr, \@val_arr);	 
            @val_arr = ($risk,$encl_id);
	    $fault_tbl->deleteallrowswithvaluez(\@col_arr, \@val_arr);	  
	}
    }#end for
    #print("Fault table finish");
    #$fault_tbl->print_table();
    return 1;
}

1;
