package VRTS::SNAS::S03Disks;

require Exporter;
require VRTS::Util;
require VRTS::Summary;
require VRTS::CMD;
require VRTS::Constants::P_ARRAY_PDEV;
require VRTS::Constants::P_ARRAY_DEVICE_GROUP;
require VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES;
require VRTS::Constants::P_ARRAY_PDEV_NODES;
require VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP;

use VRTS::Agent;
use strict;
use Switch;
use vars qw($VERSION @ISA @EXPORT );
use Data::Dumper;
use JSON;

# TODO: This needs to move to some appropriate place.
use VRTS::SNAS::SNASUtils;

$VERSION = '1.00';
@ISA     = qw(Exporter);
@EXPORT  = qw(initialize discover);

sub initialize {

	my $agent = shift;

}

sub discover {

	my $agent = shift;

	my $context = $agent->get_context();


	VRTS::Logger::log( 'debug', "Discovering SNAS Disks");

	my $table = $agent->gettable("P_ARRAY_PDEV");
        my $dg_map_tb = $agent->gettable("P_ARRAY_DEVICEGROUP_MAP");

	unless($table) {
		VRTS::Logger::log( 'debug', "No table P_ARRAY_PDEV");
		return 0;
	}

	my $nodes_tbl = $agent->gettable("P_ARRAY_SUBSYSTEM_NODES");
	my $disk_node_tbl = $agent->gettable("P_ARRAY_PDEV_NODES");

	my $get_node_name; 
        if (defined $context->{cluster_show} ){
	      $get_node_name = $context->{cluster_show};
        }else {
              $get_node_name = run_clish_command 'cluster show';
	}	
	
        my $jsonobject = from_json_out($get_node_name);
        return 0 if (!defined $jsonobject );
    
	if ( $jsonobject->{header}{status} ne 'SUCCESS' ) {
		VRTS::Logger::log( 'debug', "Failed to get cluster nodes information");
		return 0;
	}
	my %node_name_id_map = ();
	my @nodenames;
	my %disks_of_nodes;
	my $localhost = `/bin/hostname`;
	chomp($localhost);

	init_disk_vdid();
        
	my $num_of_nodes = @{$jsonobject->{output}->{clusterload}->{nodes}};
	for (my $j=0 ; $j < $num_of_nodes; $j++){
		$nodenames[$j] = $jsonobject->{output}->{clusterload}->{nodes}->[$j]->{nodename};
		my $node_row = $nodes_tbl->getrowwithvalue(&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES::NAME, $nodenames[$j]);
		$node_name_id_map{$nodenames[$j]} = $node_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES::ID];
		VRTS::Logger::log( 'debug', "node_name_id_map :: $nodenames[$j]---$node_name_id_map{$nodenames[$j]}");

        #
        #   Added code to get disk media type and vendor 
        #   from all nodes of the cluster.
        #

		my $media_cmd = '/sbin/vxdisk -x MEDIA_TYPE -x VID -x STATE -p list';
		my $hacli = "/opt/VRTSvcs/bin/hacli -cmd '$media_cmd' -sys $nodenames[$j]";
		my @node_disk_list = `$hacli`;
		if ( @node_disk_list ) {
			shift @node_disk_list;
			shift @node_disk_list;
			shift @node_disk_list;
			pop @node_disk_list;
			foreach my $disk (@node_disk_list){
				my ($dname,$dmedia,$dvendor,$dstate) = split(/\s+/,$disk);
				if (!exists ($disks_of_nodes{$nodenames[$j]}{$dname})) {
					$disks_of_nodes{$nodenames[$j]}{$dname}{media} = $dmedia if ($dmedia ne "-");
					$disks_of_nodes{$nodenames[$j]}{$dname}{vendor} = $dvendor if ($dvendor ne "-");
					$disks_of_nodes{$nodenames[$j]}{$dname}{state} = $dstate if ($dstate ne "-");
				}
			}
		}
	}

        my %dg_disk_cap_map = ();
        my %g_disk_map = ();

	my $out = run_clish_command 'storage disk list detail';
	return unless ( defined $out );
	VRTS::Logger::log( 'debug', "Updating table @ Disks... work in progress");
        my $encl_id = get_clusteruuid();
	
	my $jsonobject = from_json_out($out);
        return 0 if (!defined $jsonobject );
    
	my $disk_details;
	if ( $jsonobject->{header}{status} eq 'SUCCESS' ) {
		$disk_details = $jsonobject->{output};
		if ( defined $disk_details) {
			my $disks = $disk_details->{disks};
			foreach (@$disks) {
				next if ( $_->{enclosure} eq 'ERR' );
				my @row;
				my @disk_node_row;
				$row[&VRTS::Constants::P_ARRAY_PDEV::NAME] = $_->{diskname};
				$row[&VRTS::Constants::P_ARRAY_PDEV::SIZE] = convert_size( $_->{size} );
				$row[&VRTS::Constants::P_ARRAY_PDEV::SUBSYS_ID] = $encl_id;
				$row[&VRTS::Constants::P_ARRAY_PDEV::ID] = get_vdid($_->{diskname},$_->{id});
				$row[&VRTS::Constants::P_ARRAY_PDEV::SERIAL] = $_->{serial};
				$row[&VRTS::Constants::P_ARRAY_PDEV::AVAILABLE] = calculate_available( $_->{size}, $_->{usage} );
				$row[&VRTS::Constants::P_ARRAY_PDEV::TYPE] = $_->{enclosure};
				$row[&VRTS::Constants::P_ARRAY_PDEV::PID] = $_->{poolname};
				$row[&VRTS::Constants::P_ARRAY_PDEV::IS_SHARED] = "1";
				$row[&VRTS::Constants::P_ARRAY_PDEV::VENDOR] = $disks_of_nodes{$localhost}{$_->{diskname}}{vendor};
				$row[&VRTS::Constants::P_ARRAY_PDEV::MEDIA_TYPE] = $disks_of_nodes{$localhost}{$_->{diskname}}{media};
				if ($disks_of_nodes{$localhost}{$_->{diskname}}{state} =~ /online/){
					$row[&VRTS::Constants::P_ARRAY_PDEV::STATUS] = 0;
				} else {
					$row[&VRTS::Constants::P_ARRAY_PDEV::STATUS] = 1;
				}
				$table->addrow(\@row);

                                # Now perform the mapping with RG
                                if ($_->{poolname} ne "-")
                                { 
                                    my @dgmap_row;
                                    if (defined $dg_disk_cap_map{$encl_id."_".$_->{poolname}})
                                    {
                                        my $tmp_cap_map = $dg_disk_cap_map{$encl_id."_".$_->{poolname}};
                                        $tmp_cap_map->{"size"} += $row[&VRTS::Constants::P_ARRAY_PDEV::SIZE];
                                        $tmp_cap_map->{"available"} += $row[&VRTS::Constants::P_ARRAY_PDEV::AVAILABLE];
                                        $tmp_cap_map->{"count"} += 1;
                                    }
                                    else
                                    {
                                        my %tmp_cap_map = ();
                                        $tmp_cap_map{"size"} = $row[&VRTS::Constants::P_ARRAY_PDEV::SIZE];
                                        $tmp_cap_map{"available"} = $row[&VRTS::Constants::P_ARRAY_PDEV::AVAILABLE];
                                        $tmp_cap_map{"count"} = 1;

                                        $dg_disk_cap_map{$encl_id."_".$_->{poolname}} = \%tmp_cap_map;
                                    }

                                    $dgmap_row[&VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::DEVICE_GROUP_ID] =  $encl_id."_".$_->{poolname};
                                    $dgmap_row[&VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::DEVICE_GROUP_TYPE] = "RG";
                                    $dgmap_row[&VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::DEVICE_ID] =  get_vdid($_->{diskname},$_->{id});
                                    $dgmap_row[&VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::DEVICE_TYPE] = "PDEV";
                                    $dgmap_row[&VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::SUBSYS_ID] = $encl_id;
                                    $dgmap_row[&VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::DEVICE_GROUP_NAME] = $_->{poolname};

                                    $dg_map_tb->addrow(\@dgmap_row);
                                    #print STDERR "Non FSS Disks Name = [$_->{diskname}]\n";
                                    $g_disk_map{$_->{diskname}} = 1;
                                }
 
				for (my $j=0 ; $j < $num_of_nodes; $j++){
					$disk_node_row[&VRTS::Constants::P_ARRAY_PDEV_NODES::SUBSYS_ID] = $encl_id;
					$disk_node_row[&VRTS::Constants::P_ARRAY_PDEV_NODES::NODE_ID] = $node_name_id_map{$nodenames[$j]};
					$disk_node_row[&VRTS::Constants::P_ARRAY_PDEV_NODES::PDEV_ID] = get_vdid($_->{diskname},$_->{id});
					$disk_node_row[&VRTS::Constants::P_ARRAY_PDEV_NODES::STATUS] =  $disks_of_nodes{$nodenames[$j]}{$_->{diskname}}{state};
					$disk_node_row[&VRTS::Constants::P_ARRAY_PDEV_NODES::IS_EXPORT] = 2;
					$disk_node_tbl->addrow(\@disk_node_row);
					VRTS::Logger::log( 'debug', "Added disk $_->{diskname}, node $node_name_id_map{$nodenames[$j]} to P_ARRAY_PDEV_NODES");
					$disks_of_nodes{$nodenames[$j]}{$_->{diskname}}{is_shared} = "yes";
				}  
			}
		}
	}
	else {
             VRTS::Logger::log( 'error', "Cannot get disk list detail");
	}


	VRTS::Logger::log( 'debug', "Discovering FSS disks.");
	my $out = run_clish_command 'storage fss disk list';
	return unless ( defined $out );
	
        my $jsonobject = from_json_out($out);
        return 0 if (!defined $jsonobject );
    
	if ( $jsonobject->{header}{status} eq 'SUCCESS' ) {
		VRTS::Logger::log( 'debug', "IN process");
		my $num_of_nodes = @{$jsonobject->{output}->{nodes}};
		VRTS::Logger::log( 'debug', "num_of_nodes :: $num_of_nodes");
		for (my $i = 0; $i < $num_of_nodes; $i++) {
			$disk_details = $jsonobject->{output}->{nodes}->[$i];
			if ( defined $disk_details) {
				my $disks = $disk_details->{disks};
				my $nodename= $disk_details->{nodename};
				foreach (@$disks) {
					my @row;
					my @disk_node_row;
					my $disk_name = $_->{disk_name};
					my @dstatus = split (",",$_->{disk_status});
                                        my $disk_status = $dstatus[0];
					if (( $_->{disk_status} =~ /exported/) || ($disks_of_nodes{$nodename}{$_->{disk_name}}{is_shared} ne "yes") && ($_->{disk_status} !~ /remote/)){
						$row[&VRTS::Constants::P_ARRAY_PDEV::NAME] = $_->{disk_name};
						$row[&VRTS::Constants::P_ARRAY_PDEV::SIZE] = convert_size( $_->{disk_size} );
						$row[&VRTS::Constants::P_ARRAY_PDEV::SUBSYS_ID] = $encl_id;
						$row[&VRTS::Constants::P_ARRAY_PDEV::ID] = get_vdid($_->{disk_name},$_->{disk_id});
						$row[&VRTS::Constants::P_ARRAY_PDEV::SERIAL] = "NA";
						$row[&VRTS::Constants::P_ARRAY_PDEV::AVAILABLE] = calculate_available( $_->{disk_size}, $_->{disk_usage} );
						$row[&VRTS::Constants::P_ARRAY_PDEV::TYPE] = "disk";
						$row[&VRTS::Constants::P_ARRAY_PDEV::PID] = $_->{disk_pool};
						$row[&VRTS::Constants::P_ARRAY_PDEV::IS_SHARED] = "0";
						$row[&VRTS::Constants::P_ARRAY_PDEV::VENDOR] = $disks_of_nodes{$nodename}{$_->{disk_name}}{vendor};
						$row[&VRTS::Constants::P_ARRAY_PDEV::MEDIA_TYPE] = $disks_of_nodes{$nodename}{$_->{disk_name}}{media};

						if ($_->{disk_status} =~ /online/){
							$row[&VRTS::Constants::P_ARRAY_PDEV::STATUS] = 0;
						}else{
							$row[&VRTS::Constants::P_ARRAY_PDEV::STATUS] = 1;
						}
						
						
						$table->addrow(\@row);

                                                # Now perform the mapping with RG
                                                if ($_->{disk_pool} ne "-")
                                                {
                                                    my @dgmap_row;
                                                    print STDERR "FSS disk name = [$_->{disk_name}]\n";
                                                    if (not defined $g_disk_map{$_->{disk_name}})
                                                    {
                                                        if (defined $dg_disk_cap_map{$encl_id."_".$_->{disk_pool}})
                                                        {
                                                            my $tmp_cap_map = $dg_disk_cap_map{$encl_id."_".$_->{disk_pool}};
                                                            $tmp_cap_map->{"size"} += $row[&VRTS::Constants::P_ARRAY_PDEV::SIZE];
                                                            $tmp_cap_map->{"available"} += $row[&VRTS::Constants::P_ARRAY_PDEV::AVAILABLE];
                                                            $tmp_cap_map->{"count"} += 1;
                                                        }
                                                        else
                                                        {
                                                            my %tmp_cap_map = ();
                                                            $tmp_cap_map{"size"} = $row[&VRTS::Constants::P_ARRAY_PDEV::SIZE];
                                                            $tmp_cap_map{"available"} = $row[&VRTS::Constants::P_ARRAY_PDEV::AVAILABLE];
                                                            $tmp_cap_map{"count"} = 1;

                                                            $dg_disk_cap_map{$encl_id."_".$_->{disk_pool}} = \%tmp_cap_map;
                                                        }
                                                        $g_disk_map{$_->{disk_name}} = 1;
                                                    }


                                                    $dgmap_row[&VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::DEVICE_GROUP_ID] =  $encl_id."_".$_->{disk_pool};
                                                    $dgmap_row[&VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::DEVICE_GROUP_TYPE] = "RG";
                                                    $dgmap_row[&VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::DEVICE_ID] = get_vdid($_->{disk_name},$_->{disk_id});
                                                    $dgmap_row[&VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::DEVICE_TYPE] = "PDEV";
                                                    $dgmap_row[&VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::SUBSYS_ID] = $encl_id;
                                                    $dgmap_row[&VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::DEVICE_GROUP_NAME] = $_->{disk_pool};

                                                    $dg_map_tb->addrow(\@dgmap_row);
                                                }

						$disk_node_row[&VRTS::Constants::P_ARRAY_PDEV_NODES::SUBSYS_ID] = $encl_id;
						$disk_node_row[&VRTS::Constants::P_ARRAY_PDEV_NODES::NODE_ID] = $node_name_id_map{$nodename};
						$disk_node_row[&VRTS::Constants::P_ARRAY_PDEV_NODES::PDEV_ID] = get_vdid($_->{disk_name},$_->{disk_id});
						$disk_node_row[&VRTS::Constants::P_ARRAY_PDEV_NODES::STATUS] = $disk_status;
						if (( $_->{disk_status} =~ /exported/) )
						{
					        	$disk_node_row[&VRTS::Constants::P_ARRAY_PDEV_NODES::IS_EXPORT] = 0;
						}
						else
						{
							$disk_node_row[&VRTS::Constants::P_ARRAY_PDEV_NODES::IS_EXPORT] = 1;
						}
						$disk_node_tbl->addrow(\@disk_node_row);
						VRTS::Logger::log( 'debug', "Added FSS disk " . $_->{disk_name});
					}
				}
			}
		}
	}
	else {
            VRTS::Logger::log( 'error', "Cannot get FSS disk list detail");
	}


	VRTS::Logger::log( 'debug', "Discovering SNAS Pools");

	$table = $agent->gettable("P_ARRAY_DEVICE_GROUP");
	#VRTS::Logger::log( 'debug', Dumper($table));
	unless($table) {
		VRTS::Logger::log( 'debug', "No table P_ARRAY_DEVICE_GROUP");
		return 0;
	}

	$out = run_clish_command 'storage pool free';
	return unless ( defined $out );

	my $out1 = run_clish_command 'storage pool list';
	return unless ( defined $out1 );
	VRTS::Logger::log( 'debug', "Updating table @ Pool... work in progress");

	my $pool_details;
	my $pool_meta_details;
	
	my $jsonobject = from_json_out($out);
        return 0 if (!defined $jsonobject );
    
        my $jsonobject1 = from_json_out($out1);
        return 0 if (!defined $jsonobject1 );
    
	$pool_meta_details = $jsonobject1->{output};
	if ( $jsonobject->{header}{status} eq 'SUCCESS' ) {
		$pool_details = $jsonobject->{output};
		if ( defined $pool_details) {
			my $pools = $pool_details->{pools};
			foreach (@$pools) {
				my @row;
				my $master_poolname = $_->{poolname};
				my $total_size = convert_size($_->{totalspace});
				my $free_size = convert_size($_->{freespace});
				my $used_size = ($total_size - $free_size);
				# Poolname is uniq within a cluster. Thus using it as ID
				$row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::ID] = $encl_id."_".$_->{poolname};
				$row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::SUBSYS_ID] = $encl_id;
				$row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::NAME] = $_->{poolname};

                                $row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::RAW_SIZE] = $total_size;
                                $row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::DEVICE_SIZE] = $total_size;
				$row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::TOTAL_SIZE] = $total_size;
				$row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::FREE_SIZE] = $free_size;
				$row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::USED_SIZE] = $used_size;
                                $row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::OVERHEAD_SIZE] = 0;

                                # TODO - The above capacties dont seem to be working :-(
                                if (defined $dg_disk_cap_map{$encl_id."_".$_->{poolname}})
                                {
                                    $row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::NDEVICES] = 
                                                   $dg_disk_cap_map{$encl_id."_".$_->{poolname}}->{"count"};
                                    $row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::TOTAL_SIZE] = 
                                                   $dg_disk_cap_map{$encl_id."_".$_->{poolname}}->{"size"};
				    $row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::USED_SIZE] = 
                                                   ($dg_disk_cap_map{$encl_id."_".$_->{poolname}}->{"size"} -
                                                   $dg_disk_cap_map{$encl_id."_".$_->{poolname}}->{"available"});
				    $row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::FREE_SIZE] = 
                                                   $dg_disk_cap_map{$encl_id."_".$_->{poolname}}->{"available"};
                                }

				$row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::TYPE] = "RG"; 
				$row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::RAID_LEVEL] = "RAID0"; 
				$row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::STATE] = 1; 

				if ( defined $pool_meta_details) {
					my $meta_pools = $pool_meta_details->{pools};
					foreach (@$meta_pools) {
						my $pool_name = $_->{poolname};
						my $metapool  = $_->{metapool};
						if ( $master_poolname eq $pool_name ) 
						{
							$row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::SUBTYPE] = $metapool;
							$row[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::LAYOUT] = $_->{pool_type};
						}
					}
				} 
				$table->addrow(\@row);
				VRTS::Logger::log( 'debug', "Added pool " . $_->{poolname});
			}
		}
	}
	
    #
    # Faults as reported by CLISH
    #
        
    my $context = $agent->get_context();
    if (not defined $context->{snasevents}){
    	VRTS::Logger::log( 'debug', "No SNAS event was reported.");
    	return 1;
    }
    if (not defined $context->{snasevents}->{pool}){
    	VRTS::Logger::log( 'debug', "No pool event was reported.");
    	return 1;
    }
    my $enc_name = $context->{"enclosure_name"};
    my $enclr_id = $context->{"enclosure_vdid"};
    VRTS::Logger::log( 'debug', "Events\n",@{ $context->{snasevents}->{pool}});
=pod    
    foreach (@{ $context->{snasevents}->{pool}}){
    	VRTS::Logger::log( 'debug', "$_->{eventDesc}" );
    	if ($_->{eventDesc} =~ /All pools are (detached|disabled)/){
    		VRTS::Summary->generate($agent, 
    								"vrts_ddl_enclosure", $enclr_id,
	                    			"vrts_ddl_enclosure", $enclr_id, $enc_name, 
	                    			$enc_name, "ALL_SNAS_POOL_DISABLED",
	                    			undef, undef, undef, undef, undef, undef, undef, undef, undef);
		}
     }
=cut     
     return 1;
}
my $disk_vdid={};
sub start_dclid {
    VRTS::Logger::log( 'debug', "Starting vxdcli");
    my $cmd = '/opt/VRTSsfmh/adm/dclisetup.sh';
    my @out = qx/$cmd/;
    VRTS::Logger::log( "Dcli started:\n".Dumper(@out));
}	
sub init_disk_vdid{

    my $cmd = '/opt/VRTSsfmh/bin/vxlist -B  Device,VDID disk';
    my @lines = qx/$cmd 2>&1/;
    if ($lines[0] =~ /Authentication or communication could not be established with the server/){
       start_dclid();
       @lines = qx/$cmd 2>&1/;
    }
    foreach(@lines){
       chomp($_);
       my @kv = split(/,/,$_);
       $disk_vdid->{$kv[0]}=$kv[1];
    }
    VRTS::Logger::log( 'debug', Dumper($disk_vdid));
}

sub get_vdid{
    my ($disk,$default)=@_;
    return $disk_vdid->{$disk} if (defined $disk_vdid->{$disk}); 
    return $default;
}
1;
