#
# $Id
#
# $Copyright:
#

package VRTS::SNAS::S04Filesystem;

require Exporter;
require VRTS::Util;
require VRTS::Summary;
require VRTS::CMD;
require VRTS::Constants::P_ARRAY_NAS_FILESYSTEM;

use VRTS::Agent;
use strict;
use Switch;
use vars qw($VERSION @ISA @EXPORT );
use Data::Dumper;
use JSON;

# TODO: This needs to move to some appropriate place.
use VRTS::SNAS::SNASUtils;

$VERSION = '1.00';
@ISA     = qw(Exporter);
@EXPORT  = qw(initialize discover);

sub initialize {

    VRTS::Logger::log('debug', "In SNAS Node discovery");
    my $agent = shift;
}

sub discover {

    my $agent = shift;

    my $context = $agent->get_context();

    my $enc_name = $context->{"enclosure_name"};

    VRTS::Logger::log( 'debug', "Discovering SNAS FILESYSTEMS");

    my $table = $agent->gettable("P_ARRAY_NAS_FILESYSTEM");
    unless($table) {
        VRTS::Logger::log( 'debug', "No table P_ARRAY_NAS_FILESYSTEM");
        return 0;
    }

    my $out = run_clish_command 'storage fs list';
    return unless ( defined $out );
    VRTS::Logger::log( 'debug', "Updating table @ P_ARRAY_NAS_FILESYSTEM");
    
    my $hsh = from_json_out($out);
    return 0 if (!defined $hsh );
    		
    my $enclr_id =  get_clusteruuid();
    my $fault_tbl = $agent->gettable("FAULT_OBJ");
    my @col_arr = (&VRTS::Constants::FAULT_OBJ::FID, &VRTS::Constants::FAULT_OBJ::OBJ_ID);
    my @val_arr=();
    

    if ( $hsh->{header}{status} eq 'SUCCESS' ) {
        for my $item(@{$hsh->{"output"}->{"filesystems"}}){
            my @row;
            $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::ID] = $enclr_id."_".$item->{"fsname"};
            $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::SUBSYS_ID] = $enclr_id;
            $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::NAME] = $item->{"fsname"};
            $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::FILE_STORAGE_CAPACITY] =  convert_size($item->{"size"});
            $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::FILE_STORAGE_USED] = get_storage_used($item->{"use"},$item->{"size"});
            $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::LAYOUT] = $item->{"layout"};
            $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::STATUS] = $item->{"status"};
	    if ($item->{"nmirrors"} eq "-"){
		$row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::MIRRORS] = 1;
            }else {		
                $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::MIRRORS] = $item->{"nmirrors"};
	    }	
	    if ($item->{"ncols"} eq "-"){
		$row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::COLUMNS] = 1;
            }else {		
                $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::COLUMNS] = $item->{"ncols"};
	    }

            $item->{"use"} =~ s/\%//g;
	    #print ("FS ". $item->{"fsname"}." Usage ". $item->{"use"} );
	    #
	    # Temp hack
	    @val_arr = ("NAS_FS_INODE_USAGE_RISK_THRESHOLD",$enclr_id."_".$item->{"fsname"});
	    $fault_tbl->deleteallrowswithvaluez(\@col_arr, \@val_arr);	 
	    @val_arr = ("NAS_FS_INODE_USAGE_RISK_THRESHOLD",$enclr_id);
	    $fault_tbl->deleteallrowswithvaluez(\@col_arr, \@val_arr);	  

 	    @val_arr = ("NAS_FS_CONS_RISK_THRESHOLD",$enclr_id."_".$item->{"fsname"});
            $fault_tbl->deleteallrowswithvaluez(\@col_arr, \@val_arr);	 
            @val_arr = ("NAS_FS_CONS_RISK_THRESHOLD",$enclr_id);
	    $fault_tbl->deleteallrowswithvaluez(\@col_arr, \@val_arr);

	    if ($item->{"use"} > 90 ){
            	# Fault condition as the consumed size is sitting at 90%

	        VRTS::Summary->generate($agent, 
			            "vrts_ddl_enclosure", $enclr_id."_".$item->{"fsname"},
                                    "vrts_array_enclosure_node", $enclr_id."_".$item->{"fsname"}, $item->{"fsname"}, 
                                    $enclr_id."_".$item->{"fsname"}, "NAS_FS_CONS_RISK_THRESHOLD",
                                    undef, undef, undef, undef, undef, undef, $enc_name,$enclr_id,"vrts_ddl_enclosure");
	
        	# Propogate the risk condition to the enclosure level as well...
	        VRTS::Summary->generate($agent, 
			            "vrts_ddl_enclosure", $enclr_id, 
				    "vrts_ddl_enclosure",$enclr_id."_".$item->{"fsname"}, $item->{"fsname"}, 
				    $enc_name, "NAS_FS_CONS_RISK_THRESHOLD",
                                    $enclr_id."_".$item->{"fsname"}, "vrts_ddl_enclosure", "NAS_FS_CONS_RISK_THRESHOLD");
	    }else{
            	 @val_arr = ("NAS_FS_CONS_RISK_THRESHOLD",$enclr_id."_".$item->{"fsname"});
                 $fault_tbl->deleteallrowswithvaluez(\@col_arr, \@val_arr);	 
                 @val_arr = ("NAS_FS_CONS_RISK_THRESHOLD",$enclr_id);
                 $fault_tbl->deleteallrowswithvaluez(\@col_arr, \@val_arr);	  
	    }

	    #print("Fault table finish");
	    #$fault_tbl->print_table();
            my $shared_str;
            if ($item->{"nfs_shared"} eq "yes" ) {
	        $shared_str = "1";
            } else {
                $shared_str = "0";
            }
            if ($item->{"cifs_shared"} eq "yes" ) {
                $shared_str = $shared_str."1";
	    } else {
                $shared_str = $shared_str."0";
	    }
            if ($item->{"ftp_shared"} eq "yes") {
                $shared_str = $shared_str."1";
            } else {
	        $shared_str = $shared_str."0";
	    }
            $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::SHARE] = $shared_str;
            my $cmd = 'storage fs list '.$item->{"fsname"};

            my $fs_list_fname = run_clish_command $cmd;
            
            my $flag;
            my $fs_list_fname_ref = from_json_out($fs_list_fname);
    	    return 0 if (!defined $fs_list_fname_ref );
    
            my %p_hash;
            my @p_arr;


            if ( $fs_list_fname_ref->{header}{status} eq 'SUCCESS' ) {
		#BUG BUG this should be interger
		#
=pod
		if ($fs_list_fname_ref->{"output"}->{"fullfsck_status"} eq "Not Running") {
                    $flag = "0";
                } else {
                    $flag = "1";
                }
                if ($fs_list_fname_ref->{"output"}->{"defrag_status"} eq "Not Running") {
                    $flag = $flag."0";
                } else {
                    $flag = $flag."1";
                }
		 

                $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::FLAGS] = $flag;
=cut		
		# BUGBUG this has to be integer . Clish output is "block_size":"Bytes",
		#$row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::BLOCK_SIZE] = $fs_list_fname_ref->{"output"}->{"block_size"};
		#
                $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::SUBTYPE] = $fs_list_fname_ref->{"output"}->{"fs_type"};
                for my $tiers (@{$fs_list_fname_ref->{"output"}->{"tiers"}}) {
                    if ( $tiers->{"type"} eq "Primary" ) {
			#BUGUG this should be integer, Clish output is "stripeunit":"0.00 K"     
			#$row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::STRIPE_UNIT] = $tiers->{"stripeunit"};
                    }

                    #
                    # Get list of pools
                    #  

                    for my $mirror (@{$tiers->{mirrors}}){
                        push(@p_arr,@{$mirror->{pools}});
                    }
                }
                for my $pname (@p_arr) {
                    if (!exists($p_hash{$pname})) { $p_hash{$pname} = 1; }
                }
                my @unique_parr = keys(%p_hash);
                my $pool_list = join(",",@unique_parr);
                $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::DEVICE_GROUP_ID] = $enclr_id."_".$pool_list;
            }
            $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::IS_QUOTA_ENABLED] = 0;
            $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::HIGH_WATERMARK] = 0;
            $row[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::CACHE_ID] = "NA";
            $table->addrow(\@row);
            VRTS::Logger::log( 'debug', "Row added for FS  $item->{'fsname'}");
        } #end for
    }#end if        
        
    #
    # Faults as reported by CLISH
    #
        
    my $context = $agent->get_context();
    if (not defined $context->{snasevents}){
    	VRTS::Logger::log( 'debug', "No SNAS event was reported.");
    	return 1;
    }
    if (not defined $context->{snasevents}->{file}){
    	VRTS::Logger::log( 'debug', "No filesystem event was reported.");
    	return 1;
    }
    my $enc_name = $context->{"enclosure_name"};
    VRTS::Logger::log( 'debug', "Events\n",@{ $context->{snasevents}->{file}});
    foreach (@{ $context->{snasevents}->{file}}){
    	VRTS::Logger::log( 'debug', "$_->{eventDesc}" );
    	if ($_->{eventDesc} =~ /Filesystem\s+(.*)\s+disabled on(.*)/){
    		my $obj=$1;
    		VRTS::Logger::log( 'debug', "FaultyNode $_->{eventDesc}" );
                VRTS::Summary->generate($agent, "vrts_array_nasfilesystem", $enclr_id.$obj,
	                    "vrts_array_nasfilesystem", $enclr_id."_".$obj, $obj, 
	                    $enclr_id."_".$obj, "NAS_FS_DISABLED",
	                    undef, undef, undef, undef, undef, undef, $enc_name,$enclr_id,"vrts_ddl_enclosure");
	        # Propagate the risk condition to the enclosure level as well...
	        VRTS::Summary->generate($agent, "vrts_ddl_enclosure", $enclr_id, "vrts_array_nasfilesystem",
	                    $enclr_id."_".$obj, $obj, $enc_name, "NAS_FS_DISABLED",
	                    $enclr_id."_".$obj, "vrts_array_nasfilesystem", "NAS_FS_DISABLED");
	}elsif ($_->{eventDesc} =~ /numinodes\s+(.*)\s+File System\s+(.*), current usage(.*)/){
		my $obj=$2;
=pod		
    		VRTS::Logger::log( 'debug', "FaultyNode $_->{eventDesc} **OBJ: $obj " );
                VRTS::Summary->generate($agent, "vrts_array_nasfilesystem", $enclr_id.$obj,
	                    "vrts_array_nasfilesystem", $enclr_id."_".$obj, $obj, 
	                    $enclr_id."_".$obj, "NAS_FS_INODE_USAGE_RISK_THRESHOLD",
	                    undef, undef, undef, undef, undef, undef, $enc_name,$enclr_id,"vrts_ddl_enclosure");
	        # Propagate the risk condition to the enclosure level as well...
	        VRTS::Summary->generate($agent, "vrts_ddl_enclosure", $enclr_id, "vrts_array_nasfilesystem",
	                    $enclr_id."_".$obj, $obj, $enc_name, "NAS_FS_INODE_USAGE_RISK_THRESHOLD",
	                    $enclr_id."_".$obj, "vrts_array_nasfilesystem", "NAS_FS_INODE_USAGE_RISK_THRESHOLD");
=cut	
        }	

    }
    
    return 1;
}
    
1;
