#
# $Id
#
# $Copyright:
#

package VRTS::SNAS::S05Shares;

require Exporter;
require VRTS::Util;
require VRTS::Summary;
require VRTS::CMD;
require VRTS::Constants::P_ARRAY_NAS_SHARE;
require VRTS::Constants::P_ARRAY_SHARE_CONSUMER;
require VRTS::Constants::P_ARRAY_NAS_FILESYSTEM;
require VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES;
use VRTS::Agent;
use strict;
use vars qw($VERSION @ISA @EXPORT );
use Data::Dumper;
use JSON;

# TODO: This needs to move to some appropriate place.
use VRTS::SNAS::SNASUtils;

$VERSION = '1.00';
@ISA     = qw(Exporter);
@EXPORT  = qw(initialize discover);

sub initialize {

    VRTS::Logger::log('debug', "In SNAS Shares discovery");
    my $agent = shift;
}

sub discover {

    my $agent = shift;
    my $context = $agent->get_context();

    VRTS::Logger::log( 'debug', "Discovering SNAS SHARES");

    my $table = $agent->gettable("P_ARRAY_NAS_SHARE");
    unless($table) {
        VRTS::Logger::log( 'debug', "No table P_ARRAY_NAS_SHARE");
        return 0;
    }
    my $table1 = $agent->gettable("P_ARRAY_SHARE_CONSUMER");
    unless($table) {
        VRTS::Logger::log( 'debug', "No table P_ARRAY_SHARE_CONSUMER");
        return 0;
    }
    my $fs_tbl = $agent->gettable("P_ARRAY_NAS_FILESYSTEM");
    unless($fs_tbl) {
        VRTS::Logger::log( 'debug', "No table P_ARRAY_NAS_FILESYSTEM");
        return 0;
    }

    my $nodes_tbl = $agent->gettable("P_ARRAY_SUBSYSTEM_NODES");
    my $disk_node_tbl = $agent->gettable("P_ARRAY_PDEV_NODES");

    my $get_node_name; 
    if (defined $context->{cluster_show} ){
	  $get_node_name = $context->{cluster_show};
    }else {
          $get_node_name = run_clish_command 'cluster show';
	  return unless ( defined $get_node_name );
    }

    my $jsonobject = from_json_out( $get_node_name);
    return 0 if (!defined $jsonobject );

    if ( $jsonobject->{header}{status} ne 'SUCCESS' ) {
            VRTS::Logger::log( 'debug', "Failed to get cluster nodes information");
            return 0;
    }

    my $num_of_nodes = @{$jsonobject->{output}->{clusterload}->{nodes}};

    my $out = run_clish_command 'nfs share show';
    return unless ( defined $out );
    VRTS::Logger::log( 'debug', "Updating table @ P_ARRAY_NAS_SHARE and P_ARRAY_SHARE_CONSUMER");
    
    my $nfs_share_hsh = from_json_out($out);
    return 0 if (!defined $nfs_share_hsh);
    
    my $enclr_id =  get_clusteruuid();
    if ( $nfs_share_hsh->{header}{status} eq 'SUCCESS' ) {
        my %falulted_shares;
        for my $item(@{$nfs_share_hsh->{"output"}->{faulteds}}){
            my @nodes = split(/\s+/,$item->{onfaulted});
            my $no_of_nodes = @nodes;
            # If Share faulted  on all nodes then only we will mark
            # share as faulted, as it will not be accessible form any nodes.
            if ( $no_of_nodes == $num_of_nodes ) {
                $falulted_shares{$item->{path}} = "faulted";
            }
        }

        for my $share(@{$nfs_share_hsh->{'output'}->{'shares'}}){
            my @row;
            my @row_share_consumer;
            my @fs_name = split ("\/",$share->{'path'});
            my $fsname = $fs_name[2];
            my $fs_row = $fs_tbl->getrowwithvalue(&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::NAME, $fsname);
            my $fs_id = $fs_row->[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::ID];
            if (defined $fs_id) {
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::ID] = $enclr_id."_NFS_".$share->{'path'};
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::NAME] = $share->{'path'};
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::PATH] = $share->{'path'};
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::PARENT_ID] = $fs_id;
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::PARENT_TYPE] = "NFS";
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::MOUNT_POINT_TYPE] = "NFS";
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::SUBSYS_ID] = $enclr_id;
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::SUBTYPE] = 'NA';
                if (exists($falulted_shares{$share->{'path'}})) {
                    $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::STATUS] = 'Faulted';
                } else {
                    $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::STATUS] = 'Online'; 
                }
                $row_share_consumer[&VRTS::Constants::P_ARRAY_SHARE_CONSUMER::PRIVILEGE] = $share->{'options'};
                $row_share_consumer[&VRTS::Constants::P_ARRAY_SHARE_CONSUMER::SHARE_ID] = $enclr_id."_NFS_".$share->{'path'};
                $row_share_consumer[&VRTS::Constants::P_ARRAY_SHARE_CONSUMER::HOST_NAME] = $share->{'clients'};
#                $row_share_consumer[&VRTS::Constants::P_ARRAY_SHARE_CONSUMER::HOST_ID] = $share->{'clients'};
                $table->addrow(\@row);
                $table1->addrow(\@row_share_consumer);
                VRTS::Logger::log( 'debug', "Row added for NFS SHARE  $share->{'path'}");
            }
        }
    }

    # Discovery for cifs share
    VRTS::Logger::log( 'debug', "Will start running discovery on CIFS...");

    my $out = run_clish_command 'cifs share show';
    return unless ( defined $out );

    VRTS::Logger::log( 'debug', "Updating table @ P_ARRAY_NAS_SHARE and P_ARRAY_SHARE_CONSUMER");
    
    my $cifs_share_hsh = from_json_out($out);
    return 0 if (!defined $cifs_share_hsh);
    
    
    my $enclr_id =  get_clusteruuid();
    if ( $cifs_share_hsh->{header}{status} eq 'SUCCESS' ) {
        for my $share(@{$cifs_share_hsh->{'output'}->{'shares'}}){
            my @row;
            my @row_share_consumer;
            my @fs_name = split ("\/",$share->{'filesystem'});
            my $fsname = $fs_name[0];
            my $fs_row = $fs_tbl->getrowwithvalue(&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::NAME, $fsname);
            my $fs_id = $fs_row->[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::ID];
            my $fs_status = $fs_row->[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::STATUS];
            if (defined $fs_id) {
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::ID] = $enclr_id."_CIFS_".$share->{'sharename'};
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::NAME] = $share->{'sharename'};
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::PATH] = "/vx/".$share->{'filesystem'};
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::PARENT_ID] = $fs_id;
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::PARENT_TYPE] = "CIFS";
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::MOUNT_POINT_TYPE] = "CIFS";
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::SUBSYS_ID] = $enclr_id;
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::SUBTYPE] = 'NA';

                # As in clish we are not getting any status 
                # for CIFS share, so we are keepting share status
                # same as its filesystem status.
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::STATUS] = $fs_status;

                $row_share_consumer[&VRTS::Constants::P_ARRAY_SHARE_CONSUMER::PRIVILEGE] = $share->{'options'};
                $row_share_consumer[&VRTS::Constants::P_ARRAY_SHARE_CONSUMER::SHARE_ID] = $enclr_id."_CIFS_".$share->{'sharename'};
                $row_share_consumer[&VRTS::Constants::P_ARRAY_SHARE_CONSUMER::HOST_NAME] = "*";
                $row_share_consumer[&VRTS::Constants::P_ARRAY_SHARE_CONSUMER::HOST_ID] = "*";
                $table->addrow(\@row);
                $table1->addrow(\@row_share_consumer);
                VRTS::Logger::log( 'debug', "Row added for CIFS SHARE  $share->{'sharename'}");
            }
        }
    }
    
    # Discovery for s3 share
    VRTS::Logger::log( 'debug', "Will start running discovery on S3...");
    my $out = run_clish_command 'ObjectAccess share show';

    return unless ( defined $out );
    VRTS::Logger::log( 'debug', "Updating table @ P_ARRAY_NAS_SHARE and P_ARRAY_SHARE_CONSUMER");
    
    my $s3_share_hsh = from_json_out($out);
    return 0 if (!defined $s3_share_hsh);
    
    my $enclr_id =  get_clusteruuid();
    if ( $s3_share_hsh->{header}{status} eq 'SUCCESS' ) {
        my @output = @{$s3_share_hsh->{'output'}};
        for my $share_idx(0..$#output){
            if($#{$output[$share_idx]} != 1) {
                next;
            }
            my $sharename = $output[$share_idx][0];
            my $sharetype = $output[$share_idx][1];
            my @row;
            my @row_share_consumer;
            my @fs_name = split ("\/",$sharename);
            my $fsname = $fs_name[2];
            my $fs_row = $fs_tbl->getrowwithvalue(&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::NAME, $fsname);
            my $fs_id = $fs_row->[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::ID];
            my $fs_status = $fs_row->[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::STATUS];
            if (defined $fs_id) {
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::ID] = $enclr_id."_".$sharetype."_".$sharename;
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::NAME] = $sharename;
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::PATH] = $sharename;
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::PARENT_ID] = $fs_id;
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::PARENT_TYPE] = $sharetype;
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::MOUNT_POINT_TYPE] = $sharetype;
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::SUBSYS_ID] = $enclr_id;
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::SUBTYPE] = 'NA';

                # As in clish we are not getting any status
                # for S3 share, so we are keepting share status
                # same as its filesystem status.
                $row[&VRTS::Constants::P_ARRAY_NAS_SHARE::STATUS] = $fs_status;
                
                $row_share_consumer[&VRTS::Constants::P_ARRAY_SHARE_CONSUMER::PRIVILEGE] = "";
                $row_share_consumer[&VRTS::Constants::P_ARRAY_SHARE_CONSUMER::SHARE_ID] = $enclr_id."_".$sharetype."_".$sharename;
                $row_share_consumer[&VRTS::Constants::P_ARRAY_SHARE_CONSUMER::HOST_NAME] = "*";
                $row_share_consumer[&VRTS::Constants::P_ARRAY_SHARE_CONSUMER::HOST_ID] = "*";
                $table->addrow(\@row);
                $table1->addrow(\@row_share_consumer);
                VRTS::Logger::log( 'debug', "Row added for S3 SHARE  $sharename");
            }
        }
    }
    
    return 1;
}

1;

