#
# $Id: S06Capabilities.pm, 2016/03/02 21:58:51, venkat $
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#

package VRTS::SNAS::S06Capabilities;

require Exporter;
require VRTS::Util;
require VRTS::Summary;
require VRTS::CMD;
require VRTS::Constants::P_ARRAY_CAPABILITY;

use VRTS::Agent;
use strict;
use vars qw($VERSION @ISA @EXPORT );
use Data::Dumper;
use JSON;

# TODO: This needs to move to some appropriate place.
use VRTS::SNAS::SNASUtils;


my %capabilities_col_map = ("VOL_CAPABLE" => {"index" => 21, "prop_name" => "VOL_CAPABLE"},
     "SHOW_RG_LDEV" => {"index" => 22, "prop_name" => "SHOW_RG_LDEV"},
     "SHOW_RG_PDEV" => {"index" => 23, "prop_name" => "SHOW_RG_PDEV"},
     "SHOW_PDEV" => {"index" => 24, "prop_name" => "SHOW_PDEV"},
     "SHOW_LDEV" => {"index" => 25, "prop_name" => "SHOW_LDEV"},
     "SHOW_REPLICATION" => {"index" => 26, "prop_name" => "SHOW_REPLICATION"},
     "SHOW_TP_SOURCELDEV" => {"index" => 27, "prop_name" => "SHOW_TP_SOURCELDEV"},
     "SHOW_TP_THINLDEV" => {"index" => 28, "prop_name" => "SHOW_TP_THINLDEV"},
     "SHOW_LDEV_PDEV" => {"index" => 29, "prop_name" => "SHOW_LDEV_PDEV"},
     "PRIMARY_REMOTE_REP_CAPABLE" => {"index" => 30, "prop_name" => "PRIMARY_REMOTE_REP_CAPABLE"},
     "SHOW_VSET" => {"index" => 31, "prop_name" => "SHOW_VSET"},
     "SHOW_TP" => {"index" => 32, "prop_name" => "SHOW_TP"},
     "SHOW_RG" => {"index" => 33, "prop_name" => "SHOW_RG"},
     "SHOW_PDEV_FOR_THIN_LDEV" => {"index" => 35, "prop_name" => "SHOW_PDEV_FOR_THIN_LDEV"},
     "SHOW_PDEV_FOR_TP_SOURCE_THINLDEV" => {"index" => 36, "prop_name" => "SHOW_PDEV_FOR_TP_SOURCE_THINLDEV"},
     "SHOW_DEVICE_GROUP_REPLICATION" => {"index" => 37, "prop_name" => "SHOW_DEVICE_GROUP_REPLICATION"},
     "SHOW_NAS_FS_REPLICATION" => {"index" => 70, "prop_name" => "SHOW_NAS_FS_REPLICATION"},
     "SECONDARY_REMOTE_REP_CAPABLE" => {"index" => 38, "prop_name" => "SECONDARY_REMOTE_REP_CAPABLE"},
     "LDEV_PDEV_MAP" => {"index" => 39, "prop_name" => "LDEV_PDEV_MAP"},
     "SHOW_VFILER" => {"index" => 42, "prop_name" => "SHOW_VFILER"},
     "SHOW_RANK" => {"index" => 52, "prop_name" => "SHOW_RANK"},
     "NAS_CAPABLE" => {"index" => 53, "prop_name" => "NAS_CAPABLE"},
     "STATS_CAPABLE" => {"index" => 54, "prop_name" => "STATS_CAPABLE"},
     "CONSIDER_SEGMENT_LUN" => {"index" => 62, "prop_name" => "CONSIDER_SEGMENT_LUN"},
     "SHOW_NODE" => {"index" => 63, "prop_name" => "SHOW_NODE"},
     "SHOW_NODE_CAPACITY" => {"index" => 64, "prop_name" => "SHOW_NODE_CAPACITY"},
     "SHOW_INITIATOR" => {"index" => 65, "prop_name" => "SHOW_INITIATOR"},
     "SHOW_PORT" => {"index" => 66, "prop_name" => "SHOW_PORT"},
     "SHOW_VFILER_OVERVIEW" => {"index" => 67, "prop_name" => "SHOW_VFILER_OVERVIEW"},
     "SHOW_QTREE_FILE_CAP" => {"index" => 68, "prop_name" => "SHOW_QTREE_FILE_CAP"},
     "SHOW_TIER" => {"index" => 55, "prop_name" => "SHOW_TIER"},
     "SHOW_PHYSICAL_OVERHEAD_SIZE" => {"index" => 72, "prop_name" => "SHOW_PHYSICAL_OVERHEAD_SIZE"},
     "SHOW_LOGICAL_PDEV" => {"index" => 73, "prop_name" => "SHOW_MDISK_PHYSICAL_DISK"},
     "SHOW_TP_TLDEV" => {"index" => 74, "prop_name" => "SHOW_TP_TLDEV"},
     "GROUP_RG_TP" => {"index" => 75, "prop_name" => "GROUP_RG_TP"},
     "HIDE_SV_RAID_OVERHEAD" => {"index" => 77,"prop_name" => "HIDE_SV_RAID_OVERHEAD"},
     "SHOW_TP_PDEV" => {"index" => 76, "prop_name" => "SHOW_TP_PDEV"},
     "HIDE_BLOCK_STORAGE_MAPPING" => {"index" => 78, "prop_name" => "HIDE_BLOCK_STORAGE_MAPPING"},
     "SHOW_BUCKET" => {"index" => 79, "prop_name" => "SHOW_BUCKET"});

my %supported_capabilities = ("VOL_CAPABLE" => 0, "SHOW_RG_LDEV" => 0, "HIDE_BLOCK_STORAGE_MAPPING" => 1,
                              "SHOW_RG_PDEV" => 1, "SHOW_PDEV" => 1, "SHOW_LDEV" => 1, "SHOW_BUCKET" => 1,
                              "SHOW_REPLICATION" => 0, "SHOW_TP_SOURCELDEV" => 1, "SHOW_TP_THINLDEV" => 0, 
                              "SHOW_LDEV_PDEV" => 1, "LDEV_PDEV_MAP" => 0, "SHOW_NAS_FS_REPLICATION" => 1,
                              "PRIMARY_REMOTE_REP_CAPABLE" => 0, "SHOW_VSET" => 0, "SHOW_TP" => 0, 
                              "SHOW_RG" => 1, "SECONDARY_REMOTE_REP_CAPABLE" => 0, "NAS_CAPABLE" => 1,
                              "SHOW_PDEV_FOR_THIN_LDEV" => 1, "SHOW_PDEV_FOR_TP_SOURCE_THINLDEV" => 0, 
                              "PRIMARY_REMOTE_REP_CAPABLE" => 0, "SHOW_QTREE_FILE_CAP" => 0,
                              "SHOW_DEVICE_GROUP_REPLICATION" => 1, "STATS_CAPABLE" => 1,
                              "HIDE_BLOCK_STORAGE_MAPPING" => 1, "SHOW_BUCKET" => 1);

# TODO: This needs to move to some appropriate place.
use VRTS::SNAS::SNASUtils;

$VERSION = '1.00';
@ISA     = qw(Exporter);
@EXPORT  = qw(initialize discover);

sub initialize {

    VRTS::Logger::log('debug', "In SNAS Node discovery");
    my $agent = shift;
}

sub discover {

    my $agent = shift;

    my $context = $agent->get_context();

    VRTS::Logger::log( 'debug', "Discovering SNAS Nodes");

    my $cab_tb = $agent->gettable("P_ARRAY_CAPABILITY");

    
    my ($error, $e_vdid) = generate_uuid();
    if (defined $error and $error != 0)
    {
        return 1;
    }

    while(my($key, $value) = each (%capabilities_col_map))
    {
        my @cab_row;

        $cab_row[&VRTS::Constants::P_ARRAY_CAPABILITY::ARRAY_ID] = $e_vdid;
        $cab_row[&VRTS::Constants::P_ARRAY_CAPABILITY::ARRAY_PROP_ID] = $value->{index};
        $cab_row[&VRTS::Constants::P_ARRAY_CAPABILITY::VALUE] = 0;

        $cab_tb->addrow(\@cab_row);
    }

    while(my($key, $value) = each (%supported_capabilities))
    {
        my @cab_row;

        $cab_row[&VRTS::Constants::P_ARRAY_CAPABILITY::ARRAY_ID] = $e_vdid;
        $cab_row[&VRTS::Constants::P_ARRAY_CAPABILITY::ARRAY_PROP_ID] = $capabilities_col_map{$key}->{index};
        $cab_row[&VRTS::Constants::P_ARRAY_CAPABILITY::VALUE] = $value;

        $cab_tb->addrow(\@cab_row);
    }

    #$cab_tb->print_table();

    return 0;
}

1;
