#
# $Id
#
# $Copyright:
#

package VRTS::SNAS::S07IPAddress;

require Exporter;
require VRTS::Util;
require VRTS::Summary;
require VRTS::CMD;
require VRTS::Constants::P_ARRAY_NODE_INTF_ADDR;
require VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES;

use VRTS::Agent;
use strict;
use vars qw($VERSION @ISA @EXPORT );
use Data::Dumper;
use JSON;

# TODO: This needs to move to some appropriate place.
use VRTS::SNAS::SNASUtils;

$VERSION = '1.00';
@ISA     = qw(Exporter);
@EXPORT  = qw(initialize discover);

sub initialize {

	VRTS::Logger::log('debug', "In ISA IPAddress discovery");
	my $agent = shift;
}

sub discover {

	my $agent = shift;

	my $context = $agent->get_context();

	VRTS::Logger::log( 'debug', "Discovering ISA IPAddresses");

	my $table = $agent->gettable("P_ARRAY_NODE_INTF_ADDR");
	#VRTS::Logger::log( 'debug', Dumper($table));
	unless($table) {
	   VRTS::Logger::log( 'debug', "No table P_ARRAY_NODE_INTF_ADDR");
	   return 0;
	}

	my $nodes_tbl = $agent->gettable("P_ARRAY_SUBSYSTEM_NODES");
	
	my $get_node_name; 
        if (defined $context->{cluster_show} ){
            $get_node_name = $context->{cluster_show};
        }else {
            $get_node_name = run_clish_command 'cluster show';
	    return unless ( defined $get_node_name );
        }
    
	my $jsonobject = from_json_out($get_node_name);
    return 0 if (!defined $jsonobject );

	my %node_name_id_map = ();
	my @nodenames;
	my $num_of_nodes = @{$jsonobject->{output}->{clusterload}->{nodes}};
	for (my $j=0 ; $j < $num_of_nodes; $j++){
		$nodenames[$j] = $jsonobject->{output}->{clusterload}->{nodes}->[$j]->{nodename};
		my $node_row = $nodes_tbl->getrowwithvalue(&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES::NAME, $nodenames[$j]);
		$node_name_id_map{$nodenames[$j]} = $node_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES::ID];
		VRTS::Logger::log( 'debug', "NODE_NAME_ID_MAP :: $nodenames[$j]---$node_name_id_map{$nodenames[$j]}");
	}

	my $out = run_clish_command 'network ip addr show';
	return unless ( defined $out );

	VRTS::Logger::log( 'debug', "Updating table @ IPAddress");

	my $hsh = from_json_out($out);
        return 0 if (!defined $hsh);

	if ( $hsh->{header}{status} eq 'SUCCESS' ) {	
		VRTS::Logger::log( 'debug', "CMD SUCCESS.. FOUND..");
		
		for my $item(@{$hsh->{"output"}->{"pips"}}){
            		my @row;
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::IP] = $item->{ip};
			VRTS::Logger::log( 'debug', "IPAddress:: $item->{ip}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::NETMASK] = $item->{netmask};
			VRTS::Logger::log( 'debug', "Netmask:: $item->{netmask}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::NODE_ID] = $node_name_id_map{$item->{node}};
			VRTS::Logger::log( 'debug', "Node:: $node_name_id_map{$item->{node}}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::DEV_NAME] = $item->{device};
			VRTS::Logger::log( 'debug', "Device:: $item->{device}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::TYPE] = "PHYSICAL";
			VRTS::Logger::log( 'debug', "Type:: PHYSICAL");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::STATUS] = $item->{status};
			VRTS::Logger::log( 'debug', "Status:: $item->{status}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::TAG] = $item->{tag};
			VRTS::Logger::log( 'debug', "TAG:: $item->{tag}");
			$table->addrow(\@row);
			VRTS::Logger::log( 'debug', "Row added for physical IP: $item->{ip} --------");
		}

		for my $item(@{$hsh->{"output"}->{"vips"}}){
            		my @row;
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::IP] = $item->{ip};
			VRTS::Logger::log( 'debug', "IPAddress:: $item->{ip}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::NETMASK] = $item->{netmask};
			VRTS::Logger::log( 'debug', "Netmask:: $item->{netmask}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::NODE_ID] = $node_name_id_map{$item->{node}};
			VRTS::Logger::log( 'debug', "Node:: $node_name_id_map{$item->{node}}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::DEV_NAME] = $item->{device};
			VRTS::Logger::log( 'debug', "Device:: $item->{device}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::TYPE] = "VIRTUAL";
			VRTS::Logger::log( 'debug', "Type:: VIRTUAL");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::STATUS] = $item->{status};
			VRTS::Logger::log( 'debug', "Status:: $item->{status}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::TAG] = $item->{tag};
			VRTS::Logger::log( 'debug', "TAG:: $item->{tag}");
			$table->addrow(\@row);
			VRTS::Logger::log( 'debug', "Row added for virtual IP: $item->{ip} --------");
		}

		my $cip = $hsh->{"output"}->{"cip"};
		if ( defined($cip) ){
		#for my $item({$hsh->{"output"}->{"cip"}}){
            		my @row;
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::IP] = $cip->{ip};
			VRTS::Logger::log( 'debug', "IPAddress:: $cip->{ip}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::NETMASK] = $cip->{netmask};
			VRTS::Logger::log( 'debug', "Netmask:: $cip->{netmask}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::NODE_ID] = $node_name_id_map{$cip->{node}};
			VRTS::Logger::log( 'debug', "Node:: $node_name_id_map{$cip->{node}}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::DEV_NAME] = $cip->{device};
			VRTS::Logger::log( 'debug', "Device:: $cip->{device}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::TYPE] = "VIRTUAL";
			VRTS::Logger::log( 'debug', "Type:: VIRTUAL");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::STATUS] = $cip->{status};
			VRTS::Logger::log( 'debug', "Status:: $cip->{status}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::TAG] = "CONSOLE";
			VRTS::Logger::log( 'debug', "TAG:: CONSOLE");
			$table->addrow(\@row);
			VRTS::Logger::log( 'debug', "Row added for Console IP: $cip->{ip}  --------");

			# Adding console information in P_ARRAY_SUBSYSTEM_NODES_PROP table
			# so it can be used in user account to S3 share mapping.
			# This is temparary fix, we need to change it.
			my $table_r = $agent->gettable("P_ARRAY_SUBSYSTEM_NODES_PROP");
			unless($table_r) {
				VRTS::Logger::log( 'debug', "No table P_ARRAY_SUBSYSTEM_NODES_PROP");
				return 0;
			}
		        my @row_r;
        		$row_r[&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES_PROP::NODE_ID] = $node_name_id_map{$cip->{node}};
        		$row_r[&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES_PROP::KEY] = "console";
        		$row_r[&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES_PROP::VALUE] = "Yes";
        		$row_r[&VRTS::Constants::P_ARRAY_SUBSYSTEM_NODES_PROP::TYPE] = "node_property";
        		$table_r->addrow(\@row_r);
		}
	
		my $bip = $hsh->{"output"}->{"bip"};
		if ( defined($bip) ){
		#for my $item({$hsh->{"output"}->{"bip"}}){
            		my @row;
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::IP] = $bip->{ip};
			VRTS::Logger::log( 'debug', "IPAddress:: $bip->{ip}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::NETMASK] = $bip->{netmask};
			VRTS::Logger::log( 'debug', "Netmask:: $bip->{netmask}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::NODE_ID] = $node_name_id_map{$bip->{node}};
			VRTS::Logger::log( 'debug', "Node:: $node_name_id_map{$bip->{node}}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::DEV_NAME] = $bip->{device};
			VRTS::Logger::log( 'debug', "Device:: $bip->{device}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::TYPE] = "VIRTUAL";
			VRTS::Logger::log( 'debug', "Type:: VIRTUAL");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::STATUS] = $bip->{status};
			VRTS::Logger::log( 'debug', "Status:: $bip->{status}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::TAG] = "NETBACKUP";
			VRTS::Logger::log( 'debug', "TAG:: NETBACKUP");
			$table->addrow(\@row);
			VRTS::Logger::log( 'debug', "Row added for Backup IP: $bip->{ip} --------");
		}

		my $rip = $hsh->{"output"}->{"rip"};
		if ( defined($rip) ){
		#for my $item({$hsh->{"output"}->{"rip"}}){
            	my @row;
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::IP] = $rip->{ip};
			VRTS::Logger::log( 'debug', "IPAddress:: $rip->{ip}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::NETMASK] = $rip->{netmask};
			VRTS::Logger::log( 'debug', "Netmask:: $rip->{netmask}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::NODE_ID] = $node_name_id_map{$rip->{node}};
			VRTS::Logger::log( 'debug', "Node:: $node_name_id_map{$rip->{node}}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::DEV_NAME] = $rip->{device};
			VRTS::Logger::log( 'debug', "Device:: $rip->{device}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::TYPE] = "VIRTUAL";
			VRTS::Logger::log( 'debug', "Type:: VIRTUAL");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::STATUS] = $rip->{status};
			VRTS::Logger::log( 'debug', "Status:: $rip->{status}");
			$row[&VRTS::Constants::P_ARRAY_NODE_INTF_ADDR::TAG] = "REPLICATION";
			VRTS::Logger::log( 'debug', "TAG:: REPLICATION");
			$table->addrow(\@row);
			VRTS::Logger::log( 'debug', "Row added for Replication IP: $rip->{ip} --------");
		}

	}
	else {
		VRTS::Logger::log( 'debug', "FAILURE:: Nothing Captured");
	}	
	#$table->print_table();
	return 1;
}

1;
