#
# $Id
#
# $Copyright:
#

package VRTS::SNAS::S08Schedules;

require Exporter;
require VRTS::Util;
require VRTS::Summary;
require VRTS::CMD;
require VRTS::Constants::P_ARRAY_SCHEDULE;

use VRTS::Agent;
use strict;
use vars qw($VERSION @ISA @EXPORT );
use Data::Dumper;
use JSON;
use VRTS::SNAS::SNASUtils;

$VERSION = '1.00';
@ISA     = qw(Exporter);
@EXPORT  = qw(initialize discover);

sub initialize {

    VRTS::Logger::log('debug', "In SNAS Schedule discovery");
    my $agent = shift;
}

sub discover {

    my $agent = shift;

    my $context = $agent->get_context();

    VRTS::Logger::log( 'debug', "Discovering SNAS Schedules");

    my $table = $agent->gettable("P_ARRAY_SCHEDULE");
    unless($table) {
        VRTS::Logger::log( 'debug', "No table P_ARRAY_SCHEDULE");
        return 0;
    }

    #
    # Get the schedule information for storage snapshot
    #

    my $out = run_clish_command 'storage snapshot schedule show';
    return unless ( defined $out );
    VRTS::Logger::log( 'debug', "Updating table @ P_ARRAY_SCHEDULE");

    my $hsh = from_json_out($out);
    return 0 if (!defined $hsh);

    my $encl_id = get_clusteruuid();
    for my $item(@{$hsh->{"output"}->{"schedules"}}){
        my @row;
        $row[&VRTS::Constants::P_ARRAY_SCHEDULE::SUBSYS_ID] = $encl_id; 
        $row[&VRTS::Constants::P_ARRAY_SCHEDULE::ID] = $encl_id."_".$item->{"fs_name"}."_".$item->{"schedule_name"};
        $row[&VRTS::Constants::P_ARRAY_SCHEDULE::NAME] = $item->{"schedule_name"};
        $row[&VRTS::Constants::P_ARRAY_SCHEDULE::TYPE] = "storage";
        $row[&VRTS::Constants::P_ARRAY_SCHEDULE::MINUTE] = $item->{"minute"};
        $row[&VRTS::Constants::P_ARRAY_SCHEDULE::HOUR] = $item->{"hour"};
        $row[&VRTS::Constants::P_ARRAY_SCHEDULE::DAY] = $item->{"day"};
        $row[&VRTS::Constants::P_ARRAY_SCHEDULE::MONTH] = $item->{"month"};
        $row[&VRTS::Constants::P_ARRAY_SCHEDULE::WEEKDAY] = $item->{"weekday"};
        $table->addrow(\@row);
        VRTS::Logger::log( 'debug', "Row added for storage snapshot schedule :  $item->{'schedule_name'}");
    }
    return 1;
}

1;
