#
# $Id
#
# $Copyright:
#

package VRTS::SNAS::S09snapshot;

require Exporter;
require VRTS::Util;
require VRTS::Summary;
require VRTS::CMD;
require VRTS::Constants::P_ARRAY_SCHEDULE;
require VRTS::Constants::P_ARRAY_NAS_FS_SNAPSHOT;

use VRTS::Agent;
use strict;
use Switch;
use vars qw($VERSION @ISA @EXPORT );
use Data::Dumper;
use JSON;
use VRTS::SNAS::SNASUtils;

$VERSION = '1.00';
@ISA     = qw(Exporter);
@EXPORT  = qw(initialize discover);

sub initialize {

    VRTS::Logger::log('debug', "In SNAS Snapshot discovery");
    my $agent = shift;
}

sub discover {

    my $agent = shift;

    VRTS::Logger::log( 'debug', "Discovering SNAS Snapshots");

    my $table = $agent->gettable("P_ARRAY_NAS_FS_SNAPSHOT");
    unless($table) {
        VRTS::Logger::log( 'debug', "No table P_ARRAY_NAS_FS_SNAPSHOT");
        return 0;
    }

    my $sched_table = $agent->gettable("P_ARRAY_SCHEDULE");

    my $out = run_clish_command 'storage snapshot list';
    return unless ( defined $out );

    my $hsh = from_json_out($out);
    return 0 if (!defined $hsh );

    my $enclr_id =  get_clusteruuid();
    my %fs_hsh;
    my @fs_arr;
    if ( $hsh->{header}{status} eq 'SUCCESS' ) {

        #
        # Get list of filesystems for which snapshot exists
        #

        for my $item(@{$hsh->{"output"}->{"snapshots"}}) {
            if(!exists($fs_hsh{$item->{fsname}})) { $fs_hsh{$item->{fsname}} = 1; }
        }
        @fs_arr = keys %fs_hsh;
    } else {
        VRTS::Logger::log( 'debug',"CMD FAILED : storage snapshot list");
        return 0;
    }
    #
    # Get list of snapshots information for filesystem 
    #

    for my $fname (@fs_arr) {
        my $cmd = 'storage snapshot list '.$fname;
        my $snap_cmd_out = run_clish_command "$cmd";
        return unless ( defined $snap_cmd_out );

        VRTS::Logger::log( 'debug', "OUT: $snap_cmd_out");

        my $snap_hsh = from_json_out($snap_cmd_out);
        return 0 if (!defined $snap_hsh );

        if ( $snap_hsh->{header}{status} eq 'SUCCESS' ) {
            for my $snapshot (@{$snap_hsh->{"output"}->{"snapshots"}}){
                my @row;
                $row[&VRTS::Constants::P_ARRAY_NAS_FS_SNAPSHOT::ID] = $fname."_".$snapshot->{"name"};
                $row[&VRTS::Constants::P_ARRAY_NAS_FS_SNAPSHOT::SUBSYS_ID] = $enclr_id;
                $row[&VRTS::Constants::P_ARRAY_NAS_FS_SNAPSHOT::NAME] = $snapshot->{"name"};
                $row[&VRTS::Constants::P_ARRAY_NAS_FS_SNAPSHOT::FS_ID] = $enclr_id."_".$fname;
                $row[&VRTS::Constants::P_ARRAY_NAS_FS_SNAPSHOT::CTIME] = $snapshot->{"ctime"};
                $row[&VRTS::Constants::P_ARRAY_NAS_FS_SNAPSHOT::MTIME] = $snapshot->{"mtime"};
                $row[&VRTS::Constants::P_ARRAY_NAS_FS_SNAPSHOT::STATUS] = $snapshot->{"status"};

                my $flag_str;
                if ($snapshot->{"removable"} eq "yes" ) {
                    $flag_str = "1";
                } else {
                    $flag_str = "0";
                }
                if ($snapshot->{"preserved"} eq "yes" ) {
                    $flag_str = $flag_str."1";
                } else {
                    $flag_str = $flag_str."0";
                }
                $row[&VRTS::Constants::P_ARRAY_NAS_FS_SNAPSHOT::FLAGS] = $flag_str;
                $row[&VRTS::Constants::P_ARRAY_NAS_FS_SNAPSHOT::SIZE] = $snapshot->{"size"};

                if ($snapshot->{"schedule_name"} eq "-") {
                    $row[&VRTS::Constants::P_ARRAY_NAS_FS_SNAPSHOT::SCHEDULE_ID] = "-";
                } else {
                    my $sched_row = $sched_table->getrowwithvalue(&VRTS::Constants::P_ARRAY_SCHEDULE::NAME, $snapshot->{"schedule_name"});
                    my $sname = $snapshot->{"schedule_name"};
                    my $sched_id = $sched_row->[&VRTS::Constants::P_ARRAY_SCHEDULE::ID];
                    $row[&VRTS::Constants::P_ARRAY_NAS_FS_SNAPSHOT::SCHEDULE_ID] = $sched_id;
                }
                $table->addrow(\@row);
                VRTS::Logger::log( 'debug', "Row added for Snapshot  $snapshot->{'name'}");
            }
        } else {
            VRTS::Logger::log( 'debug',"CMD FAILED : $cmd");
            return 0;
        }
    }
    return 1;
}

1;
