#
# $Id
#
# $Copyright:
#

package VRTS::SNAS::S10UserAccount;

require Exporter;
require VRTS::Util;
require VRTS::Summary;
require VRTS::CMD;
require VRTS::Constants::P_ARRAY_USER_ACCOUNT;

use VRTS::Agent;
use strict;
use vars qw($VERSION @ISA @EXPORT );
use Data::Dumper;
use JSON;
use VRTS::SNAS::SNASUtils;

$VERSION = '1.00';
@ISA     = qw(Exporter);
@EXPORT  = qw(initialize discover);

sub initialize {

    VRTS::Logger::log('debug', "In ISA User Account discovery");
    my $agent = shift;
}

sub discover {

    my $agent = shift;

    my $context = $agent->get_context();

    VRTS::Logger::log( 'debug', "Discovering S3 User Account information.");

    my $table = $agent->gettable("P_ARRAY_USER_ACCOUNT");
    unless($table) {
        VRTS::Logger::log( 'debug', "No table P_ARRAY_USER_ACCOUNT");
        return 0;
    }

    #
    # Get S3 User Account information.
    #
    my $out = run_clish_command 'objectaccess account show';
    return unless ( defined $out );
       
    my $hsh_acc = from_json_out($out);
    return 0 if (!defined $hsh_acc );

    my $encl_id = get_clusteruuid();
    my $sharetype = "S3";
    if ($hsh_acc->{"header"}->{"status"} eq "SUCCESS"){
        for my $item(@{$hsh_acc->{"output"}}){ 
            my @row;
            $row[&VRTS::Constants::P_ARRAY_USER_ACCOUNT::SUBSYS_ID] = $encl_id; 
            $row[&VRTS::Constants::P_ARRAY_USER_ACCOUNT::ID] = $encl_id."_".$item->{name};
            $row[&VRTS::Constants::P_ARRAY_USER_ACCOUNT::NAME] = $item->{name};
            $row[&VRTS::Constants::P_ARRAY_USER_ACCOUNT::SHARE_ID] = $encl_id."_".$sharetype."_".$item->{fsname};
            $row[&VRTS::Constants::P_ARRAY_USER_ACCOUNT::ACCESS_KEY] = $item->{accesskey};
            $row[&VRTS::Constants::P_ARRAY_USER_ACCOUNT::SECRET_KEY] = $item->{secretkey};
            $row[&VRTS::Constants::P_ARRAY_USER_ACCOUNT::TYPE] = $sharetype;
            $table->addrow(\@row);
            VRTS::Logger::log( 'debug', "Row added for S3 user account :  $item->{name}");
        }
        return 1;
    } else  {
        VRTS::Logger::log( 'debug', "No table P_ARRAY_USER_ACCOUNT");
        return 0;
    }
}

1;
