#
# $Id: S30Capacities.pm, 2016/03/02 21:58:51, venkat $
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#

package VRTS::SNAS::S30Capacities;

require Exporter;
require VRTS::Util;
require VRTS::Summary;
require VRTS::CMD;
require VRTS::Constants::P_ARRAY_CAPACITY;

use VRTS::Agent;
use strict;
use vars qw($VERSION @ISA @EXPORT );
use Data::Dumper;
use JSON;

$VERSION = '1.00';
@ISA     = qw(Exporter);
@EXPORT  = qw(initialize discover);

my ($array_tb, $pdev_tb, $ldev_tb, $ldev_map_tb, $dg_tb, $rg_tb, $fs_tb);

my %capacities_col_map = ("physical_size" => {"index" => 1, "prop_name" => "PHYSICAL_SIZE"},
     "allocated_size" => {"index" => 2, "prop_name" => "ALLOCATED_SIZE"},
     "spare_size" => {"index" => 3, "prop_name" => "SPARE_SIZE"},
     "unallocated_size" => {"index" => 4, "prop_name" => "UNALLOCATED_SIZE"},
     "rg_total_size" => {"index" => 5, "prop_name" => "RG_TOTAL_SIZE"},
     "rg_addressable_size" => {"index" => 6, "prop_name" => "RG_ADDRESSABLE_SIZE"},
     "rg_overhead_size" => {"index" => 7, "prop_name" => "RG_OVERHEAD_SIZE"},
     "rg_free_size" => {"index" => 8, "prop_name" => "RG_FREE_SIZE"},
     "logical_total_size" => {"index" => 9, "prop_name" => "LOGICAL_TOTAL_SIZE"},
     "logical_configured_size" => {"index" => 10, "prop_name" => "LOGICAL_CONFIGURED_SIZE"},
     "logical_assigned_size" => {"index" => 11, "prop_name" => "LOGICAL_ASSIGNED_SIZE"},
     "logical_unassigned_size" => {"index" => 12, "prop_name" => "LOGICAL_UNASSIGNED_SIZE"},
     "logical_admin_size" => {"index" => 13, "prop_name" => "LOGICAL_ADMIN_SIZE"},
     "logical_reptarget_size" => {"index" => 14, "prop_name" => "LOGICAL_REPTARGET_SIZE"},
     "subs_logical_total_size" => {"index" => 15, "prop_name" => "SUBS_LOGICAL_TOTAL_SIZE"},
     "subs_logical_configured_size" => {"index" => 16, "prop_name" => "SUBS_LOGICAL_CONFIGURED_SIZE"},
     "subs_logical_assigned_size" => {"index" => 17, "prop_name" => "SUBS_LOGICAL_ASSIGNED_SIZE"},
     "subs_logical_unassigned_size" => {"index" => 18, "prop_name" => "SUBS_LOGICAL_UNASSIGNED_SIZE"},
     "subs_logical_admin_size" => {"index" => 19, "prop_name" => "SUBS_LOGICAL_ADMIN_SIZE"},
     "subs_logical_reptarget_size" => {"index" => 20, "prop_name" => "SUBS_LOGICAL_REPTARGET_SIZE"},
     "ext_physical" => {"index" => 34, "prop_name" => "EXT_PHYSICAL"},
     "rg_nas_file_capacity" => {"index" => 48, "prop_name" => "RG_FILE_CAPACITY"},
     "rg_block_capacity" => {"index" => 49, "prop_name" => "RG_BLOCK_CAPACITY"},
     "logical_nas_filesystem_free" => {"index" => 50, "prop_name" => "LOGICAL_NAS_FILESYSTEM_FREE"},
     "logical_nas_used" => {"index" => 51, "prop_name" => "LOGICAL_NAS_USED"},
     "array_vol_capacity" => {"index" => 56, "prop_name" => "ARRAY_VOL_CAPACITY"},
     "array_vol_allocated" => {"index" => 57, "prop_name" => "ARRAY_VOL_ALLOCATED"},
     "array_vol_available" => {"index" => 58, "prop_name" => "ARRAY_VOL_AVAILABLE"},
     "volume_snap_reserve_cap" => {"index" => 59, "prop_name" => "VOLUME_SNAP_RESV_CAP"},
     "volume_overhead" => {"index" => 60, "prop_name" => "VOLUME_OVERHEAD"},
     "overwrite_reserve_logical" => {"index" => 61, "prop_name" => "VOLUME_OVERWRITE_RESERVE"},
     "physical_overhead_size" => {"index" => 71, "prop_name" => "PHYSICAL_OVERHEAD_SIZE"}
);

my %capacities_hash = ("physical_size" => 0,
                  "allocated_size" => 0,
                  "spare_size" => 0,
                  "unallocated_size" => 0,
                  "rg_total_size" => 0,
                  "rg_addressable_size" => 0,
                  "rg_overhead_size" => 0,
                  "rg_free_size" => 0,
                  "logical_total_size" => 0,
                  "logical_configured_size" => 0,
                  "logical_assigned_size" => 0,
                  "logical_unassigned_size" => 0,
                  "logical_admin_size" => 0,
                  "logical_reptarget_size" => 0,
                  "subs_logical_total_size" => 0,
                  "subs_logical_configured_size" => 0,
                  "subs_logical_assigned_size" => 0,
                  "subs_logical_unassigned_size" => 0,
                  "subs_logical_admin_size" => 0,
                  "ext_physical" => 0,
                  "subs_logical_reptarget_size" => 0);


sub initialize {

    my $agent = shift;
}

sub _discover_physical_capacities
{
    my $agent = shift;

    $capacities_hash{allocated_size} = 0;
    $capacities_hash{physical_size} = 0;
    $capacities_hash{available_size} = 0;
    $capacities_hash{spare_size} = 0;
    $capacities_hash{unallocated_size} = 0;

    my $pdev_rows = $pdev_tb->each_current();
    foreach my $pdev_elem (@{$pdev_rows})
    {
        if ($pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::STATUS] == 0)
        {
            $capacities_hash{physical_size} +=
                $pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SIZE];

            $capacities_hash{spare_size} +=
                $pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SPARE_SIZE];

            $capacities_hash{available_size} += $pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::AVAILABLE];

            # For every pdev, which is part of atleast one RG (raid|array group), account it into allocated size.
            my @tmp_cols = (&VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::DEVICE_ID,
                                    &VRTS::Constants::P_ARRAY_DEVICEGROUP_MAP::DEVICE_TYPE);
            my @tmp_vals = ($pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::ID], 'PDEV');

            my $dg_rows = $dg_tb->getallrowswithvaluez(\@tmp_cols, \@tmp_vals);
    
            if (defined $dg_rows and scalar(@{$dg_rows}) >= 1)
            {
                if ($pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SPARE_SIZE] != 0 and
                         ($pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SIZE] !=
                               $pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SPARE_SIZE]))
                {
                    $capacities_hash{allocated_size} +=
                         ($pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SIZE] - 
                                         $pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SPARE_SIZE]);
                }
                else
                {
                    $capacities_hash{allocated_size} += $pdev_elem->[1]->[&VRTS::Constants::P_ARRAY_PDEV::SIZE];
                }
            }
        }
    }

    if ($capacities_hash{allocated_size} != 0)
    {
        $capacities_hash{unallocated_size} =
             ($capacities_hash{physical_size} - ($capacities_hash{allocated_size} + $capacities_hash{spare_size}));
    }
    else
    {
        if ($capacities_hash{available_size} > 0)
        {
            $capacities_hash{unallocated_size} = $capacities_hash{available_size};

            $capacities_hash{allocated_size} = ($capacities_hash{physical_size} - $capacities_hash{spare_size} -
                                               $capacities_hash{unallocated_size});
        }
        else
        {
            $capacities_hash{unallocated_size} = $capacities_hash{physical_size} - $capacities_hash{spare_size};
        }
    }
}


sub _discover_sp_capacity
{
    my $agent = shift;

    my $rg_rows = $rg_tb->getallrowswithvalue(&VRTS::Constants::P_ARRAY_DEVICE_GROUP::TYPE, 'RG');
    next if (not defined $rg_rows);

    foreach my $rg_r (@{$rg_rows})
    {
        $capacities_hash{rg_total_size} += $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::RAW_SIZE];
        $capacities_hash{rg_overhead_size} += $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::OVERHEAD_SIZE];
        $capacities_hash{rg_addressable_size} += $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::DEVICE_SIZE];
        $capacities_hash{rg_free_size} += $rg_r->[&VRTS::Constants::P_ARRAY_DEVICE_GROUP::FREE_SIZE];
    }

    return 0;
}


sub report_all_capacities
{
    my $agent = shift;
    my $cap_tb = shift;

    my $context = $agent->get_context();
    my $e_vdid = $context->{"enclosure_vdid"};

    my $array_row = $array_tb->getrowwithvalue(&VRTS::Constants::P_ARRAY_SUBSYSTEM::ID, $e_vdid);

    return if (not defined $e_vdid or $e_vdid eq '');

    # Defaulting all capacities to zero first for starters.
    while(my($key, $value) = each (%capacities_col_map))
    {
        my @cap_row;

        $cap_row[&VRTS::Constants::P_ARRAY_CAPACITY::ARRAY_ID] = $e_vdid;
        $cap_row[&VRTS::Constants::P_ARRAY_CAPACITY::ARRAY_PROP_ID] = $value->{index};
        $cap_row[&VRTS::Constants::P_ARRAY_CAPACITY::VALUE] = 0;

        $cap_tb->addrow(\@cap_row);
    }

    # Now it is time to report the capacities into the
    while(my($key, $value) = each (%capacities_hash))
    {
        my @cap_row;

        next if (not defined $capacities_col_map{$key});

        $cap_row[&VRTS::Constants::P_ARRAY_CAPACITY::ARRAY_ID] = $e_vdid;
        $cap_row[&VRTS::Constants::P_ARRAY_CAPACITY::ARRAY_PROP_ID] = $capacities_col_map{$key}->{index};
        $cap_row[&VRTS::Constants::P_ARRAY_CAPACITY::VALUE] = $value;

        $cap_tb->addrow(\@cap_row);
    }

    #$cap_tb->print_table();

    # Set the total storage on the enclosure
    $array_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::PHYSICAL_SIZE] = $capacities_hash{physical_size};
    $array_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::ALLOCATED_SIZE] = $capacities_hash{allocated_size};
    $array_row->[&VRTS::Constants::P_ARRAY_SUBSYSTEM::RAID_OVERHEAD] = 0; # As there is no RAID concept, all is RAID0

    $array_tb->addrow($array_row);
}

sub _discover_nas_capacity
{
    my $agent = shift;

    $capacities_hash{"rg_nas_file_capacity"} = 0;
    $capacities_hash{"logical_nas_used"} = 0;
    $capacities_hash{"logical_nas_filesystem_free"} = 0;

    my $fs_rows = $fs_tb->each_current();
    foreach my $fs_elem (@{$fs_rows})
    {
        $capacities_hash{"rg_nas_file_capacity"} += 
                 $fs_elem->[1]->[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::FILE_STORAGE_CAPACITY];
        
        $capacities_hash{"logical_nas_used"} += 
                 $fs_elem->[1]->[&VRTS::Constants::P_ARRAY_NAS_FILESYSTEM::FILE_STORAGE_USED];
    }

    $capacities_hash{"logical_nas_filesystem_free"} = ($capacities_hash{"rg_nas_file_capacity"} - 
                                                           $capacities_hash{"logical_nas_used"});
}

sub discover {

    my $agent = shift;

    my $context = $agent->get_context();

    VRTS::Logger::log( 'debug', "Discovering Capacities for SNAS enclosure");

    my $cap_tb = $agent->gettable("P_ARRAY_CAPACITY");

    $array_tb = $agent->gettable("P_ARRAY_SUBSYSTEM");
    $pdev_tb = $agent->gettable("P_ARRAY_PDEV");
    $ldev_tb = $agent->gettable("P_ARRAY_LDEV");
    $ldev_map_tb = $agent->gettable("P_ARRAY_LDEV_MAP");
    $dg_tb = $agent->gettable("P_ARRAY_DEVICEGROUP_MAP");
    $rg_tb = $agent->gettable("P_ARRAY_DEVICE_GROUP");
    $fs_tb = $agent->gettable("P_ARRAY_NAS_FILESYSTEM");

    _discover_physical_capacities($agent);

    _discover_sp_capacity($agent);

    _discover_nas_capacity($agent);

    report_all_capacities($agent, $cap_tb);

    #$cap_tb->print_table();

    return 0;
}

1;
