#
# $Id:
#
#

package VRTS::SNAS::SNASUtils;

use strict;
use Switch;
use vars qw($VERSION @ISA @EXPORT);

require Exporter;
require VRTS::CMD;

@ISA = qw(Exporter);
@EXPORT  = qw(get_clusteruuid get_unique_id generate_uuid convert_size calculate_available get_storage_used from_json_out run_clish_command);

sub initialize {
    VRTS::Logger::log('debug', "SNAS Helper initialize. This itself does not do any discovery");
    my $agent = shift;
}

sub discover {
    VRTS::Logger::log('debug', "SNAS Helper discover. This itself does not do any discovery");
    return 1;
}

sub get_clusteruuid {
    my $sfmh_haclus =   &VRTS::CMD::SFMH_HACLUS;
    my $devnull = &VRTS::CMD::DEVNULL;
    my $clus_id; 

    open(H, "$sfmh_haclus -display 2>$devnull |" ) or
        VRTS::Logger::log( 'error', 'cannot open command'.$sfmh_haclus);
    while(<H>)
    {
        if (/^CID/)
        {
            # take the second word
            s/\s*CID\s*//;
            s/^\s*//;
            s/\s*$//;
            $clus_id = $_;
            if ($clus_id =~ /^$/)
            {
                VRTS::Logger::log('error', "CID not found. This is not expected");
                $clus_id = "-";
            }
            else
            {
                VRTS::Logger::log('debug', "CID found. " . $clus_id);
            }
        }
    }
    close(H) or
       VRTS::Logger::log('error', 'cannot close command haclus');
    return $clus_id;
}


sub get_unique_id
{
    my $id = "";
    my $len = 16;

    for(my $i=0 ; $i< $len ;)
    {
        my $j = chr(int(rand(127)));
        if($j =~ /[a-zA-Z0-9]/)
        {
            $id .=$j;
            $i++;
        }
    }
    return $id;
}

sub generate_uuid()
{
    my $error = undef;
    my $sfmh_haclus =   &VRTS::CMD::SFMH_HACLUS;
    my $devnull = &VRTS::CMD::DEVNULL;
    if(!-f $sfmh_haclus)
    {
        VRTS::Logger::log('error', "$sfmh_haclus not found.");
    }

    my $id = get_clusteruuid();
    if ($id ne 0) {
        VRTS::Logger::log( 'debug', "Old ClusterUUID present. Will not generate new.");
        return 0, $id;
    } 
    $id = get_unique_id();

    # Use VCS haclus binary if VCS version is greater than 5.1

    if (defined $id and $id ne "")
    {
        # Setting cluster UUID as it is not defined..
        $ENV{ALLOW_CLUSTERUUID_MOD} = "true";
        my $cmd = $sfmh_haclus. " -modify ClusterUUID $id 2>$devnull";
        VRTS::Logger::log( 'debug', "Generating ClusterUUID using cmd " . $cmd);
        my $ret = system ("$cmd");
        VRTS::Logger::log( 'debug', "New ClusterUUID is " . $id);
        return $ret, $id;
    }
    return 0, undef;
}

sub convert_percent {
    my $s = shift;
    if ( $s eq '-' ) {
        return 0;
    }
    $s =~ s/%$//;
    return $s;
}

sub convert_size {
    my $size = shift;
    my $power = chop($size);
    switch ($power) {
        case "K"      { return $size * 1024 }
        case "M"      { return $size * 1024 * 1024 }
        case "G"      { return $size * 1024 * 1024 * 1024 }
        case "T"      { return $size * 1024 * 1024 * 1024 * 1024 }

    }
}

sub calculate_available {
    my $total = shift;
    my $usage = shift;
    my $available = convert_size( $total ) - (convert_size( $total ) * convert_percent( $usage )) / 100; 
    return $available;
}

sub get_storage_used {
    my ($used,$size) = @_; #5%
    if ( $used eq '-' ) {
        return 0;
    }
    $used =~ s/%$//;
    my $convert_size_kb = convert_size($size);
    my $used_size_kb = ($used/100) * $convert_size_kb;
    return $used_size_kb;
}

sub from_json_out{
    my ($out )=@_;
    my $json = new JSON();
    my $res=undef;
    eval {
    	$res  = $json->decode($out);
    };
    if ($@) {
    	VRTS::Logger::log( 'debug', "Unable to parse CLISH output $out.\n" .$@); 
    }
    return $res;
}

sub run_clish_command{
    my ($short_cmd) = @_;
    my $full_cmd = 'bash -c "source /var/opt/VRTSsfmh/spool/addons/store/VRTSsfmsnas-1.0.0.0/be/cs/utils/unset.sh; source /var/opt/VRTSsfmh/spool/addons/store/VRTSsfmsnas-1.0.0.0/be/cs/utils/myenv.sh; export NAS_OUTPUT=json; /opt/SYMCsnas/clish/bin/clish  -u master -c \'' . $short_cmd . '\' 2>/dev/null"';
    my $out = `$full_cmd`;
    VRTS::Logger::log( 'debug', "CMD: $full_cmd");
    VRTS::Logger::log( 'debug', "OUT: $out");

    return $out;
}

1;
