#!/opt/VRTSsfmh/bin/perl 
# $Id: verify_user.pl, 2016/03/18 21:19:10, venkat $
#
# $Copyright: Copyright (c) 2016 Veritas Technologies LLC.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# VERITAS TECHNOLOGIES LLC.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF VERITAS TECHNOLOGIES LLC.
#
# The  Licensed  Software  and  Documentation  are  deemed  to be commercial
# computer  software  as  defined  in  FAR  12.212 and subject to restricted
# rights  as  defined in FAR Section 52.227-19 "Commercial Computer Software
# - Restricted  Rights"  and  DFARS 227.7202,  et seq.  "Commercial Computer
# Software  and  Commercial Computer Software Documentation," as applicable,
# and any successor regulations, whether delivered by Veritas as on premises
# or  hosted  services.  Any  use,  modification,  reproduction  release,
# performance,  display  or  disclosure  of  the  Licensed  Software  and
# Documentation by the U.S. Government shall be solely in accordance with
# the terms of this Agreement. $
#

#
# This script calls clish and returns value depending upon SUCCESS (0)
# or ERROR (1)
#

# Only supported on Unix platforms
if  ($^O =~ m/Win32/i)
{
    printf ("Please run this utility on UNIX platform.\n"); 
    exit(1);
}
BEGIN {push @INC , '/opt/VRTSsfmh/lib/modules';}

use strict;
use Getopt::Long;
use JSON;
use POSIX;
use Fcntl qw(:flock SEEK_END);

require VRTS::Logger;
use VRTS::Schedule;
use VRTS::Canonical;
use VRTS::Util;
use VRTS::OS;
use VRTS::Unix;

#
# Help routines
#

sub _show_help_msg
{
    printf("verify_user.pl -u <user_name>\n");	
    exit(1);
}


#
# Main entry point
#
sub main
{
    my ($user, $r);
    if (scalar @ARGV < 1)
    {
        _show_help_msg();
    }
    # get args
    $r = GetOptions( "user|u:s" => \$user);
    if (not defined $user)
    {
        _show_help_msg();
    }
    my $cmd = '/usr/bin/id '.$user;
    my @out =  qx/$cmd 2>&1 /;
    $r = $?;
    $r = $r >> 8 unless ($r == -1);

out:
    my $res={};
    my $message={};
    my $js = new JSON;
    if ($r)
    {
	$message->{message}= "Failed to verify the user $user.";
	$res->{output} = $message;
	printf($js->objToJson($res));
        exit(1);
    }else{
        $message->{message}= "User $user exists";
	$res->{output} = $message;
	printf($js->objToJson($res));
        exit(0);
    }
}
main();
